@@ -429,7 +429,7 @@ TStateModel* CreateStateModel_VAR_matlab(char *filename)
 	  }
 	break;
       case 4:
-	printf("Case %d not implimented.\n",4);
+	printf("Case %d not implemented.\n",4);
 	dw_exit(0);
       default:
 	printf("Unknown type.\n");
