@@ -32,6 +32,9 @@
 #include <string.h>
 #include <math.h>
 
+// define to use ECB normalization versions
+//#define ECB_VERSION
+
 static int GetNumberDraws(FILE *f)
 {
   int count=0, n;
@@ -289,6 +292,15 @@ TVector** impulse_response_percentile_ergodic_full(int draws, FILE *posterior_fi
 	done=1;
       else
 	{
+#ifdef OCTAVE_MEX_FILE
+      OCTAVE_QUIT;
+#elif defined(MATLAB_MEX_FILE)
+      if (utIsInterruptPending())
+        {
+          mexPrintf("\n\n** Ctrl-C detected, quitting...\n");
+          dw_exit(0);
+        }
+#endif
 	  dw_NextLine(posterior_file,thin-1);
 	  if ((k >= K) || !GetPosteriorDraw(posterior_file,model,flag))
 	    {
@@ -562,6 +574,7 @@ TVector** impulse_response_percentile_regime_full(FILE *posterior_file, int npos
   return IR;
 }
 
+#ifdef ECB_VERSION
 /*
    Normalization for ECB.  Works for three cases.
      (1) One Markov process controlling coefficients
@@ -882,7 +895,9 @@ TVector** impulse_response_percentile_regime_full_normalized(FILE *posterior_fil
 	  GetIRMatrices(model->sv->nstates,A0_Xi,B,p);
 
 	  // Get Normalization
+#ifdef ECB_VERSION
 	  sbvar_normalize_regimes_ecb(N,model);
+#endif
 	  for (i=0; i < horizon; i++) S[i]=N[s];
 
 	  // all the same?
@@ -944,6 +959,7 @@ TVector** impulse_response_percentile_regime_full_normalized(FILE *posterior_fil
 
   return IR;
 }
+#endif  // ECB_VERSION
 
 
 /*
@@ -1251,7 +1267,9 @@ void dw_sbvar_impulse_response_command_line(int nargs, char **args)
 	  if (dw_FindArgument_String(nargs,args,"regimes") != -1)
 	    for (s=0; s < p->nstates; s++)
 	      {
+#ifdef ECB_VERSION
 		sbvar_normalize_regimes_ecb_set_globals(nargs,args,model);
+#endif
 		if (posterior_file) rewind(posterior_file);
 		// s must not contain more than 10 digits 
 		sprintf(buffer=(char*)dw_malloc(strlen(fmt) + 10),fmt,s);
@@ -1261,9 +1279,11 @@ void dw_sbvar_impulse_response_command_line(int nargs, char **args)
 		  {		
 		    printf("Constructing percentiles for impulse responses - regime %d\n",s);
 		    //if (histogram=impulse_response_percentile_regime(posterior_file,sample,0.01,thin,s,horizon,model,type))      //
-		    //if (IR=impulse_response_percentile_regime_full(posterior_file,nposterior_draws,thin,s,horizon,model,type))   //
-
+#ifdef ECB_VERSION
 		    if (IR=impulse_response_percentile_regime_full_normalized(posterior_file,nposterior_draws,thin,s,horizon,explosive_cutoff,model,type))
+#else
+		    if (IR=impulse_response_percentile_regime_full(posterior_file,nposterior_draws,thin,s,horizon,model,type))
+#endif
 		      {
 			SortIR(IR);                                                                                              //
 			for (i=0; i < DimV(percentiles); i++)
@@ -1285,7 +1305,9 @@ void dw_sbvar_impulse_response_command_line(int nargs, char **args)
 	    if (dw_FindArgument_String(nargs,args,"regime") != -1)
               if (((s=dw_ParseInteger_String(nargs,args,"regime",-1)) >= 0) && (s < p->nstates))
 		{
+#ifdef ECB_VERSION
 		  sbvar_normalize_regimes_ecb_set_globals(nargs,args,model);
+#endif
 		  if (posterior_file) rewind(posterior_file);
 		  // s must not contain more than 10 digits 
 		  sprintf(buffer=(char*)dw_malloc(strlen(fmt) + 10),fmt,s);
@@ -1295,8 +1317,11 @@ void dw_sbvar_impulse_response_command_line(int nargs, char **args)
 		    {		
 		      printf("Constructing percentiles for impulse responses - regime %d\n",s);
 		      //if (histogram=impulse_response_percentile_regime(posterior_file,thin,s,horizon,model,type))                  //
-		      //if (IR=impulse_response_percentile_regime_full(posterior_file,nposterior_draws,thin,s,horizon,model,type))   //
+#ifdef ECB_VERSION
 		      if (IR=impulse_response_percentile_regime_full_normalized(posterior_file,nposterior_draws,thin,s,horizon,explosive_cutoff,model,type))
+#else
+		      if (IR=impulse_response_percentile_regime_full(posterior_file,nposterior_draws,thin,s,horizon,model,type))
+#endif
 			{
 			  SortIR(IR);                                                                                              //
 			  for (i=0; i < DimV(percentiles); i++)
