@@ -2054,7 +2054,7 @@ TVector ProbabilitiesState(TVector p, int s, TStateModel *model)
 */
 PRECISION LogProbabilityStates(int *S, TStateModel *model)
 {
-  printf("LogProbabilityStates() not implimented\n");
+  printf("LogProbabilityStates() not implemented\n");
   dw_exit(0);
 
 /**
