@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-# Copyright © 2019-2020 Dynare Team
+# Copyright © 2019-2021 Dynare Team
 #
 # This file is part of Dynare.
 #
@@ -15,15 +15,18 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+# along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 set -ex
 
 ROOTDIR=$(pwd)/..
 
+# Set the GCC version
+GCC_VERSION=11
+
 # Set the compilers
-CC=gcc-10
-CXX=g++-10
+CC=gcc-$GCC_VERSION
+CXX=g++-$GCC_VERSION
 
 # Set the number of threads
 NTHREADS=$(nproc)
@@ -49,31 +52,50 @@ if [[ -z $VERSION ]]; then
     fi
 fi
 
-# Install location must be truncated for installation of `gcc`
-# If it's too long, the headers of the compiled libraries cannot be modified
-# obliging recompilation on the user's system. Truncate to 5 characters
-# To allow for distribution version to appear
-LOCATION=$(echo "$VERSION" | cut -f1 -d"-" | cut -c 1-5)
-if [[ "$VERSION" == *-unstable* || "$VERSION" == [a-zA-Z]* ]]; then
-    LOCATION="$LOCATION"-"$DATE"
+# Install location must not be too long for gcc.
+# Otherwise, the headers of the compiled libraries cannot be modified
+# obliging recompilation on the user's system.
+# If VERSION is not a official release number, then do some magic
+# to get something not too long, still more or less unique.
+if [[ "$VERSION" =~ ^[0-9\.]+$ ]]; then
+    LOCATION=$VERSION
+else
+    # Get the first component, truncate it to 5 characters, and add the date
+    LOCATION=$(echo "$VERSION" | cut -f1 -d"-" | cut -c 1-5)-"$DATE"
 fi
 
+## Hack for statically linking libquadmath, similar to the one used in
+## deps/Makefile for several libraries (there is no -static-libquadmath flag,
+## see https://gcc.gnu.org/bugzilla/show_bug.cgi?id=46539).
+##
+## NB: The hack done for Windows (see m4/ax_mexopts.m4) does not work for two reasons:
+## - the macOS linker is different from GNU ld and does not have the equivalent of -Bstatic/-Bdynamic
+## - libgfortran.spec does not include --as-needed on macOS, hence it will link the library anyways
+## Also, it does not seem possible to override libgfortran.spec with the --specs option.
+QUADMATH_DIR=$(mktemp -d)
+ln -s /usr/local/opt/gcc/lib/gcc/$GCC_VERSION/libquadmath.a $QUADMATH_DIR
 
 ##
 ## Compile Dynare doc, dynare++, preprocessor, mex for MATLAB < 2018a
 ##
+## NB: In Homebrew, -static-libgfortran is implied by -static-libgcc (see “gfortran -dumpspecs”)
+## NB2: We use the hack for libquadmath in LDFLAGS
 cd "$ROOTDIR"
 [[ -f configure ]] || autoreconf -si
-CC=$CC CXX=$CXX ./configure \
+./configure \
   PACKAGE_VERSION="$VERSION" \
   PACKAGE_STRING="dynare $VERSION" \
-  CXXFLAGS=-I/usr/local/include \
-  LDFLAGS=-static-libgcc \
+  CC=$CC \
+  CXX=$CXX \
+  CPPFLAGS=-I/usr/local/include \
+  LDFLAGS="-static-libgcc -L$QUADMATH_DIR" \
+  LEX=/usr/local/opt/flex/bin/flex \
+  YACC=/usr/local/opt/bison/bin/bison \
   --with-gsl="$LIB64"/gsl \
   --with-matio="$LIB64"/matio \
   --with-slicot="$LIB64"/Slicot/with-underscore \
   --disable-octave \
-  --with-matlab=/Applications/MATLAB_R2016b.app MATLAB_VERSION=R2016b
+  --with-matlab=/Applications/MATLAB_R2016b.app
 if [[ -z $CI ]]; then
     # If not in Gitlab CI, clean the source and build the doc
     make clean
@@ -88,9 +110,9 @@ make -j"$NTHREADS"
 NAME=dynare-"$VERSION"
 PKGFILES="$ROOTDIR"/macOS/pkg/"$NAME"
 mkdir -p \
-      "$PKGFILES"/mex/matlab/maci64-7.9-9.3 \
-      "$PKGFILES"/mex/matlab/maci64-9.4-9.9 \
-      "$PKGFILES"/mex/octave \
+      "$PKGFILES"/preprocessor \
+      "$PKGFILES"/mex/matlab/maci64-8.3-9.3 \
+      "$PKGFILES"/mex/matlab/maci64-9.4-9.11 \
       "$PKGFILES"/doc/dynare++ \
       "$PKGFILES"/dynare++ \
       "$PKGFILES"/scripts \
@@ -107,7 +129,13 @@ cp -p  "$ROOTDIR"/license.txt                                        "$PKGFILES"
 cp -pr "$ROOTDIR"/matlab                                             "$PKGFILES"
 cp -pr "$ROOTDIR"/examples                                           "$PKGFILES"
 
-cp -L  "$ROOTDIR"/mex/matlab/*                                       "$PKGFILES"/mex/matlab/maci64-7.9-9.3
+cp -p  "$ROOTDIR"/preprocessor/src/dynare-preprocessor               "$PKGFILES"/preprocessor
+
+# Recreate backward-compatibility symlink
+rm -f "$ROOTDIR"/matlab/preprocessor64/dynare_m
+ln -sf ../../preprocessor/dynare-preprocessor                        "$PKGFILES"/matlab/preprocessor64/dynare_m
+
+cp -L  "$ROOTDIR"/mex/matlab/*                                       "$PKGFILES"/mex/matlab/maci64-8.3-9.3
 
 cp -p  "$ROOTDIR"/scripts/dynare.el                                  "$PKGFILES"/scripts
 cp -pr "$ROOTDIR"/contrib/ms-sbvar/TZcode/MatlabFiles                "$PKGFILES"/contrib/ms-sbvar/TZcode
@@ -136,37 +164,19 @@ cp -p  "$ROOTDIR"/macOS/deps/lib64/x13as/x13as                       "$PKGFILES"
 ##
 cd "$ROOTDIR"/mex/build/matlab
 make clean
-CC=$CC CXX=$CXX ./configure \
+./configure \
   PACKAGE_VERSION="$VERSION" \
   PACKAGE_STRING="dynare $VERSION" \
-  CXXFLAGS=-I/usr/local/include \
-  LDFLAGS=-static-libgcc \
+  CC=$CC \
+  CXX=$CXX \
+  CPPFLAGS=-I/usr/local/include \
+  LDFLAGS="-static-libgcc -L$QUADMATH_DIR" \
   --with-gsl="$LIB64"/gsl \
   --with-matio="$LIB64"/matio \
   --with-slicot="$LIB64"/Slicot/with-underscore \
-  --with-matlab=/Applications/MATLAB_R2019b.app MATLAB_VERSION=R2019b
+  --with-matlab=/Applications/MATLAB_R2019b.app
 make -j"$NTHREADS"
-cp -L  "$ROOTDIR"/mex/matlab/*                                       "$PKGFILES"/mex/matlab/maci64-9.4-9.9
-
-
-##
-## Create mex for Octave
-##
-cd "$ROOTDIR"/mex/build/octave
-OCTAVE_VERSION=$(grep OCTAVE_VERSION "$ROOTDIR"/macOS/deps/versions.mk | cut -d'=' -f2 | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
-OCTAVE_USR_DIR="/Applications/Octave-$OCTAVE_VERSION.app/Contents/Resources/usr"
-OCTAVE_BIN_DIR="$OCTAVE_USR_DIR/Cellar/octave-octave-app@$OCTAVE_VERSION/$OCTAVE_VERSION/bin"
-PATH="$OCTAVE_BIN_DIR:$PATH" CC=$CC CXX=$CXX ./configure \
-  PACKAGE_VERSION="$VERSION" \
-  PACKAGE_STRING="dynare $VERSION" \
-  CXXFLAGS=-I/usr/local/include \
-  LDFLAGS="-static-libgcc -L$OCTAVE_USR_DIR/lib " \
-  --with-gsl="$LIB64"/gsl \
-  --with-matio="$LIB64"/matio \
-  --with-slicot="$LIB64"/Slicot/with-underscore
-PATH="$OCTAVE_BIN_DIR:$PATH" make -j"$NTHREADS"
-cp -L  "$ROOTDIR"/mex/octave/*                                       "$PKGFILES"/mex/octave
-echo -e "function v = supported_octave_version\nv=\"$OCTAVE_VERSION\";\nend" > "$PKGFILES"/matlab/supported_octave_version.m
+cp -L  "$ROOTDIR"/mex/matlab/*                                       "$PKGFILES"/mex/matlab/maci64-9.4-9.11
 
 
 ##
@@ -175,12 +185,12 @@ echo -e "function v = supported_octave_version\nv=\"$OCTAVE_VERSION\";\nend" > "
 cd "$ROOTDIR"/macOS/pkg
 
 # Dynare option
-pkgbuild --root "$PKGFILES" --identifier com.cepremap.dynare --version "$VERSION" --install-location /Applications/Dynare/"$LOCATION" "$NAME".pkg
+pkgbuild --root "$PKGFILES" --identifier org.dynare --version "$VERSION" --install-location /Applications/Dynare/"$LOCATION" "$NAME".pkg
 
 # GCC option
 # Create dummy payload for GCC package; otherwise the size is displayed as 0 bytes in the installer
 dd if=/dev/zero of="$ROOTDIR"/macOS/brewfiles/dummy  bs=1m  count=800
-pkgbuild --root "$ROOTDIR"/macOS/brewfiles --identifier com.cepremap.dynare.gcc --version "$VERSION" --scripts "$ROOTDIR"/macOS/scripts --install-location /Applications/Dynare/"$LOCATION" "$NAME"-gcc.pkg
+pkgbuild --root "$ROOTDIR"/macOS/brewfiles --identifier org.dynare.gcc --version "$VERSION" --scripts "$ROOTDIR"/macOS/scripts --install-location /Applications/Dynare/"$LOCATION" "$NAME"-gcc.pkg
 
 # Replace variables in displayed files
 sed "s/VERSION_READ/$VERSION/g" "$ROOTDIR"/macOS/distribution_template.xml > distribution_tmp.xml
