@@ -27,7 +27,7 @@ function [eigenvalues_,result,info] = check(M, options, oo)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 
 if ~options.initval_file && M.exo_nbr > 1
@@ -42,7 +42,7 @@ end
 
 oo.dr=set_state_space(oo.dr,M,options);
 
-[dr,info,M,options,~] = resol(1,M,options,oo);
+[dr,info,M,~] = resol(1,M,options,oo);
 
 if info(1) ~= 0 && info(1) ~= 3 && info(1) ~= 4
     print_info(info, 0, options);
