@@ -28,7 +28,7 @@ function [posterior_sampler_options, options_] = check_posterior_sampler_options
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 
 init=0;
@@ -308,7 +308,7 @@ if init
         % slice posterior sampler does not require mode or hessian to run
         % needs to be set to 1 to skip parts in dynare_estimation_1.m
         % requiring posterior maximization/calibrated smoother before MCMC
-        options_.mh_posterior_mode_estimation=1;
+        options_.mh_posterior_mode_estimation=true;
 
         if ~ posterior_sampler_options.slice_initialize_with_mode
             % by default, slice sampler should trigger
@@ -324,7 +324,7 @@ if init
                 disp('check_posterior_sampler_options:: but no mode file nor posterior maximization is selected,')
                 error('check_posterior_sampler_options:: The option "slice_initialize_with_mode" is inconsistent with mode_compute=0 or empty mode_file.')
             else
-                options_.mh_posterior_mode_estimation=0;
+                options_.mh_posterior_mode_estimation=false;
             end
         end
 
@@ -335,7 +335,7 @@ if init
         end
 
         % moreover slice must be associated to:
-        %     options_.mh_posterior_mode_estimation = 0;
+        %     options_.mh_posterior_mode_estimation = false;
         % this is done below, but perhaps preprocessing should do this?
 
         if ~isempty(posterior_sampler_options.mode)
@@ -378,11 +378,13 @@ end
 
 % here are all samplers requiring a proposal distribution
 if ~strcmp(posterior_sampler_options.posterior_sampling_method,'slice')
-    if ~options_.cova_compute && ~(options_.load_mh_file && posterior_sampler_options.use_mh_covariance_matrix)
+    if ~options_.cova_compute && ~(options_.load_mh_file && posterior_sampler_options.use_mh_covariance_matrix) 
+        if strcmp('hessian',options_.MCMC_jumping_covariance)
         skipline()
         disp('check_posterior_sampler_options:: I cannot start the MCMC because the Hessian of the posterior kernel at the mode was not computed')
         disp('check_posterior_sampler_options:: or there is no previous MCMC to load ')
         error('check_posterior_sampler_options:: MCMC cannot start')
+        end
     end
 end
 
@@ -423,7 +425,7 @@ if strcmp(posterior_sampler_options.posterior_sampling_method,'slice')
     end
     % needs to be re-set to zero otherwise posterior analysis is filtered
     % out in dynare_estimation_1.m
-    options_.mh_posterior_mode_estimation = 0;
+    options_.mh_posterior_mode_estimation = false;
 end
 
 return
