@@ -12,7 +12,7 @@ function options_ = default_option_values(M_)
 % SPECIAL REQUIREMENTS
 %    none
 
-% Copyright (C) 2018-2020 Dynare Team
+% Copyright (C) 2018-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -27,7 +27,7 @@ function options_ = default_option_values(M_)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 options_.datafile = '';
 options_.dirname = M_.fname;
@@ -46,7 +46,7 @@ options_.gstep(2) = 1.0;
 options_.scalv = 1;
 options_.debug = false;
 options_.initval_file = false;
-options_.Schur_vec_tol = 1e-11; % used to find nonstationary variables in Schur decomposition of the
+options_.schur_vec_tol = 1e-11; % used to find nonstationary variables in Schur decomposition of the
                                 % transition matrix
 options_.qz_criterium = [];
 options_.qz_zero_threshold = 1e-6;
@@ -70,7 +70,7 @@ options_.huge_number = 1e7;
 
 % Default number of threads for parallelized mex files.
 options_.threads.kronecker.sparse_hessian_times_B_kronecker_C = num_procs;
-options_.threads.local_state_space_iteration_2 = 1;
+options_.threads.local_state_space_iteration_2 = num_procs;
 options_.threads.local_state_space_iteration_k = 1;
 options_.threads.perfect_foresight_problem = num_procs;
 options_.threads.k_order_perturbation = max(1, num_procs/2);
@@ -234,7 +234,7 @@ options_.ep = ep;
 
 % Simulations of backward looking models options
 %
-bnlms.set_dynare_seed_to_default = 1;
+bnlms.set_dynare_seed_to_default = true;
 bnlms.innovation_distribution = 'gaussian';
 options_.bnlms = bnlms;
 
@@ -246,8 +246,6 @@ particle.status = false;
 % How do we initialize the states?
 particle.initialization = 1;
 particle.initial_state_prior_std = .1;
-% Set the default order of approximation of the model (perturbation).
-particle.perturbation = 2;
 % Set the default number of particles.
 particle.number_of_particles = 5000;
 % Set the default approximation order (Smolyak)
@@ -288,11 +286,11 @@ particle.mixture_structural_shocks = 1 ;
 particle.mixture_measurement_shocks = 1 ;
 % Online approach
 particle.liu_west_delta = 0.99 ;
-particle.liu_west_chol_sigma_bar = .01 ;
 % Options for setting the weights in conditional particle filters.
 particle.cpf_weights_method.amisanotristani = true;
 particle.cpf_weights_method.murrayjonesparslow = false;
-% Copy ep structure in options_ global structure
+particle.particle_filter_options ='';
+% Copy particle structure in options_ global structure
 options_.particle = particle;
 options_.rwgmh.init_scale = 1e-4 ;
 options_.rwgmh.scale_chain = 1 ;
@@ -325,6 +323,9 @@ options_.minimal_solving_periods = 1;
 options_.endogenous_terminal_period = false;
 options_.no_homotopy = false;
 
+% Perfect foresight with expectation errors
+options_.pfwee.terminal_steady_state_as_guess_value = false;
+
 % Solution
 options_.order = 2;
 options_.pruning = false;
@@ -346,6 +347,7 @@ options_.ramsey.maxit = 500;
 
 % estimation
 options_.initial_period = NaN; %dates(1,1);
+options_.no_init_estimation_check_first_obs=false;
 options_.dataset.file = [];
 options_.dataset.series = [];
 options_.dataset.firstobs = dates();
@@ -354,6 +356,7 @@ options_.dataset.nobs = NaN;
 options_.dataset.xls_sheet = [];
 options_.dataset.xls_range = [];
 options_.Harvey_scale_factor = 10;
+options_.heteroskedastic_filter = false;
 options_.MaxNumberOfBytes = 1e8;
 options_.MaximumNumberOfMegaBytes = 111;
 options_.analytic_derivation = 0; % Not a boolean, can also take values -1 or 2
@@ -395,7 +398,10 @@ options_.mh_tune_jscale.c1 = .02;
 options_.mh_tune_jscale.c2 = .05;
 options_.mh_tune_jscale.c3 = 4;
 options_.mh_init_scale = 2*options_.mh_jscale;
-options_.mh_mode = 1;
+options_.mh_initialize_from_previous_mcmc.status = false;
+options_.mh_initialize_from_previous_mcmc.directory = '';
+options_.mh_initialize_from_previous_mcmc.record = '';
+options_.mh_initialize_from_previous_mcmc.prior = '';
 options_.mh_nblck = 2;
 options_.mh_recover = false;
 options_.mh_replic = 20000;
@@ -403,6 +409,9 @@ options_.recursive_estimation_restart = 0;
 options_.MCMC_jumping_covariance='hessian';
 options_.use_calibration_initialization = 0;
 options_.endo_vars_for_moment_computations_in_estimation=[];
+% occbin options
+options_.occbin.likelihood.status=false;
+options_.occbin.smoother.status=false;
 
 % Run optimizer silently
 options_.silent_optimizer = false;
@@ -417,15 +426,16 @@ options_.moments_varendo = false;
 options_.nk = 1;
 options_.noconstant = false;
 options_.nodiagnostic = false;
-options_.mh_posterior_mode_estimation = 0;
+options_.mh_posterior_mode_estimation = false;
 options_.prefilter = 0;
 options_.presample = 0;
 options_.prior_trunc = 1e-10;
 options_.smoother = false;
+options_.smoother_redux = false;
 options_.posterior_max_subsample_draws = 1200;
 options_.sub_draws = [];
 options_.ME_plot_tol=1e-6;
-% options_.use_mh_covariance_matrix = 0;
+options_.use_mh_covariance_matrix = false;
 options_.gradient_method = 2; %used by csminwel and newrat
 options_.gradient_epsilon = 1e-6; %used by csminwel and newrat
 options_.posterior_sampler_options.sampling_opt = []; %extended set of options for individual posterior samplers
@@ -465,6 +475,7 @@ options_.parallel = 0;
 options_.parallel_info.isHybridMatlabOctave = false;
 options_.parallel_info.leaveSlaveOpen = 0;
 options_.parallel_info.RemoteTmpFolder = '';
+options_.parallel_info.use_psexec = true;
 options_.number_of_grid_points_for_kde = 2^9;
 quarter = 1;
 years = [1 2 3 4 5 10 20 30 40 50];
@@ -478,7 +489,6 @@ options_.filter_decomposition = false;
 options_.selected_variables_only = false;
 options_.contemporaneous_correlation = false;
 options_.initialize_estimated_parameters_with_the_prior_mode = 0;
-options_.estimation_dll = false;
 options_.estimation.moments_posterior_density.indicator = true;
 options_.estimation.moments_posterior_density.gridpoints = 2^9;
 options_.estimation.moments_posterior_density.bandwidth = 0; % Rule of thumb optimal bandwidth parameter.
@@ -513,7 +523,7 @@ newrat.tolerance.f=1e-5;
 newrat.tolerance.f_analytic=1e-7;
 newrat.maxiter=1000;
 newrat.verbosity=1;
-newrat.Save_files=1;
+newrat.Save_files=0;
 
 options_.newrat=newrat;
 
@@ -696,9 +706,6 @@ options_.graph_save_formats.eps = 1;
 options_.graph_save_formats.pdf = 0;
 options_.graph_save_formats.fig = 0;
 
-% risky steady state
-options_.risky_steadystate = false;
-
 % endogenous prior
 options_.endogenous_prior = false;
 options_.endogenous_prior_restrictions.irf={};
@@ -740,4 +747,7 @@ options_.figures.textwidth=0.8;
 
 options_.varobs_id=[]; %initialize field
 
+options_.pac.estimation.ols.share_of_optimizing_agents.lb = 0.0;
+options_.pac.estimation.ols.share_of_optimizing_agents.ub = 1.0;
+
 end
