@@ -27,7 +27,7 @@ function [info, oo_, options_, M_] = discretionary_policy(M_, options_, oo_, var
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 M_=discretionary_policy_initialization(M_,options_);
 
@@ -36,15 +36,16 @@ options_.discretionary_policy = 1;
 options_.order = 1;
 [info, oo_, options_, M_] = stoch_simul(M_, options_, oo_, var_list);
 
+oo_.steady_state = oo_.dr.ys;
+
 if ~options_.noprint
-    disp_steady_state(M_,oo_)
+    disp_steady_state(M_,oo_,options_)
     for i=M_.orig_endo_nbr:M_.endo_nbr
         if strmatch('mult_', M_.endo_names{i})
             fprintf('%s \t\t %g\n', M_.endo_names{i}, oo_.dr.ys(i));
         end
     end
 end
-
 oo_.planner_objective_value = evaluate_planner_objective(M_,options_,oo_);
 
 options_.order = origorder;
