@@ -11,7 +11,7 @@ function oo_=disp_moments(y,var_list,M_,options_,oo_)
 % OUTPUTS
 %   oo_                 [structure]    Dynare's results structure,
 
-% Copyright (C) 2001-2019 Dynare Team
+% Copyright (C) 2001-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -26,7 +26,7 @@ function oo_=disp_moments(y,var_list,M_,options_,oo_)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 warning_old_state = warning;
 warning off
@@ -40,7 +40,7 @@ ivar=zeros(nvar,1);
 for i=1:nvar
     i_tmp = strmatch(var_list{i}, M_.endo_names, 'exact');
     if isempty(i_tmp)
-        error ('One of the variable specified does not exist') ;
+       error('The variable %s specified is not an endogenous variable',var_list{i});
     else
         ivar(i) = i_tmp;
     end
@@ -50,7 +50,7 @@ y = y(ivar,options_.drop+1:end)';
 
 ME_present=0;
 if ~all(M_.H==0)
-    if isoctave || matlab_ver_less_than('8.1')
+    if isoctave && octave_ver_less_than('6')
         [observable_pos_requested_vars, index_subset, index_observables] = intersect_stable(ivar, options_.varobs_id);
     else
         [observable_pos_requested_vars, index_subset, index_observables] = intersect(ivar, options_.varobs_id, 'stable');
@@ -94,8 +94,8 @@ oo_.var(zero_variance_var_index,:)=0;
 oo_.var(:,zero_variance_var_index)=0;
 
 
-labels = M_.endo_names(ivar);
-labels_TeX = M_.endo_names_tex(ivar);
+labels=get_labels_transformed_vars(M_.endo_names,ivar,options_,false);
+labels_TeX=get_labels_transformed_vars(M_.endo_names_tex,ivar,options_,true);
 
 if ~options_.nomoments
     z = [ m' s' s2' oo_.skewness oo_.kurtosis ];
@@ -204,10 +204,11 @@ if ~options_.nodecomposition
             headers = vertcat(' ', headers);
             lh = cellofchararraymaxlength(M_.endo_names(ivar))+2;
             dyntable(options_, title, vertcat(headers, 'Tot. lin. contr.'), ...
-                     M_.endo_names(ivar), [oo_.variance_decomposition sum(oo_.variance_decomposition,2)], lh, 8, 2);
+                     labels, [oo_.variance_decomposition sum(oo_.variance_decomposition,2)], lh, 8, 2);
             if ME_present
                 headers_ME = vertcat(headers, 'ME');
-                dyntable(options_, [title,' WITH MEASUREMENT ERROR'], vertcat(headers_ME, 'Tot. lin. contr.'), M_.endo_names(ivar(index_subset)), ...
+                labels_obs=get_labels_transformed_vars(M_.endo_names,ivar(index_subset),options_,false);
+                dyntable(options_, [title,' WITH MEASUREMENT ERROR'], vertcat(headers_ME, 'Tot. lin. contr.'), labels_obs, ...
                          [oo_.variance_decomposition_ME sum(oo_.variance_decomposition_ME, 2)], lh, 8, 2);
             end
             if options_.TeX
@@ -219,9 +220,10 @@ if ~options_.nodecomposition
                                 labels_TeX, [oo_.variance_decomposition sum(oo_.variance_decomposition, 2)], lh, 8, 2);
                 if ME_present
                     headers_ME = vertcat(headers, 'ME');
+                    labels_obs_TeX=get_labels_transformed_vars(M_.endo_names_tex,ivar(index_subset),options_,true);
                     dyn_latex_table(M_, options_, [title, ' WITH MEASUREMENT ERROR'], 'sim_var_decomp_ME', ...
                                     vertcat(headers_ME, 'Tot. lin. contr.'), ...
-                                    labels_TeX(ivar(index_subset)), ...
+                                    labels_obs_TeX, ...
                                     [oo_.variance_decomposition_ME sum(oo_.variance_decomposition_ME, 2)], lh, 8, 2);
                 end
             end
@@ -256,3 +258,4 @@ else
 end
 
 end
+
