@@ -1,8 +1,11 @@
-function [dr,info,M_,options_,oo_] = dr_block(dr,task,M_,options_,oo_,varargin)
-% function [dr,info,M_,options_,oo_] = dr_block(dr,task,M_,options_,oo_,varargin)
-% computes the reduced form solution of a rational expectations model
+function [dr,info,M_,oo_] = dr_block(dr,task,M_,options_,oo_,varargin)
+% function [dr,info,M_,oo_] = dr_block(dr,task,M_,options_,oo_,varargin)
+% computes the reduced form solution of a rational expectations block-decomposed model
 % (first order approximation of the stochastic model around the deterministic steady state).
 %
+% NB: This code does not work with option mfs > 0. The preprocessor has a check to avoid this
+%     configuration. See also #1726.
+%
 % INPUTS
 %   dr         [matlab structure] Decision rules for stochastic simulations.
 %   task       [integer]          if task = 0 then dr_block computes decision rules.
@@ -23,7 +26,6 @@ function [dr,info,M_,options_,oo_] = dr_block(dr,task,M_,options_,oo_,varargin)
 %                                 info=5: BK rank condition not satisfied.
 %                                 info=6: The jacobian matrix evaluated at the steady state is complex.
 %   M_         [matlab structure]
-%   options_   [matlab structure]
 %   oo_        [matlab structure]
 %
 % ALGORITHM
@@ -34,7 +36,7 @@ function [dr,info,M_,options_,oo_] = dr_block(dr,task,M_,options_,oo_,varargin)
 %   none.
 %
 
-% Copyright (C) 2010-2017 Dynare Team
+% Copyright (C) 2010-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -49,7 +51,7 @@ function [dr,info,M_,options_,oo_] = dr_block(dr,task,M_,options_,oo_,varargin)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 info = 0;
 verbose = 0;
@@ -59,24 +61,33 @@ end
 %verbose = options_.verbosity;
 if options_.order > 1
     error('2nd and 3rd order approximation not implemented with block option')
+else
+    if options_.loglinear
+        error('The loglinear option is not yet supported in first order approximation for a block decomposed model');
+    end
 end
 
 z = repmat(dr.ys,1,M_.maximum_lead + M_.maximum_lag + 1);
 zx = repmat([oo_.exo_simul oo_.exo_det_simul],M_.maximum_lead + M_.maximum_lag + 1, 1);
-if (isfield(M_,'block_structure'))
-    data = M_.block_structure.block;
-    Size = length(M_.block_structure.block);
-else
-    data = M_;
-    Size = 1;
+if isempty(zx)
+    zx = [repmat(oo_.exo_steady_state',M_.maximum_lead + M_.maximum_lag + 1,1) repmat(oo_.exo_det_steady_state',M_.maximum_lead + M_.maximum_lag + 1,1)];
 end
-if (options_.bytecode)
-    [chck, zz, data]= bytecode('dynamic','evaluate', z, zx, M_.params, dr.ys, 1, data);
+    
+if ~isfield(M_,'block_structure')
+    error('Option ''block'' has not been specified')
+end
+data = M_.block_structure.block;
+
+if options_.bytecode
+    [~, data]= bytecode('dynamic','evaluate', z, zx, M_.params, dr.ys, 1, data);
 else
-    [r, data] = feval([M_.fname '.dynamic'], options_, M_, oo_, z', zx, M_.params, dr.ys, M_.maximum_lag+1, data);
-    chck = 0;
+    T=NaN(M_.block_structure.dyn_tmp_nbr, 1);
+    it_=M_.maximum_lag+1;
+    y=dynvars_from_endo_simul(z, it_, M_);
+    for blk = 1:length(M_.block_structure.block)
+        [~, ~, T, data(blk).g1, data(blk).g1_x, data(blk).g1_xd, data(blk).g1_o]=feval([M_.fname '.dynamic'], blk, y, zx, M_.params, dr.ys, T, it_, true);
+    end
 end
-mexErrCheck('bytecode', chck);
 dr.full_rank = 1;
 dr.eigval = [];
 dr.nd = 0;
@@ -90,7 +101,7 @@ n_sv = size(dr.state_var, 2);
 dr.ghx = zeros(M_.endo_nbr, length(dr.state_var));
 dr.exo_var = 1:M_.exo_nbr;
 dr.ghu = zeros(M_.endo_nbr, M_.exo_nbr);
-for i = 1:Size
+for i = 1:length(data)
     ghx = [];
     indexi_0 = 0;
     if (verbose)
@@ -440,7 +451,7 @@ for i = 1:Size
             D = [[aa(row_indx,index_0m) zeros(n_dynamic,n_both) aa(row_indx,index_p)] ; [zeros(n_both, n_pred) eye(n_both) zeros(n_both, n_both + n_fwrd)]];
             E = [-aa(row_indx,[index_m index_0p])  ; [zeros(n_both, n_both + n_pred) eye(n_both, n_both + n_fwrd) ] ];
 
-            [err, ss, tt, w, sdim, data(i).eigval, info1] = mjdgges(E,D,options_.qz_criterium,options_.qz_zero_threshold);
+            [ss, tt, w, sdim, data(i).eigval, info1] = mjdgges(E,D,options_.qz_criterium,options_.qz_zero_threshold);
 
             if (verbose)
                 disp('eigval');
@@ -591,7 +602,7 @@ for i = 1:Size
                 elseif options_.sylvester_fp
                     ghx_other = gensylv_fp(A_, B_, C_, D_, i, options_.sylvester_fixed_point_tol);
                 else
-                    [err, ghx_other] = gensylv(1, A_, B_, C_, -D_);
+                    ghx_other = gensylv(1, A_, B_, C_, -D_);
                 end
                 if options_.aim_solver ~= 1
                     % Necessary when using Sims' routines for QZ
