@@ -115,7 +115,7 @@ function [fval,info,exit_flag,DLIK,Hess,SteadyState,trend_coeff,Model,DynareOpti
 %! @end deftypefn
 %@eod:
 
-% Copyright (C) 2004-2019 Dynare Team
+% Copyright (C) 2004-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -130,7 +130,7 @@ function [fval,info,exit_flag,DLIK,Hess,SteadyState,trend_coeff,Model,DynareOpti
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 % AUTHOR(S) stephane DOT adjemian AT univ DASH lemans DOT FR
 
@@ -140,24 +140,19 @@ SteadyState = [];
 trend_coeff = [];
 exit_flag   = 1;
 info        = zeros(4,1);
-DLIK        = [];
+if DynareOptions.analytic_derivation
+    DLIK        = NaN(1,length(xparam1));
+else
+    DLIK        = [];
+end
 Hess        = [];
 
 % Ensure that xparam1 is a column vector.
-xparam1 = xparam1(:);
-
-if DynareOptions.estimation_dll
-    [fval,exit_flag,SteadyState,trend_coeff,info,params,H,Q] ...
-        = logposterior(xparam1,DynareDataset, DynareOptions,Model, ...
-                       EstimatedParameters,BayesInfo,DynareResults);
-    mexErrCheck('logposterior', exit_flag);
-    Model.params = params;
-    if ~isequal(Model.H,0)
-        Model.H = H;
-    end
-    Model.Sigma_e = Q;
-    DynareResults.dr.ys = SteadyState;
-    return
+% (Don't do the transformation if xparam1 is empty, otherwise it would become a
+%  0×1 matrix, which create issues with older MATLABs when comparing with [] in
+%  check_bounds_and_definiteness_estimation)
+if ~isempty(xparam1)
+    xparam1 = xparam1(:);
 end
 
 % Set flag related to analytical derivatives.
@@ -178,92 +173,44 @@ end
 %------------------------------------------------------------------------------
 % 1. Get the structural parameters & define penalties
 %------------------------------------------------------------------------------
-
-% Return, with endogenous penalty, if some parameters are smaller than the lower bound of the prior domain.
-if isestimation(DynareOptions) && ~isequal(DynareOptions.mode_compute,1) && any(xparam1<BoundsInfo.lb)
-    k = find(xparam1<BoundsInfo.lb);
-    fval = Inf;
-    exit_flag = 0;
-    info(1) = 41;
-    info(4)= sum((BoundsInfo.lb(k)-xparam1(k)).^2);
-    if analytic_derivation
-        DLIK=ones(length(xparam1),1);
-    end
-    return
-end
-
-% Return, with endogenous penalty, if some parameters are greater than the upper bound of the prior domain.
-if isestimation(DynareOptions) && ~isequal(DynareOptions.mode_compute,1) && any(xparam1>BoundsInfo.ub)
-    k = find(xparam1>BoundsInfo.ub);
-    fval = Inf;
-    exit_flag = 0;
-    info(1) = 42;
-    info(4)= sum((xparam1(k)-BoundsInfo.ub(k)).^2);
-    if analytic_derivation
-        DLIK=ones(length(xparam1),1);
-    end
-    return
-end
-
-% Get the diagonal elements of the covariance matrices for the structural innovations (Q) and the measurement error (H).
 Model = set_all_parameters(xparam1,EstimatedParameters,Model);
 
-Q = Model.Sigma_e;
-H = Model.H;
-
-% Test if Q is positive definite.
-if ~issquare(Q) || EstimatedParameters.ncx || isfield(EstimatedParameters,'calibrated_covariances')
-    [Q_is_positive_definite, penalty] = ispd(Q(EstimatedParameters.Sigma_e_entries_to_check_for_positive_definiteness,EstimatedParameters.Sigma_e_entries_to_check_for_positive_definiteness));
-    if ~Q_is_positive_definite
-        fval = Inf;
-        exit_flag = 0;
-        info(1) = 43;
-        info(4) = penalty;
-        return
-    end
-    if isfield(EstimatedParameters,'calibrated_covariances')
-        correct_flag=check_consistency_covariances(Q);
-        if ~correct_flag
-            penalty = sum(Q(EstimatedParameters.calibrated_covariances.position).^2);
-            fval = Inf;
-            exit_flag = 0;
-            info(1) = 71;
-            info(4) = penalty;
-            return
-        end
-    end
+[fval,info,exit_flag,Q,H]=check_bounds_and_definiteness_estimation(xparam1, Model, EstimatedParameters, BoundsInfo);
+if info(1)
+    return
 end
 
-% Test if H is positive definite.
-if ~issquare(H) || EstimatedParameters.ncn || isfield(EstimatedParameters,'calibrated_covariances_ME')
-    [H_is_positive_definite, penalty] = ispd(H(EstimatedParameters.H_entries_to_check_for_positive_definiteness,EstimatedParameters.H_entries_to_check_for_positive_definiteness));
-    if ~H_is_positive_definite
-        fval = Inf;
-        info(1) = 44;
-        info(4) = penalty;
-        exit_flag = 0;
-        return
-    end
-    if isfield(EstimatedParameters,'calibrated_covariances_ME')
-        correct_flag=check_consistency_covariances(H);
-        if ~correct_flag
-            penalty = sum(H(EstimatedParameters.calibrated_covariances_ME.position).^2);
-            fval = Inf;
-            exit_flag = 0;
-            info(1) = 72;
-            info(4) = penalty;
-            return
-        end
-    end
-end
-
-
 %------------------------------------------------------------------------------
 % 2. call model setup & reduction program
 %------------------------------------------------------------------------------
+is_restrict_state_space = true;
+if DynareOptions.occbin.likelihood.status
+    occbin_options = set_occbin_options(DynareOptions, Model);
+    if occbin_options.opts_simul.restrict_state_space
+        [T,R,SteadyState,info,Model,DynareResults,TTx,RRx,CCx, T0, R0] = ...
+            occbin.dynare_resolve(Model,DynareOptions,DynareResults,[],'restrict');
+    else
+        is_restrict_state_space = false;
+        oldoo.restrict_var_list = DynareResults.dr.restrict_var_list;
+        oldoo.restrict_columns = DynareResults.dr.restrict_columns;
+        DynareResults.dr.restrict_var_list = BayesInfo.smoother_var_list;
+        DynareResults.dr.restrict_columns = BayesInfo.smoother_restrict_columns;
+    
+        % Linearize the model around the deterministic steady state and extract the matrices of the state equation (T and R).
+        [T,R,SteadyState,info,Model,DynareOptions,DynareResults,TTx,RRx,CCx, T0, R0] = ...
+            occbin.dynare_resolve(Model,DynareOptions,DynareResults);
 
-% Linearize the model around the deterministic steady state and extract the matrices of the state equation (T and R).
-[T,R,SteadyState,info,Model,DynareOptions,DynareResults] = dynare_resolve(Model,DynareOptions,DynareResults,'restrict');
+        DynareResults.dr.restrict_var_list = oldoo.restrict_var_list;
+        DynareResults.dr.restrict_columns = oldoo.restrict_columns;
+
+    end
+    occbin_.status = true;
+    occbin_.info= {DynareOptions, DynareResults, Model, occbin_options, TTx, RRx, CCx,T0,R0};
+else
+    % Linearize the model around the deterministic steady state and extract the matrices of the state equation (T and R).
+    [T,R,SteadyState,info,Model,DynareResults] = dynare_resolve(Model,DynareOptions,DynareResults,'restrict');
+    occbin_.status = false;
+end
 
 % Return, with endogenous penalty when possible, if dynare_resolve issues an error code (defined in resol).
 if info(1)
@@ -301,8 +248,14 @@ if info(1)
     return
 end
 
-% Define a vector of indices for the observed variables. Is this really usefull?...
-BayesInfo.mf = BayesInfo.mf1;
+if is_restrict_state_space
+%% Define a vector of indices for the observed variables. Is this really usefull?...
+    BayesInfo.mf = BayesInfo.mf1;
+else
+%get location of observed variables and requested smoothed variables in
+%decision rules
+    BayesInfo.mf = BayesInfo.smoother_var_list(BayesInfo.smoother_mf);
+end
 
 % Define the constant vector of the measurement equation.
 if DynareOptions.noconstant
@@ -335,6 +288,7 @@ kalman_tol = DynareOptions.kalman_tol;
 diffuse_kalman_tol = DynareOptions.diffuse_kalman_tol;
 riccati_tol = DynareOptions.riccati_tol;
 Y = transpose(DynareDataset.data)-trend;
+smpl = size(Y,2);
 
 %------------------------------------------------------------------------------
 % 3. Initial condition of the Kalman filter
@@ -345,6 +299,10 @@ kalman_algo = DynareOptions.kalman_algo;
 diffuse_periods = 0;
 expanded_state_vector_for_univariate_filter=0;
 singular_diffuse_filter = 0;
+if DynareOptions.heteroskedastic_filter
+    Qvec=get_Qvec_heteroskedastic_filter(Q,smpl,Model);
+end
+    
 switch DynareOptions.lik_init
   case 1% Standard initialization with the steady state of the state equation.
     if kalman_algo~=2
@@ -354,7 +312,18 @@ switch DynareOptions.lik_init
     Pstar=lyapunov_solver(T,R,Q,DynareOptions);
     Pinf  = [];
     a     = zeros(mm,1);
-    Zflag = 0;
+    a=set_Kalman_starting_values(a,Model,DynareResults,DynareOptions,BayesInfo);
+    a_0_given_tm1=T*a; %set state prediction for first Kalman step;
+
+    if DynareOptions.occbin.likelihood.status
+        Z =zeros(length(BayesInfo.mf),size(T,1));
+        for i = 1:length(BayesInfo.mf)
+            Z(i,BayesInfo.mf(i))=1;
+        end
+        Zflag = 1;
+    else
+        Zflag = 0;
+    end
   case 2% Initialization with large numbers on the diagonal of the covariance matrix if the states (for non stationary models).
     if kalman_algo ~= 2
         % Use standard kalman filter except if the univariate filter is explicitely choosen.
@@ -363,7 +332,17 @@ switch DynareOptions.lik_init
     Pstar = DynareOptions.Harvey_scale_factor*eye(mm);
     Pinf  = [];
     a     = zeros(mm,1);
-    Zflag = 0;
+    a = set_Kalman_starting_values(a,Model,DynareResults,DynareOptions,BayesInfo);
+    a_0_given_tm1 = T*a; %set state prediction for first Kalman step;
+    if DynareOptions.occbin.likelihood.status
+        Z =zeros(length(BayesInfo.mf),size(T,1));
+        for i = 1:length(BayesInfo.mf)
+            Z(i,BayesInfo.mf(i))=1;
+        end
+        Zflag = 1;
+    else
+        Zflag = 0;
+    end
   case 3% Diffuse Kalman filter (Durbin and Koopman)
         % Use standard kalman filter except if the univariate filter is explicitely choosen.
     if kalman_algo == 0
@@ -378,21 +357,29 @@ switch DynareOptions.lik_init
         Z(i,BayesInfo.mf(i))=1;
     end
     Zflag = 1;
+    if DynareOptions.heteroskedastic_filter
+        QQ=Qvec;
+    else
+        QQ=Q;
+    end
     % Run diffuse kalman filter on first periods.
     if (kalman_algo==3)
         % Multivariate Diffuse Kalman Filter
+        a = zeros(mm,1);
+        a = set_Kalman_starting_values(a,Model,DynareResults,DynareOptions,BayesInfo);
+        a_0_given_tm1 = T*a; %set state prediction for first Kalman step;
         Pstar0 = Pstar; % store Pstar
         if no_missing_data_flag
-            [dLIK,dlik,a,Pstar] = kalman_filter_d(Y, 1, size(Y,2), ...
-                                                  zeros(mm,1), Pinf, Pstar, ...
+            [dLIK,dlik,a_0_given_tm1,Pstar] = kalman_filter_d(Y, 1, size(Y,2), ...
+                                                  a_0_given_tm1, Pinf, Pstar, ...
                                                   kalman_tol, diffuse_kalman_tol, riccati_tol, DynareOptions.presample, ...
-                                                  T,R,Q,H,Z,mm,pp,rr);
+                                                  T,R,QQ,H,Z,mm,pp,rr);
         else
-            [dLIK,dlik,a,Pstar] = missing_observations_kalman_filter_d(DatasetInfo.missing.aindex,DatasetInfo.missing.number_of_observations,DatasetInfo.missing.no_more_missing_observations, ...
+            [dLIK,dlik,a_0_given_tm1,Pstar] = missing_observations_kalman_filter_d(DatasetInfo.missing.aindex,DatasetInfo.missing.number_of_observations,DatasetInfo.missing.no_more_missing_observations, ...
                                                               Y, 1, size(Y,2), ...
-                                                              zeros(mm,1), Pinf, Pstar, ...
+                                                              a_0_given_tm1, Pinf, Pstar, ...
                                                               kalman_tol, diffuse_kalman_tol, riccati_tol, DynareOptions.presample, ...
-                                                              T,R,Q,H,Z,mm,pp,rr);
+                                                              T,R,QQ,H,Z,mm,pp,rr);
         end
         diffuse_periods = length(dlik);
         if isinf(dLIK)
@@ -430,17 +417,28 @@ switch DynareOptions.lik_init
                 Pinf  = blkdiag(Pinf,zeros(pp));
                 H1 = zeros(pp,1);
                 mmm   = mm+pp;
-
+                if DynareOptions.heteroskedastic_filter
+                    clear QQ
+                    for kv=1:size(Qvec,3)
+                        QQ(:,:,kv) = blkdiag(Qvec(:,:,kv),H);
+                    end
+                    Qvec=QQ;
+                else
+                    QQ = Q;
+                end
             end
         end
 
-        [dLIK,dlik,a,Pstar] = univariate_kalman_filter_d(DatasetInfo.missing.aindex,...
+        a = zeros(mmm,1);
+        a = set_Kalman_starting_values(a,Model,DynareResults,DynareOptions,BayesInfo);
+        a_0_given_tm1 = T*a;
+        [dLIK,dlik,a_0_given_tm1,Pstar] = univariate_kalman_filter_d(DatasetInfo.missing.aindex,...
                                                          DatasetInfo.missing.number_of_observations,...
                                                          DatasetInfo.missing.no_more_missing_observations, ...
                                                          Y, 1, size(Y,2), ...
-                                                         zeros(mmm,1), Pinf, Pstar, ...
+                                                         a_0_given_tm1, Pinf, Pstar, ...
                                                          kalman_tol, diffuse_kalman_tol, riccati_tol, DynareOptions.presample, ...
-                                                         T,R,Q,H1,Z,mmm,pp,rr);
+                                                         T,R,QQ,H1,Z,mmm,pp,rr);
         diffuse_periods = size(dlik,1);
     end
     if isnan(dLIK)
@@ -455,19 +453,31 @@ switch DynareOptions.lik_init
     if kalman_algo ~= 2
         kalman_algo = 1;
     end
-    if isequal(H,0)
-        [err,Pstar] = kalman_steady_state(transpose(T),R*Q*transpose(R),transpose(build_selection_matrix(Z,mm,length(Z))));
-    else
-        [err,Pstar] = kalman_steady_state(transpose(T),R*Q*transpose(R),transpose(build_selection_matrix(Z,mm,length(Z))),H);
-    end
-    if err
+    try
+        if isequal(H,0)
+            Pstar = kalman_steady_state(transpose(T),R*Q*transpose(R),transpose(build_selection_matrix(Z,mm,length(Z))));
+        else
+            Pstar = kalman_steady_state(transpose(T),R*Q*transpose(R),transpose(build_selection_matrix(Z,mm,length(Z))),H);
+        end
+    catch ME
+        disp(ME.message)
         disp(['dsge_likelihood:: I am not able to solve the Riccati equation, so I switch to lik_init=1!']);
         DynareOptions.lik_init = 1;
         Pstar=lyapunov_solver(T,R,Q,DynareOptions);
     end
     Pinf  = [];
-    a = zeros(mm,1);
-    Zflag = 0;
+    a     = zeros(mm,1);
+    a = set_Kalman_starting_values(a,Model,DynareResults,DynareOptions,BayesInfo);
+    a_0_given_tm1 = T*a;    
+    if DynareOptions.occbin.likelihood.status
+        Z =zeros(length(BayesInfo.mf),size(T,1));
+        for i = 1:length(BayesInfo.mf)
+            Z(i,BayesInfo.mf(i))=1;
+        end
+        Zflag = 1;
+    else
+        Zflag = 0;
+    end
   case 5            % Old diffuse Kalman filter only for the non stationary variables
     [eigenvect, eigenv] = eig(T);
     eigenv = diag(eigenv);
@@ -487,7 +497,17 @@ switch DynareOptions.lik_init
     Pstar(stable, stable) = Pstar_tmp;
     Pinf  = [];
     a = zeros(mm,1);
-    Zflag = 0;
+    a = set_Kalman_starting_values(a,Model,DynareResults,DynareOptions,BayesInfo);
+    a_0_given_tm1 = T*a;
+    if DynareOptions.occbin.likelihood.status
+        Z =zeros(length(BayesInfo.mf),size(T,1));
+        for i = 1:length(BayesInfo.mf)
+            Z(i,BayesInfo.mf(i))=1;
+        end
+        Zflag = 1;
+    else
+        Zflag = 0;
+    end
   otherwise
     error('dsge_likelihood:: Unknown initialization approach for the Kalman filter!')
 end
@@ -511,7 +531,7 @@ if analytic_derivation
     AHess = [];
     iv = DynareResults.dr.restrict_var_list;
     if nargin<10 || isempty(derivatives_info)
-        [A,B,nou,nou,Model,DynareOptions,DynareResults] = dynare_resolve(Model,DynareOptions,DynareResults);
+        [A,B,nou,nou,Model,DynareResults] = dynare_resolve(Model,DynareOptions,DynareResults);
         if ~isempty(EstimatedParameters.var_exo)
             indexo=EstimatedParameters.var_exo(:,1);
         else
@@ -641,14 +661,16 @@ end
 %------------------------------------------------------------------------------
 % 4. Likelihood evaluation
 %------------------------------------------------------------------------------
+if DynareOptions.heteroskedastic_filter
+    Q=Qvec;
+end
 
 singularity_has_been_detected = false;
 % First test multivariate filter if specified; potentially abort and use univariate filter instead
 if ((kalman_algo==1) || (kalman_algo==3))% Multivariate Kalman Filter
-    if no_missing_data_flag
+    if no_missing_data_flag && ~DynareOptions.occbin.likelihood.status
         if DynareOptions.block
-            [err, LIK] = block_kalman_filter(T,R,Q,H,Pstar,Y,start,Z,kalman_tol,riccati_tol, Model.nz_state_var, Model.n_diag, Model.nobs_non_statevar);
-            mexErrCheck('block_kalman_filter', err);
+            [LIK,lik] = block_kalman_filter(T,R,Q,H,Pstar,Y,start,Z,kalman_tol,riccati_tol, Model.nz_state_var, Model.n_diag, Model.nobs_non_statevar);
         elseif DynareOptions.fast_kalman_filter
             if diffuse_periods
                 %kalman_algo==3 requires no diffuse periods (stationary
@@ -661,14 +683,14 @@ if ((kalman_algo==1) || (kalman_algo==3))% Multivariate Kalman Filter
                 return
             end
             [LIK,lik] = kalman_filter_fast(Y,diffuse_periods+1,size(Y,2), ...
-                                           a,Pstar, ...
+                                           a_0_given_tm1,Pstar, ...
                                            kalman_tol, riccati_tol, ...
                                            DynareOptions.presample, ...
                                            T,Q,R,H,Z,mm,pp,rr,Zflag,diffuse_periods, ...
                                            analytic_deriv_info{:});
         else
             [LIK,lik] = kalman_filter(Y,diffuse_periods+1,size(Y,2), ...
-                                      a,Pstar, ...
+                                      a_0_given_tm1,Pstar, ...
                                       kalman_tol, riccati_tol, ...
                                       DynareOptions.rescale_prediction_error_covariance, ...
                                       DynareOptions.presample, ...
@@ -677,15 +699,15 @@ if ((kalman_algo==1) || (kalman_algo==3))% Multivariate Kalman Filter
         end
     else
         if 0 %DynareOptions.block
-            [err, LIK,lik] = block_kalman_filter(DatasetInfo.missing.aindex,DatasetInfo.missing.number_of_observations,DatasetInfo.missing.no_more_missing_observations,...
-                                                 T,R,Q,H,Pstar,Y,start,Z,kalman_tol,riccati_tol, Model.nz_state_var, Model.n_diag, Model.nobs_non_statevar);
+            [LIK,lik] = block_kalman_filter(DatasetInfo.missing.aindex,DatasetInfo.missing.number_of_observations,DatasetInfo.missing.no_more_missing_observations,...
+                                            T,R,Q,H,Pstar,Y,start,Z,kalman_tol,riccati_tol, Model.nz_state_var, Model.n_diag, Model.nobs_non_statevar);
         else
             [LIK,lik] = missing_observations_kalman_filter(DatasetInfo.missing.aindex,DatasetInfo.missing.number_of_observations,DatasetInfo.missing.no_more_missing_observations,Y,diffuse_periods+1,size(Y,2), ...
-                                                           a, Pstar, ...
+                                                           a_0_given_tm1, Pstar, ...
                                                            kalman_tol, DynareOptions.riccati_tol, ...
                                                            DynareOptions.rescale_prediction_error_covariance, ...
                                                            DynareOptions.presample, ...
-                                                           T,Q,R,H,Z,mm,pp,rr,Zflag,diffuse_periods);
+                                                           T,Q,R,H,Z,mm,pp,rr,Zflag,diffuse_periods, occbin_);
         end
     end
     if analytic_derivation
@@ -752,7 +774,14 @@ if (kalman_algo==2) || (kalman_algo==4)
                 Z = [Z1, eye(pp)];
                 Zflag=1;
                 T = blkdiag(T,zeros(pp));
-                Q = blkdiag(Q,H);
+                if DynareOptions.heteroskedastic_filter
+                    clear Q
+                    for kv=1:size(Qvec,3)
+                        Q(:,:,kv) = blkdiag(Qvec(:,:,kv),H);
+                    end
+                else
+                    Q = blkdiag(Q,H);
+                end
                 R = blkdiag(R,eye(pp));
                 Pstar = blkdiag(Pstar,H);
                 Pinf  = blkdiag(Pinf,zeros(pp));
@@ -761,9 +790,11 @@ if (kalman_algo==2) || (kalman_algo==4)
             end
             mmm   = mm+pp;
             if singularity_has_been_detected
-                a = zeros(mmm,1);
+                a_tmp = zeros(mmm,1);
+                a_tmp(1:length(a_0_given_tm1)) = a_0_given_tm1;
+                a_0_given_tm1 = a_tmp;
             elseif ~expanded_state_vector_for_univariate_filter
-                a = [a; zeros(pp,1)];
+                a_0_given_tm1 = [a_0_given_tm1; zeros(pp,1)];
             end
         end
     end
@@ -771,7 +802,7 @@ if (kalman_algo==2) || (kalman_algo==4)
         analytic_deriv_info{5}=DH;
     end
     [LIK, lik] = univariate_kalman_filter(DatasetInfo.missing.aindex,DatasetInfo.missing.number_of_observations,DatasetInfo.missing.no_more_missing_observations,Y,diffuse_periods+1,size(Y,2), ...
-                                          a,Pstar, ...
+                                          a_0_given_tm1,Pstar, ...
                                           DynareOptions.kalman_tol, ...
                                           DynareOptions.riccati_tol, ...
                                           DynareOptions.presample, ...
@@ -898,3 +929,50 @@ if analytic_derivation==0 && nargout>3
     lik=lik(start:end,:);
     DLIK=[-lnprior; lik(:)];
 end
+
+function a=set_Kalman_starting_values(a,M_,oo_,options_,bayestopt_)
+% function a=set_Kalman_starting_values(a,M_,oo_,options_,bayestopt_)
+% Sets initial states guess for Kalman filter/smoother based on M_.filter_initial_state 
+% 
+% INPUTS 
+%   o a             [double]   (p*1) vector of states
+%   o M_            [structure] decribing the model
+%   o oo_           [structure] storing the results
+%   o options_      [structure] describing the options
+%   o bayestopt_    [structure] describing the priors
+%  
+% OUTPUTS
+%   o a             [double]    (p*1) vector of set initial states
+
+if isfield(M_,'filter_initial_state') && ~isempty(M_.filter_initial_state)
+    state_indices=oo_.dr.order_var(oo_.dr.restrict_var_list(bayestopt_.mf0));
+    for ii=1:size(state_indices,1)
+        if ~isempty(M_.filter_initial_state{state_indices(ii),1})
+            if options_.loglinear && ~options_.logged_steady_state
+                a(bayestopt_.mf0(ii)) = log(eval(M_.filter_initial_state{state_indices(ii),2})) - log(oo_.dr.ys(state_indices(ii)));
+            elseif ~options_.loglinear && ~options_.logged_steady_state
+                a(bayestopt_.mf0(ii)) = eval(M_.filter_initial_state{state_indices(ii),2}) - oo_.dr.ys(state_indices(ii));
+            else
+                error(['The steady state is logged. This should not happen. Please contact the developers'])
+            end
+        end
+    end
+end
+
+function occbin_options = set_occbin_options(DynareOptions, Model)
+
+% this builds the opts_simul options field needed by occbin.solver
+occbin_options.opts_simul = DynareOptions.occbin.simul;
+occbin_options.opts_simul.curb_retrench = DynareOptions.occbin.likelihood.curb_retrench;
+occbin_options.opts_simul.maxit = DynareOptions.occbin.likelihood.maxit;
+occbin_options.opts_simul.periods = DynareOptions.occbin.likelihood.periods;
+occbin_options.opts_simul.check_ahead_periods = DynareOptions.occbin.likelihood.check_ahead_periods;
+occbin_options.opts_simul.periodic_solution = DynareOptions.occbin.likelihood.periodic_solution;
+occbin_options.opts_simul.restrict_state_space = DynareOptions.occbin.likelihood.restrict_state_space;
+
+occbin_options.opts_simul.full_output = DynareOptions.occbin.likelihood.full_output;
+occbin_options.opts_simul.piecewise_only = DynareOptions.occbin.likelihood.piecewise_only;
+if ~isempty(DynareOptions.occbin.smoother.init_binding_indicator)
+    occbin_options.opts_simul.init_binding_indicator = DynareOptions.occbin.likelihood.init_binding_indicator;
+    occbin_options.opts_simul.init_regime_history=DynareOptions.occbin.likelihood.init_regime_history;
+end
