@@ -19,7 +19,7 @@ function [nvar,vartan,NumberOfConditionalDecompFiles] = ...
 %   vartan                           [char]     array of characters (with nvar rows).
 %   NumberOfConditionalDecompFiles   [integer]  scalar, number of prior or posterior data files (for covariance).
 
-% Copyright (C) 2009-2020 Dynare Team
+% Copyright (C) 2009-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -34,7 +34,7 @@ function [nvar,vartan,NumberOfConditionalDecompFiles] = ...
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 
 % Get informations about the _posterior_draws files.
@@ -66,6 +66,7 @@ if ~posterior
     end
     options_.varlist = options_.prior_analysis_endo_var_list;
 end
+endo_names=options_.varlist;
 [ivar,vartan ] = get_variables_list(options_, M_);
 if ~posterior
     if exist('temp','var')
@@ -83,7 +84,7 @@ MaXNumberOfConditionalDecompLines = ceil(options_.MaxNumberOfBytes/NumberOfSaved
 
 ME_present=0;
 if ~all(diag(M_.H)==0)
-    if isoctave || matlab_ver_less_than('8.1')
+    if isoctave && octave_ver_less_than('6')
         [observable_pos_requested_vars,index_subset,index_observables]=intersect_stable(ivar,options_.varobs_id);
     else
         [observable_pos_requested_vars,index_subset,index_observables]=intersect(ivar,options_.varobs_id,'stable');
@@ -145,7 +146,7 @@ for file = 1:NumberOfDrawsFiles
             dr = temp.pdraws{linee,2};
         else
             M_=set_parameters_locally(M_,temp.pdraws{linee,1});
-            [dr,info,M_,options_,oo_] = resol(0,M_,options_,oo_);
+			[dr,info,M_,oo_] = compute_decision_rules(M_,options_,oo_)
         end
         if first_call
             endo_nbr = M_.endo_nbr;
@@ -176,10 +177,10 @@ for file = 1:NumberOfDrawsFiles
             linea = 0;
             if posterior
                 save([M_.dname '/metropolis/' M_.fname '_PosteriorConditionalVarianceDecomposition' int2str(ConditionalDecompFileNumber) '.mat' ], ...
-                     'Conditional_decomposition_array');
+                     'Conditional_decomposition_array','endo_names');
             else
                 save([M_.dname '/prior/moments/' M_.fname '_PriorConditionalVarianceDecomposition' int2str(ConditionalDecompFileNumber) '.mat' ], ...
-                     'Conditional_decomposition_array');
+                     'Conditional_decomposition_array','endo_names');
             end
             if (ConditionalDecompFileNumber==NumberOfConditionalDecompFiles-1)% Prepare last round.
                 Conditional_decomposition_array = zeros(nvar, length(Steps),M_.exo_nbr,NumberOfLinesInTheLastConditionalDecompFile) ;
@@ -197,10 +198,10 @@ for file = 1:NumberOfDrawsFiles
                 linea_ME = 0;
                 if posterior
                     save([M_.dname '/metropolis/' M_.fname '_PosteriorConditionalVarianceDecompME' int2str(ConditionalDecompFileNumber_ME) '.mat' ], ...
-                         'Conditional_decomposition_array_ME');
+                         'Conditional_decomposition_array_ME','endo_names');
                 else
                     save([M_.dname '/prior/moments/' M_.fname '_PriorConditionalVarianceDecompME' int2str(ConditionalDecompFileNumber_ME) '.mat' ], ...
-                         'Conditional_decomposition_array_ME');
+                         'Conditional_decomposition_array_ME','endo_names');
                 end
                 if (ConditionalDecompFileNumber_ME==NumberOfConditionalDecompFiles_ME-1)% Prepare last round.
                     Conditional_decomposition_array_ME = zeros(nobs_ME, length(Steps),M_.exo_nbr+1,NumberOfLinesInTheLastConditionalDecompFile_ME) ;
