@@ -16,7 +16,7 @@ function dyn_latex_table(M_, options_, title, LaTeXtitle, headers, labels, value
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 if options_.noprint
     return
@@ -26,7 +26,7 @@ if length(headers) < 2
     error('headers length must be >= 2')
 end
 
-OutputDirectoryName = CheckPath('Output',M_.dname);
+OutputDirectoryName = CheckPath('latex',M_.dname);
 
 % Set width of label column
 if isempty(label_width)
