@@ -12,7 +12,7 @@ function dynare_estimation_1(var_list_,dname)
 % SPECIAL REQUIREMENTS
 %   none
 
-% Copyright (C) 2003-2018 Dynare Team
+% Copyright (C) 2003-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -27,10 +27,19 @@ function dynare_estimation_1(var_list_,dname)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global M_ options_ oo_ estim_params_ bayestopt_ dataset_ dataset_info
 
+if ~exist([M_.dname filesep 'Output'],'dir')
+    if isoctave && ~exist(M_.dname)
+        % See https://savannah.gnu.org/bugs/index.php?61166
+        % This workaround is needed for recursive estimation.
+        mkdir(M_.dname)
+    end
+    mkdir(M_.dname,'Output');
+end
+
 if isempty(estim_params_)
     mode_compute_o = options_.mode_compute;
     mh_replic_o = options_.mh_replic;
@@ -41,7 +50,6 @@ else
     reset_options_related_to_estimation = false;
 end
 
-
 %store qz_criterium
 qz_criterium_old=options_.qz_criterium;
 if isnan(options_.first_obs)
@@ -86,23 +94,13 @@ end
 if ~options_.dsge_var
     if options_.particle.status
         objective_function = str2func('non_linear_dsge_likelihood');
-        if strcmpi(options_.particle.filter_algorithm, 'sis')
-            options_.particle.algorithm = 'sequential_importance_particle_filter';
-        elseif strcmpi(options_.particle.filter_algorithm, 'apf')
-            options_.particle.algorithm = 'auxiliary_particle_filter';
-        elseif strcmpi(options_.particle.filter_algorithm, 'gf')
-            options_.particle.algorithm = 'gaussian_filter';
-        elseif strcmpi(options_.particle.filter_algorithm,  'gmf')
-            options_.particle.algorithm = 'gaussian_mixture_filter';
-        elseif strcmpi(options_.particle.filter_algorithm, 'cpf')
-            options_.particle.algorithm = 'conditional_particle_filter';
-        elseif strcmpi(options_.particle.filter_algorithm, 'nlkf')
-            options_.particle.algorithm = 'nonlinear_kalman_filter';
-        else
-            error(['Estimation: Unknown filter ' options_.particle.filter_algorithm])
-        end
+        [options_.particle] = check_particle_filter_options(options_.particle);
     else
-        objective_function = str2func('dsge_likelihood');
+        if options_.occbin.likelihood.status && options_.occbin.likelihood.inversion_filter
+            objective_function = str2func('occbin.IVF_posterior');
+        else
+            objective_function = str2func('dsge_likelihood');
+        end
     end
 else
     objective_function = str2func('dsge_var_likelihood');
@@ -117,10 +115,10 @@ if options_.dsge_var
         error('Estimation::DsgeVarLikelihood: I cannot estimate a DSGE-VAR model with missing observations!')
     end
     if options_.noconstant
-        var_sample_moments(options_.dsge_varlag, -1, dataset_);
+        dataset_info=var_sample_moments(options_.dsge_varlag, -1, dataset_, dataset_info);
     else
         % The steady state is non zero ==> a constant in the VAR is needed!
-        var_sample_moments(options_.dsge_varlag, 0, dataset_);
+        dataset_info=var_sample_moments(options_.dsge_varlag, 0, dataset_, dataset_info);
     end
 end
 
@@ -149,9 +147,6 @@ ncn = estim_params_.ncn;  % Covariance of the measurement innovations (number of
 np  = estim_params_.np ;  % Number of deep parameters.
 nx  = nvx+nvn+ncx+ncn+np; % Total number of parameters to be estimated.
 
-% Set the names of the priors.
-pnames = {''; 'beta'; 'gamm'; 'norm'; 'invg'; 'unif'; 'invg2'; ''; 'weibl'};
-
 dr = oo_.dr;
 
 if ~isempty(estim_params_)
@@ -176,53 +171,51 @@ catch % if check fails, provide info on using calibration if present
     rethrow(e);
 end
 
-if isequal(options_.mode_compute,0) && isempty(options_.mode_file) && options_.mh_posterior_mode_estimation==0
-    if options_.smoother
-        [atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,T,R,P,PK,decomp,Trend,state_uncertainty,M_,oo_,options_,bayestopt_] = DsgeSmoother(xparam1,gend,transpose(data),data_index,missing_value,M_,oo_,options_,bayestopt_,estim_params_);
-        [oo_]=store_smoother_results(M_,oo_,options_,bayestopt_,dataset_,dataset_info,atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,P,PK,decomp,Trend,state_uncertainty);
+if isequal(options_.mode_compute,0) && isempty(options_.mode_file) && ~options_.mh_posterior_mode_estimation
+    if options_.order==1 && ~options_.particle.status
+        if options_.smoother
+            if options_.occbin.smoother.status && options_.occbin.smoother.inversion_filter
+                [~, ~, ~, ~, ~, ~, ~, ~, ~, ~, oo_, atT, innov] = occbin.IVF_posterior(xparam1,dataset_,dataset_info,options_,M_,estim_params_,bayestopt_,prior_bounds(bayestopt_,options_.prior_trunc),oo_);
+                updated_variables = atT*nan;
+                measurement_error=[];
+                ys = oo_.dr.ys;
+                trend_coeff = zeros(length(options_.varobs_id),1);
+                bayestopt_.mf = bayestopt_.smoother_var_list(bayestopt_.smoother_mf);
+                [oo_]=store_smoother_results(M_,oo_,options_,bayestopt_,dataset_,dataset_info,atT,innov,measurement_error,updated_variables,ys,trend_coeff);
+            else
+                if options_.occbin.smoother.status
+                    [atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,T,R,P,PK,decomp,Trend,state_uncertainty,M_,oo_,bayestopt_] = occbin.DSGE_smoother(xparam1,gend,transpose(data),data_index,missing_value,M_,oo_,options_,bayestopt_,estim_params_,dataset_,dataset_info);
+                else
+                [atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,T,R,P,PK,decomp,Trend,state_uncertainty,M_,oo_,bayestopt_] = DsgeSmoother(xparam1,gend,transpose(data),data_index,missing_value,M_,oo_,options_,bayestopt_,estim_params_);
+                end
+                [oo_]=store_smoother_results(M_,oo_,options_,bayestopt_,dataset_,dataset_info,atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,P,PK,decomp,Trend,state_uncertainty);
+            end
+            if options_.forecast > 0
+                oo_.forecast = dyn_forecast(var_list_,M_,options_,oo_,'smoother',dataset_info);
+            end
+        end
+        %reset qz_criterium
+        options_.qz_criterium=qz_criterium_old;
+        return
+    else %allow to continue, e.g. with MCMC_jumping_covariance
+        if options_.smoother
+            error('Estimation:: Particle Smoothers are not yet implemented.')
+        end
     end
-    %reset qz_criterium
-    options_.qz_criterium=qz_criterium_old;
-    return
 end
 
 %% Estimation of the posterior mode or likelihood mode
 
 if ~isequal(options_.mode_compute,0) && ~options_.mh_posterior_mode_estimation
-    %prepare settings for newrat
-    if options_.mode_compute==5
-        %get whether outer product Hessian is requested
-        newratflag=[];
-        if ~isempty(options_.optim_opt)
-            options_list = read_key_value_string(options_.optim_opt);
-            for i=1:rows(options_list)
-                if strcmp(options_list{i,1},'Hessian')
-                    newratflag=options_list{i,2};
-                end
-            end
-        end
-        if options_.analytic_derivation
-            options_analytic_derivation_old = options_.analytic_derivation;
-            options_.analytic_derivation = -1;
-            if ~isempty(newratflag) && newratflag~=0 %numerical hessian explicitly specified
-                error('newrat: analytic_derivation is incompatible with numerical Hessian.')
-            else %use default
-                newratflag=0; %exclude DYNARE numerical hessian
-            end
-        elseif ~options_.analytic_derivation
-            if isempty(newratflag)
-                newratflag=options_.newrat.hess; %use default numerical dynare hessian
-            end
-        end
-    end
 
     [xparam1, fval, exitflag, hh, options_, Scale, new_rat_hess_info] = dynare_minimize_objective(objective_function,xparam1,options_.mode_compute,options_,[bounds.lb bounds.ub],bayestopt_.name,bayestopt_,hh,dataset_,dataset_info,options_,M_,estim_params_,bayestopt_,bounds,oo_);
     fprintf('\nFinal value of minus the log posterior (or likelihood):%f \n', fval);
 
-    if isnumeric(options_.mode_compute) && options_.mode_compute==5 && options_.analytic_derivation==-1 %reset options changed by newrat
-        options_.analytic_derivation = options_analytic_derivation_old; %reset
+    if isnumeric(options_.mode_compute) && options_.mode_compute==5
+        newratflag = new_rat_hess_info.newratflag;
+        new_rat_hess_info = new_rat_hess_info.new_rat_hess_info;
     elseif isnumeric(options_.mode_compute) && options_.mode_compute==6 %save scaling factor
-        save([M_.fname '_optimal_mh_scale_parameter.mat'],'Scale');
+        save([M_.dname filesep 'Output' filesep M_.fname '_optimal_mh_scale_parameter.mat'],'Scale');
         options_.mh_jscale = Scale;
         bayestopt_.jscale(:) = options_.mh_jscale;
     end
@@ -235,7 +228,7 @@ if ~isequal(options_.mode_compute,0) && ~options_.mh_posterior_mode_estimation
                                      dataset_,dataset_info,options_,M_,estim_params_,bayestopt_,bounds,oo_);
                 options_.analytic_derivation = ana_deriv_old;
             elseif ~isnumeric(options_.mode_compute) || ~(isequal(options_.mode_compute,5) && newratflag~=1 && strcmp(func2str(objective_function),'dsge_likelihood'))
-                % with flag==0, we force to use the hessian from outer product gradient of optimizer 5
+                % with flag==0 or 2, we force to use the hessian from outer product gradient of optimizer 5
                 if options_.hessian.use_penalized_objective
                     penalized_objective_function = str2func('penalty_objective_function');
                     hh = hessian(penalized_objective_function, xparam1, options_.gstep, objective_function, fval, dataset_, dataset_info, options_, M_, estim_params_, bayestopt_, bounds,oo_);
@@ -254,7 +247,7 @@ if ~isequal(options_.mode_compute,0) && ~options_.mh_posterior_mode_estimation
                 % with diagonal elements computed with numerical second order derivatives
                 %
                 % uses univariate filters, so to get max # of available
-                % densitities for outer product gradient
+                % densities for outer product gradient
                 kalman_algo0 = options_.kalman_algo;
                 compute_hessian = 1;
                 if ~((options_.kalman_algo == 2) || (options_.kalman_algo == 4))
@@ -268,7 +261,7 @@ if ~isequal(options_.mode_compute,0) && ~options_.mh_posterior_mode_estimation
                 if compute_hessian
                     crit = options_.newrat.tolerance.f;
                     newratflag = newratflag>0;
-                    hh = reshape(mr_hessian(xparam1,objective_function,fval,newratflag,crit,new_rat_hess_info,dataset_, dataset_info, options_,M_,estim_params_,bayestopt_,bounds,oo_), nx, nx);
+                    hh = reshape(mr_hessian(xparam1,objective_function,fval,newratflag,crit,new_rat_hess_info,[bounds.lb bounds.ub],bayestopt_.p2,0,dataset_, dataset_info, options_,M_,estim_params_,bayestopt_,bounds,oo_), nx, nx);
                 end
                 options_.kalman_algo = kalman_algo0;
             end
@@ -276,9 +269,9 @@ if ~isequal(options_.mode_compute,0) && ~options_.mh_posterior_mode_estimation
     end
     parameter_names = bayestopt_.name;
     if options_.cova_compute || options_.mode_compute==5 || options_.mode_compute==6
-        save([M_.fname '_mode.mat'],'xparam1','hh','parameter_names','fval');
+        save([M_.dname filesep 'Output' filesep M_.fname '_mode.mat'],'xparam1','hh','parameter_names','fval');
     else
-        save([M_.fname '_mode.mat'],'xparam1','parameter_names','fval');
+        save([M_.dname filesep 'Output' filesep M_.fname '_mode.mat'],'xparam1','parameter_names','fval');
     end
 end
 
@@ -355,9 +348,17 @@ if ~options_.cova_compute
     stdh = NaN(length(xparam1),1);
 end
 
+if options_.particle.status && isfield(options_.particle,'posterior_sampler')
+    if strcmpi(options_.particle.posterior_sampler,'Herbst_Schorfheide')
+        Herbst_Schorfheide_sampler(objective_function,xparam1,bounds,dataset_,dataset_info,options_,M_,estim_params_,bayestopt_,oo_)
+    elseif strcmpi(options_.particle.posterior_sampler,'DSMH')
+        DSMH_sampler(objective_function,xparam1,bounds,dataset_,dataset_info,options_,M_,estim_params_,bayestopt_,oo_)
+    end
+end
+        
 if any(bayestopt_.pshape > 0) && ~options_.mh_posterior_mode_estimation
     % display results table and store parameter estimates and standard errors in results
-    oo_ = display_estimation_results_table(xparam1, stdh, M_, options_, estim_params_, bayestopt_, oo_, pnames, 'Posterior', 'posterior');
+    oo_ = display_estimation_results_table(xparam1, stdh, M_, options_, estim_params_, bayestopt_, oo_, prior_dist_names, 'Posterior', 'posterior');
     % Laplace approximation to the marginal log density:
     if options_.cova_compute
         estim_params_nbr = size(xparam1,1);
@@ -378,12 +379,12 @@ if any(bayestopt_.pshape > 0) && ~options_.mh_posterior_mode_estimation
     end
 
 elseif ~any(bayestopt_.pshape > 0) && ~options_.mh_posterior_mode_estimation
-    oo_=display_estimation_results_table(xparam1, stdh, M_, options_, estim_params_, bayestopt_, oo_, pnames, 'Maximum Likelihood', 'mle');
+    oo_=display_estimation_results_table(xparam1, stdh, M_, options_, estim_params_, bayestopt_, oo_, prior_dist_names, 'Maximum Likelihood', 'mle');
 end
 
 if np > 0
     pindx = estim_params_.param_vals(:,1);
-    save([M_.fname '_params.mat'],'pindx');
+    save([M_.dname filesep 'Output' filesep M_.fname '_params.mat'],'pindx');
 end
 
 switch options_.MCMC_jumping_covariance
@@ -440,14 +441,19 @@ if (any(bayestopt_.pshape  >0 ) && options_.mh_replic) || ...
     % Tunes the jumping distribution's scale parameter
     if options_.mh_tune_jscale.status
         if strcmp(options_.posterior_sampler_options.posterior_sampling_method, 'random_walk_metropolis_hastings')
+            %get invhess in case of use_mh_covariance_matrix
+            posterior_sampler_options_temp = options_.posterior_sampler_options.current_options;
+            posterior_sampler_options_temp.invhess = invhess;
+            posterior_sampler_options_temp = check_posterior_sampler_options(posterior_sampler_options_temp, options_);
+
             options = options_.mh_tune_jscale;
             options.rwmh = options_.posterior_sampler_options.rwmh;
             options_.mh_jscale = calibrate_mh_scale_parameter(objective_function, ...
-                                                              invhess, xparam1, [bounds.lb,bounds.ub], ...
+                                                              posterior_sampler_options_temp.invhess, xparam1, [bounds.lb,bounds.ub], ...
                                                               options, dataset_, dataset_info, options_, M_, estim_params_, bayestopt_, bounds, oo_);
+            clear('posterior_sampler_options_temp','options')
             bayestopt_.jscale(:) = options_.mh_jscale;
-            disp(sprintf('mh_jscale has been set equal to %s', num2str(options_.mh_jscale)))
-            skipline()
+            fprintf('mh_jscale has been set equal to %s\n', num2str(options_.mh_jscale))
         else
             warning('mh_tune_jscale is only available with Random Walk Metropolis Hastings!')
         end
@@ -469,13 +475,15 @@ if (any(bayestopt_.pshape  >0 ) && options_.mh_replic) || ...
     %% Here I discard first mh_drop percent of the draws:
     CutSample(M_, options_, estim_params_);
     if options_.mh_posterior_mode_estimation
+        [~,~,posterior_mode,~] = compute_mh_covariance_matrix();
+        oo_=fill_mh_mode(posterior_mode',NaN(length(posterior_mode),1),M_,options_,estim_params_,bayestopt_,oo_,'posterior');
         %reset qz_criterium
         options_.qz_criterium=qz_criterium_old;
         return
     else
         %get stored results if required
         if options_.load_mh_file && options_.load_results_after_load_mh
-            oo_load_mh=load([M_.fname '_results'],'oo_');
+            oo_load_mh=load([M_.dname filesep 'Output' filesep M_.fname '_results'],'oo_');
         end
         if ~options_.nodiagnostic
             if (options_.mh_replic>0 || (options_.load_mh_file && ~options_.load_results_after_load_mh))
@@ -490,7 +498,7 @@ if (any(bayestopt_.pshape  >0 ) && options_.mh_replic) || ...
         if options_.mh_replic || (options_.load_mh_file && ~options_.load_results_after_load_mh)
             [marginal,oo_] = marginal_density(M_, options_, estim_params_, oo_, bayestopt_);
             % Store posterior statistics by parameter name
-            oo_ = GetPosteriorParametersStatistics(estim_params_, M_, options_, bayestopt_, oo_, pnames);
+            oo_ = GetPosteriorParametersStatistics(estim_params_, M_, options_, bayestopt_, oo_, prior_dist_names);
             if ~options_.nograph
                 oo_ = PlotPosteriorDistributions(estim_params_, M_, options_, bayestopt_, oo_);
             end
@@ -530,13 +538,36 @@ if (any(bayestopt_.pshape  >0 ) && options_.mh_replic) || ...
                 if error_flag
                     error('Estimation::mcmc: I cannot compute the posterior moments for the endogenous variables!')
                 end
+                if options_.load_mh_file && options_.mh_replic==0 %user wants to recompute results
+                   [MetropolisFolder, info] = CheckPath('metropolis',M_.dname);
+                   if ~info
+                       generic_post_data_file_name={'Posterior2ndOrderMoments','decomposition','PosteriorVarianceDecomposition','correlation','PosteriorCorrelations','conditional decomposition','PosteriorConditionalVarianceDecomposition'};
+                       for ii=1:length(generic_post_data_file_name)
+                           delete_stale_file([MetropolisFolder filesep M_.fname '_' generic_post_data_file_name{1,ii} '*']);
+                       end
+                       % restore compatibility for loading pre-4.6.2 runs where estim_params_ was not saved; see 6e06acc7 and !1944
+                       NumberOfDrawsFiles = length(dir([M_.dname '/metropolis/' M_.fname '_posterior_draws*' ]));
+                       if NumberOfDrawsFiles>0
+                           temp=load([M_.dname '/metropolis/' M_.fname '_posterior_draws1']);
+                           if ~isfield(temp,'estim_params_')
+                               for file_iter=1:NumberOfDrawsFiles
+                                   save([M_.dname '/metropolis/' M_.fname '_posterior_draws' num2str(file_iter)],'estim_params_','-append') 
+                               end
+                           end
+                       end
+                   end
+                end
                 oo_ = compute_moments_varendo('posterior',options_,M_,oo_,var_list_);
             end
             if options_.smoother || ~isempty(options_.filter_step_ahead) || options_.forecast
                 if error_flag
                     error('Estimation::mcmc: I cannot compute the posterior statistics!')
                 end
-                prior_posterior_statistics('posterior',dataset_,dataset_info);
+                if options_.order==1 && ~options_.particle.status
+                    prior_posterior_statistics('posterior',dataset_,dataset_info); %get smoothed and filtered objects and forecasts
+                else
+                    error('Estimation::mcmc: Particle Smoothers are not yet implemented.')
+                end
             end
         else
             fprintf('Estimation:mcmc: sub_draws was set to 0. Skipping posterior computations.')
@@ -555,16 +586,29 @@ end
 if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.pshape> 0) && options_.load_mh_file)) ...
     || ~options_.smoother ) && ~options_.partial_information  % to be fixed
     %% ML estimation, or posterior mode without Metropolis-Hastings or Metropolis without Bayesian smoothes variables
-    [atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,T,R,P,PK,decomp,Trend,state_uncertainty,M_,oo_,options_,bayestopt_] = DsgeSmoother(xparam1,dataset_.nobs,transpose(dataset_.data),dataset_info.missing.aindex,dataset_info.missing.state,M_,oo_,options_,bayestopt_,estim_params_);
-    [oo_,yf]=store_smoother_results(M_,oo_,options_,bayestopt_,dataset_,dataset_info,atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,P,PK,decomp,Trend,state_uncertainty);
-
+    if options_.occbin.smoother.status && options_.occbin.smoother.inversion_filter
+        [~, ~, ~, ~, ~, ~, ~, ~, ~, ~, oo_, atT, innov] = occbin.IVF_posterior(xparam1,dataset_,dataset_info,options_,M_,estim_params_,bayestopt_,prior_bounds(bayestopt_,options_.prior_trunc),oo_);
+        updated_variables = atT*nan;
+        measurement_error=[];
+        ys = oo_.dr.ys;
+        trend_coeff = zeros(length(options_.varobs_id),1);
+        bayestopt_.mf = bayestopt_.smoother_var_list(bayestopt_.smoother_mf);
+        [oo_, yf]=store_smoother_results(M_,oo_,options_,bayestopt_,dataset_,dataset_info,atT,innov,measurement_error,updated_variables,ys,trend_coeff);
+    else
+        if options_.occbin.smoother.status
+            [atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,T,R,P,PK,decomp,Trend,state_uncertainty,M_,oo_,bayestopt_] = occbin.DSGE_smoother(xparam1,dataset_.nobs,transpose(dataset_.data),dataset_info.missing.aindex,dataset_info.missing.state,M_,oo_,options_,bayestopt_,estim_params_,dataset_,dataset_info);
+        else
+            [atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,T,R,P,PK,decomp,Trend,state_uncertainty,M_,oo_,bayestopt_] = DsgeSmoother(xparam1,dataset_.nobs,transpose(dataset_.data),dataset_info.missing.aindex,dataset_info.missing.state,M_,oo_,options_,bayestopt_,estim_params_);
+        end
+        [oo_,yf]=store_smoother_results(M_,oo_,options_,bayestopt_,dataset_,dataset_info,atT,innov,measurement_error,updated_variables,ys,trend_coeff,aK,P,PK,decomp,Trend,state_uncertainty);
+    end
     if ~options_.nograph
         [nbplt,nr,nc,lr,lc,nstar] = pltorg(M_.exo_nbr);
-        if ~exist([M_.fname '/graphs'],'dir')
-            mkdir(M_.fname,'graphs');
+        if ~exist([M_.dname '/graphs'],'dir')
+            mkdir(M_.dname,'graphs');
         end
         if options_.TeX && any(strcmp('eps',cellstr(options_.graph_format)))
-            fidTeX = fopen([M_.fname, '/graphs/' M_.fname '_SmoothedShocks.tex'],'w');
+            fidTeX = fopen([M_.dname, '/graphs/' M_.fname '_SmoothedShocks.tex'],'w');
             fprintf(fidTeX,'%% TeX eps-loader file generated by dynare_estimation_1.m (Dynare).\n');
             fprintf(fidTeX,['%% ' datestr(now,0) '\n']);
             fprintf(fidTeX,' \n');
@@ -589,11 +633,6 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
                 plot(1:gend,innov(k,:),marker_string{2,1},'linewidth',1)
                 hold off
                 name = M_.exo_names{k};
-                if isempty(NAMES)
-                    NAMES = name;
-                else
-                    NAMES = char(NAMES,name);
-                end
                 if ~isempty(options_.XTick)
                     set(gca,'XTick',options_.XTick)
                     set(gca,'XTickLabel',options_.XTickLabel)
@@ -602,23 +641,16 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
                     xlim([1 gend])
                 end
                 if options_.TeX
-                    texname = M_.exo_names_tex{k};
-                    if isempty(TeXNAMES)
-                        TeXNAMES = ['$ ' deblank(texname) ' $'];
-                    else
-                        TeXNAMES = char(TeXNAMES,['$ ' deblank(texname) ' $']);
-                    end
+                    title(['$' M_.exo_names_tex{k} '$'],'Interpreter','latex')
+                else
+                    title(name,'Interpreter','none')
                 end
-                title(name,'Interpreter','none')
             end
-            dyn_saveas(fh,[M_.fname, '/graphs/' M_.fname '_SmoothedShocks' int2str(plt)],options_.nodisplay,options_.graph_format);
+            dyn_saveas(fh,[M_.dname, '/graphs/' M_.fname '_SmoothedShocks' int2str(plt)],options_.nodisplay,options_.graph_format);
             if options_.TeX && any(strcmp('eps',cellstr(options_.graph_format)))
                 fprintf(fidTeX,'\\begin{figure}[H]\n');
-                for jj = 1:nstar0
-                    fprintf(fidTeX,'\\psfrag{%s}[1][][0.5][0]{%s}\n',deblank(NAMES(jj,:)),deblank(TeXNAMES(jj,:)));
-                end
                 fprintf(fidTeX,'\\centering \n');
-                fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_SmoothedShocks%s}\n',options_.figures.textwidth*min(i/nc,1),[M_.fname, '/graphs/' M_.fname],int2str(plt));
+                fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_SmoothedShocks%s}\n',options_.figures.textwidth*min(i/nc,1),[M_.dname, '/graphs/' M_.fname],int2str(plt));
                 fprintf(fidTeX,'\\caption{Smoothed shocks.}');
                 fprintf(fidTeX,'\\label{Fig:SmoothedShocks:%s}\n',int2str(plt));
                 fprintf(fidTeX,'\\end{figure}\n');
@@ -643,15 +675,13 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
         if ~options_.nograph
             [nbplt,nr,nc,lr,lc,nstar] = pltorg(number_of_plots_to_draw);
             if options_.TeX && any(strcmp('eps',cellstr(options_.graph_format)))
-                fidTeX = fopen([M_.fname, '/graphs/' M_.fname '_SmoothedObservationErrors.tex'],'w');
+                fidTeX = fopen([M_.dname, '/graphs/' M_.fname '_SmoothedObservationErrors.tex'],'w');
                 fprintf(fidTeX,'%% TeX eps-loader file generated by dynare_estimation_1.m (Dynare).\n');
                 fprintf(fidTeX,['%% ' datestr(now,0) '\n']);
                 fprintf(fidTeX,' \n');
             end
             for plt = 1:nbplt
                 fh = dyn_figure(options_.nodisplay,'Name','Smoothed observation errors');
-                NAMES = [];
-                if options_.TeX, TeXNAMES = []; end
                 nstar0=min(nstar,number_of_plots_to_draw-(plt-1)*nstar);
                 if gend==1
                     marker_string{1,1}='-ro';
@@ -671,34 +701,22 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
                     if gend>1
                         xlim([1 gend])
                     end
-                    if isempty(NAMES)
-                        NAMES = name;
-                    else
-                        NAMES = char(NAMES,name);
-                    end
                     if ~isempty(options_.XTick)
                         set(gca,'XTick',options_.XTick)
                         set(gca,'XTickLabel',options_.XTickLabel)
                     end
                     if options_.TeX
                         idx = strmatch(options_.varobs{index(k)}, M_.endo_names, 'exact');
-                        texname = M_.endo_names_tex{idx};
-                        if isempty(TeXNAMES)
-                            TeXNAMES = ['$ ' texname ' $'];
-                        else
-                            TeXNAMES = char(TeXNAMES,['$ ' texname ' $']);
-                        end
+                        title(['$' M_.endo_names_tex{idx} '$'],'Interpreter','latex')
+                    else
+                        title(name,'Interpreter','none')
                     end
-                    title(name,'Interpreter','none')
                 end
-                dyn_saveas(fh,[M_.fname, '/graphs/' M_.fname '_SmoothedObservationErrors' int2str(plt)],options_.nodisplay,options_.graph_format);
+                dyn_saveas(fh,[M_.dname, '/graphs/' M_.fname '_SmoothedObservationErrors' int2str(plt)],options_.nodisplay,options_.graph_format);
                 if options_.TeX && any(strcmp('eps',cellstr(options_.graph_format)))
                     fprintf(fidTeX,'\\begin{figure}[H]\n');
-                    for jj = 1:nstar0
-                        fprintf(fidTeX,'\\psfrag{%s}[1][][0.5][0]{%s}\n',deblank(NAMES(jj,:)),deblank(TeXNAMES(jj,:)));
-                    end
                     fprintf(fidTeX,'\\centering \n');
-                    fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_SmoothedObservationErrors%s}\n',options_.figures.textwidth*min(i/nc,1),[M_.fname, '/graphs/' M_.fname],int2str(plt));
+                    fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_SmoothedObservationErrors%s}\n',options_.figures.textwidth*min(i/nc,1),[M_.dname, '/graphs/' M_.fname],int2str(plt));
                     fprintf(fidTeX,'\\caption{Smoothed observation errors.}');
                     fprintf(fidTeX,'\\label{Fig:SmoothedObservationErrors:%s}\n',int2str(plt));
                     fprintf(fidTeX,'\\end{figure}\n');
@@ -718,7 +736,7 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
     if ~options_.nograph
         [nbplt,nr,nc,lr,lc,nstar] = pltorg(n_varobs);
         if options_.TeX && any(strcmp('eps',cellstr(options_.graph_format)))
-            fidTeX = fopen([M_.fname, '/graphs/' M_.fname '_HistoricalAndSmoothedVariables.tex'],'w');
+            fidTeX = fopen([M_.dname, '/graphs/' M_.fname '_HistoricalAndSmoothedVariables.tex'],'w');
             fprintf(fidTeX,'%% TeX eps-loader file generated by dynare_estimation_1.m (Dynare).\n');
             fprintf(fidTeX,['%% ' datestr(now,0) '\n']);
             fprintf(fidTeX,' \n');
@@ -726,7 +744,6 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
         for plt = 1:nbplt
             fh = dyn_figure(options_.nodisplay,'Name','Historical and smoothed variables');
             NAMES = [];
-            if options_.TeX, TeXNAMES = []; end
             nstar0=min(nstar,n_varobs-(plt-1)*nstar);
             if gend==1
                 marker_string{1,1}='-ro';
@@ -743,11 +760,6 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
                 plot(1:gend,rawdata(:,k),marker_string{2,1},'linewidth',1)
                 hold off
                 name = options_.varobs{k};
-                if isempty(NAMES)
-                    NAMES = name;
-                else
-                    NAMES = char(NAMES,name);
-                end
                 if ~isempty(options_.XTick)
                     set(gca,'XTick',options_.XTick)
                     set(gca,'XTickLabel',options_.XTickLabel)
@@ -757,23 +769,16 @@ if (~((any(bayestopt_.pshape > 0) && options_.mh_replic) || (any(bayestopt_.psha
                 end
                 if options_.TeX
                     idx = strmatch(options_.varobs{k}, M_.endo_names,'exact');
-                    texname = M_.endo_names_tex{idx};
-                    if isempty(TeXNAMES)
-                        TeXNAMES = ['$ ' texname ' $'];
-                    else
-                        TeXNAMES = char(TeXNAMES,['$ ' texname ' $']);
-                    end
+                    title(['$' M_.endo_names_tex{idx} '$'],'Interpreter','latex')
+                else
+                    title(name,'Interpreter','none')
                 end
-                title(name,'Interpreter','none')
             end
-            dyn_saveas(fh,[M_.fname, '/graphs/' M_.fname '_HistoricalAndSmoothedVariables' int2str(plt)],options_.nodisplay,options_.graph_format);
+            dyn_saveas(fh,[M_.dname, '/graphs/' M_.fname '_HistoricalAndSmoothedVariables' int2str(plt)],options_.nodisplay,options_.graph_format);
             if options_.TeX && any(strcmp('eps',cellstr(options_.graph_format)))
                 fprintf(fidTeX,'\\begin{figure}[H]\n');
-                for jj = 1:nstar0
-                    fprintf(fidTeX,'\\psfrag{%s}[1][][0.5][0]{%s}\n',deblank(NAMES(jj,:)),deblank(TeXNAMES(jj,:)));
-                end
                 fprintf(fidTeX,'\\centering \n');
-                fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_HistoricalAndSmoothedVariables%s}\n',options_.figures.textwidth*min(i/nc,1),[M_.fname, '/graphs/' M_.fname],int2str(plt));
+                fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_HistoricalAndSmoothedVariables%s}\n',options_.figures.textwidth*min(i/nc,1),[M_.dname, '/graphs/' M_.fname],int2str(plt));
                 fprintf(fidTeX,'\\caption{Historical and smoothed variables.}');
                 fprintf(fidTeX,'\\label{Fig:HistoricalAndSmoothedVariables:%s}\n',int2str(plt));
                 fprintf(fidTeX,'\\end{figure}\n');
@@ -794,7 +799,7 @@ end
 
 if np > 0
     pindx = estim_params_.param_vals(:,1);
-    save([M_.fname '_pindx.mat'] ,'pindx');
+    save([M_.dname filesep 'Output' filesep M_.fname '_pindx.mat'] ,'pindx');
 end
 
 %reset qz_criterium
