@@ -32,7 +32,7 @@ function [dataset_, dataset_info, xparam1, hh, M_, options_, oo_, estim_params_,
 % SPECIAL REQUIREMENTS
 %   none
 
-% Copyright (C) 2003-2020 Dynare Team
+% Copyright (C) 2003-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -47,7 +47,7 @@ function [dataset_, dataset_info, xparam1, hh, M_, options_, oo_, estim_params_,
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 hh = [];
 xparam1 = [];
@@ -621,7 +621,7 @@ end
 
 % If the steady state of the observed variables is non zero, set noconstant equal 0 ()
 if (~options_.loglinear && all(abs(oo_.steady_state(bayestopt_.mfys))<1e-9)) || (options_.loglinear && all(abs(log(oo_.steady_state(bayestopt_.mfys)))<1e-9))
-    options_.noconstant = 1;
+    options_.noconstant = 0; %identifying the constant based on just the initial parameter value is not feasible
 else
     options_.noconstant = 0;
     % If the data are prefiltered then there must not be constants in the
@@ -638,28 +638,7 @@ end
 
 %% get the non-zero rows and columns of Sigma_e and H
 
-H_non_zero_rows=find(~all(M_.H==0,1));
-H_non_zero_columns=find(~all(M_.H==0,2));
-if ~isequal(H_non_zero_rows,H_non_zero_columns')
-    error('Measurement error matrix not symmetric')
-end
-if isfield(estim_params_,'nvn_observable_correspondence')
-    estim_params_.H_entries_to_check_for_positive_definiteness=union(H_non_zero_rows,estim_params_.nvn_observable_correspondence(:,1));
-else
-    estim_params_.H_entries_to_check_for_positive_definiteness=H_non_zero_rows;
-end
-
-Sigma_e_non_zero_rows=find(~all(M_.Sigma_e==0,1));
-Sigma_e_non_zero_columns=find(~all(M_.Sigma_e==0,2));
-if ~isequal(Sigma_e_non_zero_rows,Sigma_e_non_zero_columns')
-    error('Structual error matrix not symmetric')
-end
-if isfield(estim_params_,'var_exo') && ~isempty(estim_params_.var_exo)
-    estim_params_.Sigma_e_entries_to_check_for_positive_definiteness=union(Sigma_e_non_zero_rows,estim_params_.var_exo(:,1));
-else
-    estim_params_.Sigma_e_entries_to_check_for_positive_definiteness=Sigma_e_non_zero_rows;
-end
-
+estim_params_= get_matrix_entries_for_psd_check(M_,estim_params_);
 
 if options_.load_results_after_load_mh
     if ~exist([M_.fname '_results.mat'],'file')
@@ -673,3 +652,51 @@ if options_.mh_replic || options_.load_mh_file
     [current_options, options_] = check_posterior_sampler_options([], options_, bounds);
     options_.posterior_sampler_options.current_options = current_options;
 end
+
+%% set heteroskedastic shocks
+if options_.heteroskedastic_filter
+    if options_.fast_kalman_filter
+        error('estimation option conflict: "heteroskedastic_filter" incompatible with "fast_kalman_filter"')
+    end
+    if options_.block
+        error('estimation option conflict: "heteroskedastic_filter" incompatible with block kalman filter')
+    end
+    if options_.analytic_derivation
+        error(['estimation option conflict: analytic_derivation isn''t available ' ...
+            'for heteroskedastic_filter'])
+    end
+
+    M_.heteroskedastic_shocks.Qvalue = NaN(M_.exo_nbr,options_.nobs);
+    M_.heteroskedastic_shocks.Qscale = NaN(M_.exo_nbr,options_.nobs);
+
+    for k=1:length(M_.heteroskedastic_shocks.Qvalue_orig)
+        v = M_.heteroskedastic_shocks.Qvalue_orig(k);
+        temp_periods=v.periods(v.periods<options_.nobs+options_.first_obs);
+        temp_periods=temp_periods(temp_periods>=options_.first_obs);
+        M_.heteroskedastic_shocks.Qvalue(v.exo_id, temp_periods-(options_.first_obs-1)) = v.value^2;
+    end
+    for k=1:length(M_.heteroskedastic_shocks.Qscale_orig)
+        v = M_.heteroskedastic_shocks.Qscale_orig(k);
+        temp_periods=v.periods(v.periods<options_.nobs+options_.first_obs);
+        temp_periods=temp_periods(temp_periods>=options_.first_obs);
+        M_.heteroskedastic_shocks.Qscale(v.exo_id, temp_periods-(options_.first_obs-1)) = v.scale^2;
+    end
+
+    if any(any(~isnan(M_.heteroskedastic_shocks.Qvalue) & ~isnan(M_.heteroskedastic_shocks.Qscale)))
+        fprintf('\ndynare_estimation_init: With the option "heteroskedastic_shocks" you cannot define\n')
+        fprintf('dynare_estimation_init: the scale and the value for the same shock \n')
+        fprintf('dynare_estimation_init: in the same period!\n')
+        error('Scale and value defined for the same shock in the same period with "heteroskedastic_shocks".')
+    end
+end
+
+if options_.occbin.likelihood.status && options_.occbin.likelihood.inversion_filter
+    if isempty(options_.occbin.likelihood.IVF_shock_observable_mapping)
+        options_.occbin.likelihood.IVF_shock_observable_mapping=find(diag(M.Sigma_e)~=0);
+    else
+        zero_var_shocks=find(diag(M.Sigma_e)==0);
+        if any(ismember(options_.occbin.likelihood.IVF_shock_observable_mapping,zero_var_shocks))
+            error('IVF-filter: an observable is mapped to a zero variance shock.')
+        end
+    end
+end
\ No newline at end of file
