@@ -46,7 +46,7 @@ function [pdraws, STO_REDUCEDFORM, STO_MOMENTS, STO_DYNAMIC, STO_si_dDYNAMIC, ST
 %    * skipline
 %    * vnorm
 % =========================================================================
-% Copyright (C) 2010-2020 Dynare Team
+% Copyright (C) 2010-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -61,11 +61,24 @@ function [pdraws, STO_REDUCEDFORM, STO_MOMENTS, STO_DYNAMIC, STO_si_dDYNAMIC, ST
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 % =========================================================================
 
 global M_ options_ oo_ bayestopt_ estim_params_
 
+% The TeX option crashes MATLAB R2014a run with "-nodisplay" option
+% (as is done from the testsuite).
+% Since we can’t directly test whether "-nodisplay" has been passed,
+% we test for the "TOP_TEST_DIR" environment variable, which is set
+% by the testsuite.
+% Note that it was not tested whether the crash happens with more
+% recent MATLAB versions, so when OLD_MATLAB_VERSION is increased,
+% one should make a test before removing this workaround.
+if options_.TeX && ~isoctave && matlab_ver_less_than('8.4') && ~isempty(getenv('TOP_TEST_DIR'))
+    warning('Disabling TeX option due to a bug in MATLAB R2014a with -nodisplay')
+    options_.TeX = false;
+end
+
 store_options_ = options_; % store options to restore them at the end
 fname = M_.fname; %model name
 dname = M_.dname; %model name
@@ -139,6 +152,8 @@ options_ident = set_default_option(options_ident,'tol_deriv',1.e-8);
     % tolerance level for selecting columns of non-zero derivatives
 options_ident = set_default_option(options_ident,'tol_sv',1.e-3);
     % tolerance level for selecting non-zero singular values in identification_checks.m
+options_ident = set_default_option(options_ident,'schur_vec_tol',1e-11);
+    % tolerance level used to find nonstationary variables in Schur decomposition of the transition matrix.
 
 %check whether to compute identification strength based on information matrix
 if ~isfield(options_ident,'no_identification_strength')
@@ -212,6 +227,10 @@ options_ident = set_default_option(options_ident,'order',1);
     % 3: third-order perturbation approximation, identification is based on third-order pruned state space system
 
 %overwrite values in options_, note that options_ is restored at the end of the function
+if isfield(options_ident,'prior_trunc')
+    options_.prior_trunc=options_ident.prior_trunc;
+        % sets truncation of prior
+end
 if isfield(options_ident,'TeX')
     options_.TeX=options_ident.TeX;
         % requests printing of results and graphs in LaTeX
@@ -285,7 +304,7 @@ if options_ident.order == 3
 end
 options_.ar = options_ident.ar;
 options_.prior_mc = options_ident.prior_mc;
-options_.Schur_vec_tol = 1.e-8;
+options_.schur_vec_tol = options_ident.schur_vec_tol;
 options_.nomoments = 0;
 options_.analytic_derivation=options_ident.analytic_derivation;
     % 1: analytic derivation of gradient and hessian of likelihood in dsge_likelihood.m, only works for stationary models, i.e. kalman_algo<3
@@ -302,6 +321,14 @@ options_ident = set_default_option(options_ident,'analytic_derivation_mode', opt
     %  1: kronecker products method to compute analytical derivatives as in Iskrev (2010) (only for order=1)
     % -1: numerical two-sided finite difference method to compute numerical derivatives of all identification Jacobians using function identification_numerical_objective.m (previously thet2tau.m)
     % -2: numerical two-sided finite difference method to compute numerically dYss, dg1, dg2, dg3, d2Yss and d2g1, the identification Jacobians are then computed analytically as with 0
+
+if options_.discretionary_policy || options_.ramsey_policy
+    if options_ident.analytic_derivation_mode~=-1
+        fprintf('dynare_identification: discretionary_policy and ramsey_policy require analytic_derivation_mode=-1. Resetting the option.')
+        options_ident.analytic_derivation_mode=-1;
+    end
+end
+    
 options_.analytic_derivation_mode = options_ident.analytic_derivation_mode; %overwrite setting in options_
 
 % initialize persistent variables in prior_draw
@@ -454,7 +481,7 @@ if iload <=0
     end
     options_ident.tittxt = parameters; %title text for graphs and figures
     % perform identification analysis for single point
-    [ide_moments_point, ide_spectrum_point, ide_minimal_point, ide_hess_point, ide_reducedform_point, ide_dynamic_point, derivatives_info_point, info, options_ident] = ...
+    [ide_moments_point, ide_spectrum_point, ide_minimal_point, ide_hess_point, ide_reducedform_point, ide_dynamic_point, derivatives_info_point, info] = ...
         identification_analysis(params, indpmodel, indpstderr, indpcorr, options_ident, dataset_info, prior_exist, 1); %the 1 at the end implies initialization of persistent variables
     if info(1)~=0
         % there are errors in the solution algorithm
@@ -471,7 +498,7 @@ if iload <=0
                 params = prior_draw();
                 options_ident.tittxt = 'Random_prior_params'; %title text for graphs and figures
                 % perform identification analysis
-                [ide_moments_point, ide_spectrum_point, ide_minimal_point, ide_hess_point, ide_reducedform_point, ide_dynamic_point, derivatives_info_point, info, options_ident] = ...
+                [ide_moments_point, ide_spectrum_point, ide_minimal_point, ide_hess_point, ide_reducedform_point, ide_dynamic_point, derivatives_info_point, info] = ...
                     identification_analysis(params, indpmodel, indpstderr, indpcorr, options_ident, dataset_info, prior_exist, 1);
             end
         end
@@ -525,7 +552,7 @@ if iload <=0
         end
         options_ident.tittxt = []; % clear title text for graphs and figures
         % run identification analysis
-        [ide_moments, ide_spectrum, ide_minimal, ide_hess, ide_reducedform, ide_dynamic, ide_derivatives_info, info, options_MC] = ...
+        [ide_moments, ide_spectrum, ide_minimal, ide_hess, ide_reducedform, ide_dynamic, ide_derivatives_info, info, error_indicator] = ...
             identification_analysis(params, indpmodel, indpstderr, indpcorr, options_MC, dataset_info, prior_exist, 0); % the 0 implies that we do not initialize persistent variables anymore
 
         if iteration==0 && info(1)==0 % preallocate storage in the first admissable run
@@ -537,7 +564,7 @@ if iload <=0
             STO_si_dDYNAMIC          = zeros([size(ide_dynamic.si_dDYNAMIC, 1), modparam_nbr, MAX_RUNS_BEFORE_SAVE_TO_FILE]);
             STO_DYNAMIC              = zeros(size(ide_dynamic.DYNAMIC, 1), SampleSize);
             IDE_DYNAMIC.ind_dDYNAMIC = ide_dynamic.ind_dDYNAMIC;
-            IDE_DYNAMIC.in0          = zeros(SampleSize, modparam_nbr);
+            IDE_DYNAMIC.ino          = zeros(SampleSize, modparam_nbr);
             IDE_DYNAMIC.ind0         = zeros(SampleSize, modparam_nbr);
             IDE_DYNAMIC.jweak        = zeros(SampleSize, modparam_nbr);
             IDE_DYNAMIC.jweak_pair   = zeros(SampleSize, modparam_nbr*(modparam_nbr+1)/2);
@@ -549,7 +576,7 @@ if iload <=0
                 STO_si_dREDUCEDFORM              = zeros([size(ide_reducedform.si_dREDUCEDFORM, 1), totparam_nbr, MAX_RUNS_BEFORE_SAVE_TO_FILE]);
                 STO_REDUCEDFORM                  = zeros(size(ide_reducedform.REDUCEDFORM, 1), SampleSize);
                 IDE_REDUCEDFORM.ind_dREDUCEDFORM = ide_reducedform.ind_dREDUCEDFORM;
-                IDE_REDUCEDFORM.in0              = zeros(SampleSize, 1);
+                IDE_REDUCEDFORM.ino              = zeros(SampleSize, 1);
                 IDE_REDUCEDFORM.ind0             = zeros(SampleSize, totparam_nbr);
                 IDE_REDUCEDFORM.jweak            = zeros(SampleSize, totparam_nbr);
                 IDE_REDUCEDFORM.jweak_pair       = zeros(SampleSize, totparam_nbr*(totparam_nbr+1)/2);
@@ -566,7 +593,7 @@ if iload <=0
                 STO_si_dMOMENTS          = zeros([size(ide_moments.si_dMOMENTS, 1), totparam_nbr, MAX_RUNS_BEFORE_SAVE_TO_FILE]);
                 STO_MOMENTS              = zeros(size(ide_moments.MOMENTS, 1), SampleSize);
                 IDE_MOMENTS.ind_dMOMENTS = ide_moments.ind_dMOMENTS;
-                IDE_MOMENTS.in0          = zeros(SampleSize, 1);
+                IDE_MOMENTS.ino          = zeros(SampleSize, 1);
                 IDE_MOMENTS.ind0         = zeros(SampleSize, totparam_nbr);
                 IDE_MOMENTS.jweak        = zeros(SampleSize, totparam_nbr);
                 IDE_MOMENTS.jweak_pair   = zeros(SampleSize, totparam_nbr*(totparam_nbr+1)/2);
@@ -584,7 +611,7 @@ if iload <=0
             if ~options_MC.no_identification_spectrum
                 STO_dSPECTRUM              = zeros([size(ide_spectrum.dSPECTRUM, 1), size(ide_spectrum.dSPECTRUM, 2), MAX_RUNS_BEFORE_SAVE_TO_FILE]);
                 IDE_SPECTRUM.ind_dSPECTRUM = ide_spectrum.ind_dSPECTRUM;
-                IDE_SPECTRUM.in0           = zeros(SampleSize, 1);
+                IDE_SPECTRUM.ino           = zeros(SampleSize, 1);
                 IDE_SPECTRUM.ind0          = zeros(SampleSize, totparam_nbr);
                 IDE_SPECTRUM.jweak         = zeros(SampleSize, totparam_nbr);
                 IDE_SPECTRUM.jweak_pair    = zeros(SampleSize, totparam_nbr*(totparam_nbr+1)/2);
@@ -599,12 +626,13 @@ if iload <=0
             if ~options_MC.no_identification_minimal
                 STO_dMINIMAL             = zeros([size(ide_minimal.dMINIMAL, 1), size(ide_minimal.dMINIMAL, 2), MAX_RUNS_BEFORE_SAVE_TO_FILE]);
                 IDE_MINIMAL.ind_dMINIMAL = ide_minimal.ind_dMINIMAL;
-                IDE_MINIMAL.in0          = zeros(SampleSize, 1);
+                IDE_MINIMAL.ino          = zeros(SampleSize, 1);
                 IDE_MINIMAL.ind0         = zeros(SampleSize, totparam_nbr);
                 IDE_MINIMAL.jweak        = zeros(SampleSize, totparam_nbr);
                 IDE_MINIMAL.jweak_pair   = zeros(SampleSize, totparam_nbr*(totparam_nbr+1)/2);
                 IDE_MINIMAL.cond         = zeros(SampleSize, 1);
                 IDE_MINIMAL.Mco          = zeros(SampleSize, totparam_nbr);
+                IDE_MINIMAL.minimal_state_space = zeros(SampleSize, 1);
             else
                 STO_dMINIMAL = {};
                 IDE_MINIMAL  = {};
@@ -627,7 +655,7 @@ if iload <=0
             IDE_DYNAMIC.Mco(iteration,:)        = ide_dynamic.Mco;
 
             % store results for reduced form
-            if ~options_MC.no_identification_reducedform
+            if ~options_MC.no_identification_reducedform && ~error_indicator.identification_reducedform
                 STO_REDUCEDFORM(:,iteration)            = ide_reducedform.REDUCEDFORM;
                 STO_si_dREDUCEDFORM(:,:,run_index)      = ide_reducedform.si_dREDUCEDFORM;
                 IDE_REDUCEDFORM.cond(iteration,1)       = ide_reducedform.cond;
@@ -639,7 +667,7 @@ if iload <=0
             end
 
             % store results for moments
-            if ~options_MC.no_identification_moments
+            if ~options_MC.no_identification_moments && ~error_indicator.identification_moments
                 STO_MOMENTS(:,iteration)            = ide_moments.MOMENTS;
                 STO_si_dMOMENTS(:,:,run_index)      = ide_moments.si_dMOMENTS;
                 IDE_MOMENTS.cond(iteration,1)       = ide_moments.cond;
@@ -653,7 +681,7 @@ if iload <=0
             end
 
             % store results for spectrum
-            if ~options_MC.no_identification_spectrum
+            if ~options_MC.no_identification_spectrum && ~error_indicator.identification_spectrum
                 STO_dSPECTRUM(:,:,run_index)         = ide_spectrum.dSPECTRUM;
                 IDE_SPECTRUM.cond(iteration,1)       = ide_spectrum.cond;
                 IDE_SPECTRUM.ino(iteration,1)        = ide_spectrum.ino;
@@ -664,14 +692,19 @@ if iload <=0
             end
 
             % store results for minimal system
-            if ~options_MC.no_identification_minimal
-                STO_dMINIMAL(:,:,run_index)         = ide_minimal.dMINIMAL;
-                IDE_MINIMAL.cond(iteration,1)       = ide_minimal.cond;
-                IDE_MINIMAL.ino(iteration,1)        = ide_minimal.ino;
-                IDE_MINIMAL.ind0(iteration,:)       = ide_minimal.ind0;
-                IDE_MINIMAL.jweak(iteration,:)      = ide_minimal.jweak;
-                IDE_MINIMAL.jweak_pair(iteration,:) = ide_minimal.jweak_pair;
-                IDE_MINIMAL.Mco(iteration,:)        = ide_minimal.Mco;
+            if ~options_MC.no_identification_minimal 
+                if ~error_indicator.identification_minimal
+                    STO_dMINIMAL(:,:,run_index)                  = ide_minimal.dMINIMAL;
+                    IDE_MINIMAL.cond(iteration,1)                = ide_minimal.cond;
+                    IDE_MINIMAL.ino(iteration,1)                 = ide_minimal.ino;
+                    IDE_MINIMAL.ind0(iteration,:)                = ide_minimal.ind0;
+                    IDE_MINIMAL.jweak(iteration,:)               = ide_minimal.jweak;
+                    IDE_MINIMAL.jweak_pair(iteration,:)          = ide_minimal.jweak_pair;
+                    IDE_MINIMAL.Mco(iteration,:)                 = ide_minimal.Mco;
+                    IDE_MINIMAL.minimal_state_space(iteration,:) = ide_minimal.minimal_state_space;
+                else
+                    IDE_MINIMAL.minimal_state_space(iteration,:) = ide_minimal.minimal_state_space;
+                end
             end
 
             % save results to file: either to avoid running into memory issues, i.e. (run_index==MAX_RUNS_BEFORE_SAVE_TO_FILE) or if finished (iteration==SampleSize)
@@ -873,7 +906,7 @@ if SampleSize > 1
                 fprintf('\nTesting %s.\n',tittxt);
                 if ~iload
                     options_ident.tittxt = tittxt; %title text for graphs and figures
-                    [ide_moments_max, ide_spectrum_max, ide_minimal_max, ide_hess_max, ide_reducedform_max, ide_dynamic_max, derivatives_info_max, info_max, options_ident] = ...
+                    [ide_moments_max, ide_spectrum_max, ide_minimal_max, ide_hess_max, ide_reducedform_max, ide_dynamic_max, derivatives_info_max, info_max] = ...
                         identification_analysis(pdraws(jmax,:), indpmodel, indpstderr, indpcorr, options_ident, dataset_info, prior_exist, 1); %the 1 at the end initializes some persistent variables
                     save([IdentifDirectoryName '/' fname '_identif.mat'], 'ide_hess_max', 'ide_moments_max', 'ide_spectrum_max', 'ide_minimal_max','ide_reducedform_max', 'ide_dynamic_max', 'jmax', '-append');
                 end
@@ -891,7 +924,7 @@ if SampleSize > 1
                 fprintf('Testing %s.\n',tittxt);
                 if ~iload
                     options_ident.tittxt = tittxt; %title text for graphs and figures
-                    [ide_moments_min, ide_spectrum_min, ide_minimal_min, ide_hess_min, ide_reducedform_min, ide_dynamic_min, derivatives_info_min, info_min, options_ident] = ...
+                    [ide_moments_min, ide_spectrum_min, ide_minimal_min, ide_hess_min, ide_reducedform_min, ide_dynamic_min, derivatives_info_min, info_min] = ...
                         identification_analysis(pdraws(jmin,:), indpmodel, indpstderr, indpcorr, options_ident, dataset_info, prior_exist, 1); %the 1 at the end initializes persistent variables
                     save([IdentifDirectoryName '/' fname '_identif.mat'], 'ide_hess_min', 'ide_moments_min','ide_spectrum_min','ide_minimal_min','ide_reducedform_min', 'ide_dynamic_min', 'jmin', '-append');
                 end
@@ -913,7 +946,7 @@ if SampleSize > 1
                     fprintf('\nTesting %s.\n',tittxt);
                     if ~iload
                         options_ident.tittxt = tittxt; %title text for graphs and figures
-                        [ide_moments_(j), ide_spectrum_(j), ide_minimal_(j), ide_hess_(j), ide_reducedform_(j), ide_dynamic_(j), derivatives_info_(j), info_resolve, options_ident] = ...
+                        [ide_moments_(j), ide_spectrum_(j), ide_minimal_(j), ide_hess_(j), ide_reducedform_(j), ide_dynamic_(j), derivatives_info_(j), info_resolve] = ...
                             identification_analysis(pdraws(jcrit(j),:), indpmodel, indpstderr, indpcorr, options_ident, dataset_info, prior_exist, 1);
                     end
                     advanced0 = options_ident.advanced; options_ident.advanced = 1; %make sure advanced setting is on
@@ -943,4 +976,4 @@ end
 
 fprintf('\n==== Identification analysis completed ====\n\n')
 
-options_ = store_options_; %restore options set
\ No newline at end of file
+options_ = store_options_; %restore options set
