@@ -31,13 +31,10 @@ function [initial_conditions, innovations, pfm, ep, verbosity, DynareOptions, Dy
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 ep  = DynareOptions.ep;
 
-% Set bytecode flag (see #1717)
-ep.use_bytecode = DynareOptions.bytecode;
-
 % Set verbosity levels.
 DynareOptions.verbosity = ep.verbosity;
 verbosity = ep.verbosity+ep.debug;
@@ -81,7 +78,7 @@ dr = struct();
 if ep.init
     DynareOptions.order = 1;
     DynareResults.dr=set_state_space(dr,DynareModel,DynareOptions);
-    [dr,Info,DynareModel,DynareOptions,DynareResults] = resol(0,DynareModel,DynareOptions,DynareResults);
+    [dr,Info,DynareResults] = resol(0,DynareModel,DynareOptions,DynareResults);
 end
 
 % Do not use a minimal number of perdiods for the perfect foresight solver (with bytecode and blocks)
