@@ -20,7 +20,7 @@ function [residuals,check1,jacob] = evaluate_static_model(ys,exo_ss,params,M,opt
 % SPECIAL REQUIREMENTS
 %   none
 
-% Copyright (C) 2001-2017 Dynare Team
+% Copyright (C) 2001-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -35,31 +35,40 @@ function [residuals,check1,jacob] = evaluate_static_model(ys,exo_ss,params,M,opt
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 check1 = 0;
 if options.bytecode
-    [check1, residuals] = bytecode('evaluate','static',ys,...
-                                   exo_ss, params, ys, 1);
-    mexErrCheck('bytecode', check1);
+    if nargout<3
+        [residuals]= bytecode('evaluate','static',ys,...
+                         exo_ss, params, ys, 1);
+    else
+        [residuals, junk]= bytecode('evaluate','static',ys,...
+            exo_ss, params, ys, 1);
+        jacob = junk.g1;
+    end      
 else
     fh_static = str2func([M.fname '.static']);
     if options.block
         residuals = zeros(M.endo_nbr,1);
+        T = NaN(M.block_structure_stat.tmp_nbr, 1);
         for b = 1:length(M.block_structure_stat.block)
-            mfsb = M.block_structure_stat.block(b).variable;
-            % blocks that can be directly evaluated (mfsb is empty)
-            % have zero residuals by construction
-            if M.block_structure_stat.block(b).Simulation_Type ~= 1 && ...
-                    M.block_structure_stat.block(b).Simulation_Type ~= 2
-                residuals(mfsb) = feval(fh_static,b,ys,exo_ss,params);
-            else
-                %need to evaluate the recursive blocks to compute the
-                %temporary terms
-                feval(fh_static,b,ys,exo_ss,params);
+            [r, yy, T] = feval(fh_static,b,ys,exo_ss,params,T);
+            if M.block_structure_stat.block(b).Simulation_Type == 1 || ... % evaluateForward
+               M.block_structure_stat.block(b).Simulation_Type == 2        % evaluateBackward
+                vidx = M.block_structure_stat.block(b).variable;
+                r = yy(vidx) - ys(vidx);
             end
+            residuals(M.block_structure_stat.block(b).equation) = r;
+        end
+        if nargout==3
+            jacob=NaN(length(ys));
         end
     else
-        residuals = feval(fh_static,ys,exo_ss,params);
+        if nargout<3
+            residuals = feval(fh_static,ys,exo_ss,params);
+        else
+            [residuals, jacob] = feval(fh_static,ys,exo_ss,params);
+        end
     end
 end
