@@ -22,7 +22,7 @@ function [ys,params,info] = evaluate_steady_state(ys_init,M,options,oo,steadysta
 % SPECIAL REQUIREMENTS
 %   none
 
-% Copyright (C) 2001-2018 Dynare Team
+% Copyright (C) 2001-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -37,7 +37,7 @@ function [ys,params,info] = evaluate_steady_state(ys_init,M,options,oo,steadysta
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 info = 0;
 check = 0;
@@ -52,12 +52,21 @@ if length(M.aux_vars) > 0 && ~steadystate_flag && M.set_auxiliary_variables
 end
 
 if options.ramsey_policy
+    if options.block
+        % The current implementation needs the Jacobian of the full model, which is not
+        % provided by the block-decomposed routines.
+        error('The ''block'' option is not compatible with ''ramsey_model''/''ramsey_policy''');
+    end
     if steadystate_flag
         % explicit steady state file
         [ys,params,info] = evaluate_steady_state_file(ys_init,exo_ss,M, ...
                                                       options,steadystate_check_flag);
         %test whether it solves model conditional on the instruments
-        resids = evaluate_static_model(ys,exo_ss,params,M,options);
+        if ~options.debug
+            resids = evaluate_static_model(ys,exo_ss,params,M,options);
+        else
+            [resids, ~ , jacob]= evaluate_static_model(ys,exo_ss,params,M,options);
+        end
         n_multipliers=M.ramsey_eq_nbr;
         nan_indices=find(isnan(resids(n_multipliers+1:end)));
 
@@ -118,20 +127,55 @@ if options.ramsey_policy
         end
     end
     if options.debug
-        infrow=find(isinf(ys_init));
+        if steadystate_flag
+            infrow=find(isinf(ys_init(1:M.orig_endo_nbr)));
+        else
+            infrow=find(isinf(ys_init));
+        end
         if ~isempty(infrow)
             fprintf('\nevaluate_steady_state: The initial values for the steady state of the following variables are Inf:\n');
             for iter=1:length(infrow)
                 fprintf('%s\n',M.endo_names{infrow(iter)});
             end
         end
-        nanrow=find(isnan(ys_init));
+        if steadystate_flag
+            nanrow=find(isnan(ys_init(1:M.orig_endo_nbr)));
+        else
+            nanrow=find(isnan(ys_init));
+        end
         if ~isempty(nanrow)
             fprintf('\nevaluate_steady_state: The initial values for the steady state of the following variables are NaN:\n');
             for iter=1:length(nanrow)
                 fprintf('%s\n',M.endo_names{nanrow(iter)});
             end
         end
+        if steadystate_flag
+            nan_indices_mult=find(isnan(resids(1:n_multipliers)));
+            if any(nan_indices_mult)
+                fprintf('evaluate_steady_state: The steady state results NaN for auxiliary equation %u.\n',nan_indices_mult);
+                fprintf('evaluate_steady_state: This is often a sign of problems.\n');
+            end
+            [infrow,infcol]=find(isinf(jacob));
+            
+            if ~isempty(infrow)
+                fprintf('\nevaluate_steady_state: The Jacobian of the dynamic model contains Inf. The problem is associated with:\n\n')
+                display_problematic_vars_Jacobian(infrow,infcol,M,ys,'static','evaluate_steady_state: ')
+            end
+            
+            if ~isreal(jacob)
+                [imagrow,imagcol]=find(abs(imag(jacob))>1e-15);
+                fprintf('\nevaluate_steady_state: The Jacobian of the dynamic model contains imaginary parts. The problem arises from: \n\n')
+                display_problematic_vars_Jacobian(imagrow,imagcol,M,ys,'static','evaluate_steady_state: ')
+            end
+            
+            [nanrow,nancol]=find(isnan(jacob));
+            if ~isempty(nanrow)
+                fprintf('\nevaluate_steady_state: The Jacobian of the dynamic model contains NaN. The problem is associated with:\n\n')
+                display_problematic_vars_Jacobian(nanrow,nancol,M,ys,'static','evaluate_steady_state: ')
+            end
+            
+        end
+
     end
     %either if no steady state file or steady state file without problems
     [ys,params,info] = dyn_ramsey_static(ys_init,M,options,oo);
@@ -273,7 +317,7 @@ elseif ~options.bytecode && ~options.block
                 [infrow,infcol]=find(isinf(jacob) | isnan(jacob));
                 fprintf('\nSTEADY:  The Jacobian contains Inf or NaN. The problem arises from: \n\n')
                 for ii=1:length(infrow)
-                    fprintf('STEADY:  Derivative of Equation %d with respect to Variable %s  (initial value of %s: %g) \n',infrow(ii),deblank(M.endo_names(infcol(ii),:)),deblank(M.endo_names(infcol(ii),:)),ys_init(infcol(ii)))
+                    fprintf('STEADY:  Derivative of Equation %d with respect to Variable %s  (initial value of %s: %g) \n',infrow(ii),M.endo_names{infcol(ii),:},M.endo_names{infcol(ii),:},ys_init(infcol(ii)))
                 end
                 fprintf('Check whether your model is truly linear. Put "resid(1);" before "steady;" to see the problematic equations.\n')
             end
@@ -307,15 +351,24 @@ if M.static_and_dynamic_models_differ
     z = repmat(ys,1,M.maximum_lead + M.maximum_lag + 1);
     zx = repmat([exo_ss'], M.maximum_lead + M.maximum_lag + 1, 1);
     if options.bytecode
-        [chck, r, ~]= bytecode('dynamic','evaluate', z, zx, params, ys, 1);
-        mexErrCheck('bytecode', chck);
+        [r, ~]= bytecode('dynamic','evaluate', z, zx, params, ys, 1);
     elseif options.block
-        [r, oo.dr] = feval([M.fname '.dynamic'], z', zx, params, ys, M.maximum_lag+1, oo.dr);
+        T=NaN(M.block_structure.dyn_tmp_nbr, 1);
+        for i = 1:length(M.block_structure.block)
+            [rr, yy, T, g] = feval([M.fname '.dynamic'], i, ...
+                                   dynvars_from_endo_simul(z, M.maximum_lag+1, M), ...
+                                   zx, params, ys, T, M.maximum_lag+1, false);
+            if M.block_structure.block(i).Simulation_Type == 1 || ... % evaluateForward
+               M.block_structure.block(i).Simulation_Type == 2        % evaluateBackward
+                vidx = M.block_structure.block(i).variable;
+                rr = yy(M.lead_lag_incidence(M.maximum_endo_lag+1, vidx)) - oo.steady_state(vidx);
+            end
+            idx = M.block_structure.block(i).equation;
+            r(idx) = rr;
+        end
     else
-        iyv = M.lead_lag_incidence';
-        iyr0 = find(iyv(:));
-        xys = z(iyr0);
-        r = feval([M.fname '.dynamic'], z(iyr0), zx, params, ys, M.maximum_lag + 1);
+        r = feval([M.fname '.dynamic'], dynvars_from_endo_simul(z, M.maximum_lag+1, M), ...
+                  zx, params, ys, M.maximum_lag + 1);
     end
     % Fail if residual greater than tolerance
     if max(abs(r)) > options.solve_tolf
