@@ -37,7 +37,7 @@ function [yf,int_width,int_width_ME]=forcst(dr,y0,horizon,var_list,M_,oo_,option
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 oo_=make_ex_(M_,options_,oo_);
 yf = simult_(M_,options_,y0,dr,zeros(horizon,M_.exo_nbr),1);
@@ -85,7 +85,7 @@ for i = 1:horizon
 end
 if nargout==3
     var_yf_ME=var_yf;
-    [loc_H, loc_varlist] = ismember(options_.varobs', options_.varlist);
+    [loc_H, loc_varlist] = ismember(options_.varobs, var_list);
     loc_varlist(loc_varlist==0) = [];
     if ~isempty(loc_varlist)
         var_yf_ME(:,loc_varlist) = var_yf(:,loc_varlist)+repmat(diag(M_.H(loc_H,loc_H))', horizon, 1);
