@@ -25,7 +25,7 @@ function message = get_error_message(info, DynareOptions)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 switch info(1)
     case 0
@@ -122,7 +122,7 @@ switch info(1)
     case 64
         message = 'discretionary_policy: the derivatives of the objective function contain NaN.';
     case 65
-        message = 'discretionary_policy: the model must be written in deviation form and not have constant terms.';
+        message = 'discretionary_policy: the model must be written in deviation form and not have constant terms or an analytical steady state meeds to be provided.';
     case 66
         message = 'discretionary_policy: the objective function must have zero first order derivatives.';
     case 71
@@ -166,6 +166,28 @@ switch info(1)
         message = 'Aim: A is NAN or INF.';
     case 164
         message = 'Aim: Problem in SPEIG.';
+    case 180
+        message = 'SMM: simulation resulted in NaN/Inf. You may need to enable pruning.';
+    case 201
+        message = 'Particle Filter: Initial covariance of the states is not positive definite. Try a different nonlinear_filter_initialization';
+    case 202
+        message = 'Particle Filter: Initial covariance of the states based on simulation resulted in NaN/Inf. Use pruning or try a different nonlinear_filter_initialization';
+    case 301
+        message = 'IVF: The likelihood is Inf.';
+    case 302
+        message = 'IVF: The likelihood is NaN.';
+    case 303
+        message = 'IVF: The residuals are not 0.';        
+    case 304
+        message = 'IVF: The solver returned with an error code.';        
+    case 305
+        message = 'IVF: The returned shocks are bigger than 1e8.';        
+    case 310
+        message = 'Occbin: Simulation terminated with periodic solution (no convergence).';        
+    case 311
+        message = 'Occbin: Simulation did not converge, increase maxit or check_ahead_periods.';        
+    case 312
+        message = 'Occbin: Constraint(s) are binding at the end of the sample.';        
     otherwise
         message = 'This case shouldn''t happen. Contact the authors of Dynare';
 end
\ No newline at end of file
