@@ -50,7 +50,7 @@ function x0 = stab_map_(OutputDirectoryName,opt_gsa)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 %global bayestopt_ estim_params_ dr_ options_ ys_ fname_
 global bayestopt_ estim_params_ options_ oo_ M_
@@ -310,9 +310,9 @@ if fload==0
         %try stoch_simul([]);
         try
             if ~ isempty(options_.endogenous_prior_restrictions.moment)
-                [Tt,Rr,SteadyState,info,M_,options_,oo_] = dynare_resolve(M_,options_,oo_);
+                [Tt,Rr,SteadyState,info,M_,oo_] = dynare_resolve(M_,options_,oo_);
             else
-                [Tt,Rr,SteadyState,info,M_,options_,oo_] = dynare_resolve(M_,options_,oo_,'restrict');
+                [Tt,Rr,SteadyState,info,M_,oo_] = dynare_resolve(M_,options_,oo_,'restrict');
             end
             infox(j,1)=info(1);
             if infox(j,1)==0 && ~exist('T','var')
@@ -681,7 +681,7 @@ if length(iunstable)>0 || length(iwrong)>0
         end
 
         M_ = set_all_parameters(x0,estim_params_,M_);
-        [oo_.dr,info,M_,options_,oo_] = resol(0,M_,options_,oo_);
+        [oo_.dr,info,M_,oo_] = resol(0,M_,options_,oo_);
         %     stoch_simul([]);
     else
         disp('All parameter values in the specified ranges are not acceptable!')
@@ -696,7 +696,7 @@ else
 end
 
 xparam1=x0;
-save prior_ok.mat xparam1;
+save([OutputDirectoryName filesep 'prior_ok.mat'],'xparam1');
 
 options_.periods=opt.periods;
 if isfield(opt,'nomoments')
