@@ -35,13 +35,30 @@ function DynareResults = initial_estimation_checks(objective_function,xparam1,Dy
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 %get maximum number of simultaneously observed variables for stochastic
 %singularity check
-maximum_number_non_missing_observations=max(sum(~isnan(DynareDataset.data),2));
+maximum_number_non_missing_observations=max(sum(~isnan(DynareDataset.data(2:end,:)),2));
+init_number_non_missing_observations=sum(~isnan(DynareDataset.data(1,:)),2);
 
-if DynareOptions.order>1 
+if DynareOptions.heteroskedastic_filter
+    if DynareOptions.order>1 
+        error('initial_estimation_checks:: heteroskedastic shocks are only supported with the Kalman filter/smoother')
+    end
+    observations_by_period=sum(~isnan(DynareDataset.data),2);
+    base_shocks = find(diag(Model.Sigma_e));
+    zero_shocks = ~ismember(1:Model.exo_nbr,base_shocks);
+    non_zero_shocks_by_period=repmat(length(base_shocks),size(observations_by_period));
+    % check periods for which base shocks are scaled to zero
+    non_zero_shocks_by_period = non_zero_shocks_by_period-sum(Model.heteroskedastic_shocks.Qscale(base_shocks,1:DynareOptions.nobs)==0,1)';
+    % check periods for which base shocks are set to zero
+    non_zero_shocks_by_period = non_zero_shocks_by_period-sum(Model.heteroskedastic_shocks.Qvalue(base_shocks,1:DynareOptions.nobs)==0,1)';
+    % check periods for which other shocks are set to a positive number
+    non_zero_shocks_by_period = non_zero_shocks_by_period+sum(Model.heteroskedastic_shocks.Qvalue(zero_shocks,1:DynareOptions.nobs)>0,1)';
+end
+
+if DynareOptions.order>1
     if any(any(isnan(DynareDataset.data)))
         error('initial_estimation_checks:: particle filtering does not support missing observations')
     end
@@ -51,16 +68,81 @@ if DynareOptions.order>1
     if BayesInfo.with_trend
         error('initial_estimation_checks:: particle filtering does not support trends')
     end
+    if DynareOptions.order>2 && DynareOptions.particle.pruning==1
+        error('initial_estimation_checks:: the particle filter with order>2 does not support pruning')
+    end
+    if DynareOptions.particle.pruning~=DynareOptions.pruning
+        warning('initial_estimation_checks:: the pruning settings differ between the particle filter and the one used for IRFs/simulations. Make sure this is intended.')
+    end
+end
+
+if (DynareOptions.occbin.likelihood.status && DynareOptions.occbin.likelihood.inversion_filter) || (DynareOptions.occbin.smoother.status && DynareOptions.occbin.smoother.inversion_filter)
+    err_index= find(diag(Model.Sigma_e)~=0);
+    if length(err_index)~=length(DynareOptions.varobs)
+        fprintf('initial_estimation_checks:: The IVF requires exactly as many shocks as observables.')
+    end
+    var_index=find(any(isnan(DynareDataset.data)));
+    if ~isempty(var_index)
+        fprintf('initial_estimation_checks:: The IVF requires exactly as many shocks as observables.\n')
+        fprintf('initial_estimation_checks:: The data series %s contains NaN, I am therefore dropping shock %s for these time points.\n',...
+            DynareOptions.varobs{var_index},Model.exo_names{DynareOptions.occbin.likelihood.IVF_shock_observable_mapping(var_index)})
+    end
+end
+
+if DynareOptions.order>1 || (DynareOptions.order==1 && ~ischar(DynareOptions.mode_compute) && DynareOptions.mode_compute==11)
+    if DynareOptions.order==1 && DynareOptions.mode_compute==11
+        disp_string='mode_compute=11';
+    else
+        disp_string='particle filtering';
+    end
+    if Model.H==0
+        error('initial_estimation_checks:: %s requires measurement error on the observables',disp_string)
+    else
+        if sum(diag(Model.H)>0)<length(DynareOptions.varobs)
+            error('initial_estimation_checks:: %s requires as many measurement errors as observed variables',disp_string)
+        else
+            [~,flag]=chol(Model.H);
+            if flag
+                error('initial_estimation_checks:: the measurement error matrix must be positive definite')
+            end
+        end
+    end
 end
 
 non_zero_ME=length(EstimatedParameters.H_entries_to_check_for_positive_definiteness);
 
+print_init_check_warning=false;
 if maximum_number_non_missing_observations>Model.exo_nbr+non_zero_ME
     error(['initial_estimation_checks:: Estimation can''t take place because there are less declared shocks than observed variables!'])
 end
+if init_number_non_missing_observations>Model.exo_nbr+non_zero_ME
+    if DynareOptions.no_init_estimation_check_first_obs
+        print_init_check_warning=true;
+    else
+        error(['initial_estimation_checks:: Estimation can''t take place because there are less declared shocks than observed variables in first period!'])
+    end
+end
 
-if maximum_number_non_missing_observations>length(find(diag(Model.Sigma_e)))+non_zero_ME
-    error(['initial_estimation_checks:: Estimation can''t take place because too many shocks have been calibrated with a zero variance!'])
+if DynareOptions.heteroskedastic_filter
+    if any(observations_by_period>(non_zero_shocks_by_period+non_zero_ME))
+        error(['initial_estimation_checks:: Estimation can''t take place because too many shocks have been calibrated with a zero variance: Check heteroskedastic block and shocks calibration!'])
+    end
+else
+    if maximum_number_non_missing_observations>length(find(diag(Model.Sigma_e)))+non_zero_ME
+        error(['initial_estimation_checks:: Estimation can''t take place because too many shocks have been calibrated with a zero variance!'])
+    end
+end
+if init_number_non_missing_observations>length(find(diag(Model.Sigma_e)))+non_zero_ME
+    if DynareOptions.no_init_estimation_check_first_obs
+        print_init_check_warning=true;
+    else
+        error(['initial_estimation_checks:: Estimation can''t take place because too many shocks have been calibrated with a zero variance in first period!'])
+    end
+end
+if print_init_check_warning
+    fprintf('ESTIMATION_CHECKS: You decided to ignore test of stochastic singularity in first_obs.\n');
+    fprintf('ESTIMATION_CHECKS: If this was not done on purpose (typically when observing a stock variable [capital] in first period, on top of its flow [investment]),\n');
+    fprintf('ESTIMATION_CHECKS: it may lead to a crash or provide undesired/wrong results later on!\n');
 end
 
 if (any(BayesInfo.pshape  >0 ) && DynareOptions.mh_replic) && DynareOptions.mh_nblck<1
@@ -121,6 +203,19 @@ if any(info)
     error('The initial value of the prior is -Inf')
 end
 
+if isfield(Model,'filter_initial_state') && ~isempty(Model.filter_initial_state)
+    state_indices=DynareResults.dr.order_var(DynareResults.dr.restrict_var_list(BayesInfo.mf0));
+    for ii=1:size(state_indices,1)
+        if ~isempty(Model.filter_initial_state{state_indices(ii),1})
+            try
+                evaluate_expression(Model.filter_initial_state{state_indices(ii),2},Model,DynareResults)
+            catch
+                fprintf('Unable to evaluate the expression\n %s \nfor the filter_initial_state of variable %s\n',Model.filter_initial_state{state_indices(ii),2},Model.endo_names(state_indices(ii),:))
+            end
+        end
+    end
+end
+
 if DynareOptions.ramsey_policy
     %test whether specification matches
     inst_nbr = size(DynareOptions.instruments,1);
@@ -135,6 +230,10 @@ if DynareOptions.ramsey_policy
     end
 end
 
+if ~isreal(DynareDataset.data)
+    error('initial_estimation_checks: the data contains complex values.')
+end
+
 % Evaluate the likelihood.
 ana_deriv = DynareOptions.analytic_derivation;
 DynareOptions.analytic_derivation=0;
@@ -152,6 +251,10 @@ if ~isequal(DynareOptions.mode_compute,11) || ...
         fprintf('initial_estimation_checks:: If you think the latter is the case, you should try with different initial values for the estimated parameters.\n')
         error('initial_estimation_checks:: The forecast error variance in the multivariate Kalman filter became singular.')
     end
+    if info(1)==201 || info(1)==202
+        message=get_error_message(info,DynareOptions);
+        error('initial_estimation_checks:: %s.',message)
+    end
     %reset options
     DynareOptions.use_univariate_filters_if_singularity_is_detected=use_univariate_filters_if_singularity_is_detected_old;
 else
@@ -163,6 +266,11 @@ if DynareOptions.debug
 end
 DynareOptions.analytic_derivation=ana_deriv;
 
+if DynareOptions.mode_compute==13
+    error('Options mode_compute=13 is only compatible with quadratic objective functions')
+end
+
+
 % if DynareOptions.mode_compute==5
 %     if ~strcmp(func2str(objective_function),'dsge_likelihood')
 %         error('Options mode_compute=5 is not compatible with non linear filters or Dsge-VAR models!')
@@ -197,3 +305,16 @@ end
 if ~isequal(DynareOptions.mode_compute,11)
     disp(['Initial value of the log posterior (or likelihood): ' num2str(-fval)]);
 end
+
+if DynareOptions.mh_tune_jscale.status && (DynareOptions.mh_tune_jscale.maxiter<DynareOptions.mh_tune_jscale.stepsize)
+    warning('You specified mh_tune_jscale, but the maximum number of iterations is smaller than the step size. No update will take place.')
+end
+
+if ~isempty(DynareOptions.conditional_variance_decomposition) && ~DynareOptions.moments_varendo
+    disp('The conditional_variance_decomposition-option will be ignored. You need to set moments_varendo');
+end
+
+function evaluate_expression(expression,M_,oo_)
+% function evaluate_expression(expression,M_,oo_)
+%evaluates expressions relying on M_ and oo_ having their original names
+eval(expression);
\ No newline at end of file
