@@ -13,7 +13,7 @@ function d=jacob_element(func,element,args)
 % SPECIAL REQUIREMENTS
 %    none
 
-% Copyright (C) 2010-2011 Dynare Team
+% Copyright (C) 2010-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -28,19 +28,17 @@ function d=jacob_element(func,element,args)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
+
+assert(element <= length(args));
 
 func = str2func(func);
 
-h=10e-6;
-pargs=args;
+h=1e-6;
 margs=args;
-% length(args) is used instead of size(args, 2) to avoid to transpose column vectors
-for i=1:length(args)
-    if i==element
-        pargs{i} = pargs{i} + h;
-        margs{i} = margs{i} - h;
-    end
+
+args{element} = args{element} + h;
+margs{element} = margs{element} - h;
+
+d=(func(args{:})-func(margs{:}))/(2*h);
 end
-d=(func(pargs{:})...
-   -func(margs{:}))/(2*h);
