@@ -1,7 +1,7 @@
 function [dr,info] = k_order_pert(dr,M,options)
 % Compute decision rules using the k-order DLL from Dynare++
 
-% Copyright (C) 2009-2018 Dynare Team
+% Copyright (C) 2009-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -16,7 +16,7 @@ function [dr,info] = k_order_pert(dr,M,options)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 info = 0;
 
@@ -30,8 +30,9 @@ if M.maximum_endo_lead == 0 && order>1
            'backward models'])
 end
 
-[err, dynpp_derivs, dyn_derivs] = k_order_perturbation(dr,M,options);
-if err
+try
+    [dynpp_derivs, dyn_derivs] = k_order_perturbation(dr,M,options);
+catch
     info(1)=9;
     return
 end
