@@ -12,7 +12,7 @@ function [dLIK,dlik,a,Pstar] = missing_observations_kalman_filter_d(data_index,n
 %    Y                            [double]      pp*smpl matrix of (detrended) data, where pp is the number of observed variables.
 %    start                        [integer]     scalar, first observation.
 %    last                         [integer]     scalar, last observation.
-%    a                            [double]      mm*1 vector, levels of the state variables.
+%    a                            [double]      mm*1 vector, levels of the predicted initial state variables (E_{0}(alpha_1)).
 %    Pinf                         [double]      mm*mm matrix used to initialize the covariance matrix of the state vector.
 %    Pstar                        [double]      mm*mm matrix used to initialize the covariance matrix of the state vector.
 %    kalman_tol                   [double]      scalar, tolerance parameter (rcond).
@@ -30,7 +30,7 @@ function [dLIK,dlik,a,Pstar] = missing_observations_kalman_filter_d(data_index,n
 % OUTPUTS
 %    dLIK        [double]    scalar, MINUS loglikelihood
 %    dlik        [double]    vector, density of observations in each period.
-%    a           [double]    mm*1 vector, estimated level of the states.
+%    a           [double]    mm*1 vector, current estimate of the state vector tomorrow (E_{T}(alpha_{T+1})).
 %    Pstar       [double]    mm*mm matrix, covariance matrix of the states.
 %
 % REFERENCES
@@ -42,7 +42,7 @@ function [dLIK,dlik,a,Pstar] = missing_observations_kalman_filter_d(data_index,n
 %   Second Edition, Ch. 5 and 7.2
 
 %
-% Copyright (C) 2004-2017 Dynare Team
+% Copyright (C) 2004-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -57,7 +57,7 @@ function [dLIK,dlik,a,Pstar] = missing_observations_kalman_filter_d(data_index,n
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 
 % Get sample size.
@@ -65,6 +65,12 @@ smpl = last-start+1;
 
 % Initialize some variables.
 dF   = 1;
+isqvec = false;
+if ndim(Q)>2
+    Qvec = Q;
+    Q=Q(:,:,1);
+    isqvec = true;
+end
 QQ   = R*Q*transpose(R);   % Variance of R times the vector of structural innovations.
 t    = start;              % Initialization of the time index.
 dlik = zeros(smpl,1);      % Initialization of the vector gathering the densities.
@@ -79,6 +85,9 @@ s = 0;
 while rank(Pinf,diffuse_kalman_tol) && (t<=last)
     s = t-start+1;
     d_index = data_index{t};
+    if isqvec
+        QQ = R*Qvec(:,:,t+1)*transpose(R);
+    end
     if isempty(d_index)
         %no observations, propagate forward without updating based on
         %observations
