@@ -35,7 +35,7 @@ function oo = model_comparison(ModelNames,ModelPriors,oo,options_,fname)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 NumberOfModels = size(ModelNames,2);
 
@@ -81,9 +81,9 @@ for i=1:NumberOfModels
         mstruct.oo_ = oo;
     else
         if strcmpi(ModelNames{i}(end-3:end),'.mod') || strcmpi(ModelNames{i}(end-3:end),'.dyn')
-            mstruct = load([ModelNames{i}(1:end-4) '_results.mat' ],'oo_');
+            mstruct = load([ModelNames{i}(1:end-4) filesep 'Output' ModelNames{i}(1:end-4) '_results.mat' ],'oo_');
         else
-            mstruct = load([ModelNames{i} '_results.mat' ],'oo_');
+            mstruct = load([ModelNames{i} filesep 'Output' filesep ModelNames{i} '_results.mat' ],'oo_');
         end
     end
     try
