@@ -16,7 +16,7 @@ function model_diagnostics(M,options,oo)
 %   none.
 %
 
-% Copyright (C) 1996-2018 Dynare Team
+% Copyright (C) 1996-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -31,7 +31,7 @@ function model_diagnostics(M,options,oo)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global jacob
 
@@ -40,6 +40,24 @@ endo_names = M.endo_names;
 lead_lag_incidence = M.lead_lag_incidence;
 maximum_endo_lag = M.maximum_endo_lag;
 
+if options.ramsey_policy
+    %test whether specification matches
+    inst_nbr = size(options.instruments,1);
+    if inst_nbr~=0
+        orig_endo_aux_nbr = M.orig_endo_nbr + min(find([M.aux_vars.type] == 6)) - 1;
+        implied_inst_nbr = orig_endo_aux_nbr - M.orig_eq_nbr;
+        if inst_nbr>implied_inst_nbr
+            error('You have specified more instruments than there are omitted equations')
+        elseif inst_nbr<implied_inst_nbr
+            error('You have specified fewer instruments than there are omitted equations')
+        end
+    else
+        if options.steadystate_flag
+            error('You have specified a steady state file, but not provided an instrument. Either delete the steady state file or provide an instrument')
+        end
+    end
+end
+
 problem_dummy=0;
 %
 % missing variables at the current period
@@ -71,6 +89,11 @@ end
 
 % check if ys is steady state
 options.debug=1; %locally set debug option to 1
+if options.logged_steady_state %if steady state was previously logged, undo this
+    oo.dr.ys=exp(oo.dr.ys);
+    oo.steady_state=exp(oo.steady_state);
+    options.logged_steady_state=0;
+end
 [dr.ys,M.params,check1]=evaluate_steady_state(oo.steady_state,M,options,oo,options.steadystate.nocheck);
 
 % testing for problem
@@ -107,11 +130,11 @@ exo = [oo.exo_steady_state; oo.exo_det_steady_state];
 for b=1:nb
     if options.bytecode
         if nb == 1
-            [chk, res, jacob] = bytecode(dr.ys, exo, M.params, dr.ys, 1, exo, ...
-                                         'evaluate', 'static');
+            [res, jacob] = bytecode(dr.ys, exo, M.params, dr.ys, 1, exo, ...
+                                    'evaluate', 'static');
         else
-            [chk, res, jacob] = bytecode(dr.ys, exo, M.params, dr.ys, 1, exo, ...
-                                         'evaluate', 'static',['block=' ...
+            [res, jacob] = bytecode(dr.ys, exo, M.params, dr.ys, 1, exo, ...
+                                    'evaluate', 'static',['block=' ...
                                 int2str(b)]);
         end
     else
@@ -204,8 +227,8 @@ z = repmat(dr.ys,1,klen);
 if ~options.block
     if options.order == 1
         if (options.bytecode)
-            [chck, ~, loc_dr] = bytecode('dynamic','evaluate', z,exo_simul, ...
-                                         M.params, dr.ys, 1);
+            [~, loc_dr] = bytecode('dynamic','evaluate', z,exo_simul, ...
+                                   M.params, dr.ys, 1);
             jacobia_ = [loc_dr.g1 loc_dr.g1_x loc_dr.g1_xd];
         else
             [~,jacobia_] = feval([M.fname '.dynamic'],z(iyr0),exo_simul, ...
@@ -213,19 +236,14 @@ if ~options.block
         end
     elseif options.order >= 2
         if (options.bytecode)
-            [chck, ~, loc_dr] = bytecode('dynamic','evaluate', z,exo_simul, ...
-                                         M.params, dr.ys, 1);
+            [~, loc_dr] = bytecode('dynamic','evaluate', z,exo_simul, ...
+                                   M.params, dr.ys, 1);
             jacobia_ = [loc_dr.g1 loc_dr.g1_x];
         else
             [~,jacobia_,hessian1] = feval([M.fname '.dynamic'],z(iyr0),...
                                           exo_simul, ...
                                           M.params, dr.ys, it_);
         end
-        if options.use_dll
-            % In USE_DLL mode, the hessian is in the 3-column sparse representation
-            hessian1 = sparse(hessian1(:,1), hessian1(:,2), hessian1(:,3), ...
-                              size(jacobia_, 1), size(jacobia_, 2)*size(jacobia_, 2));
-        end
     end
 
     if any(any(isinf(jacobia_) | isnan(jacobia_)))
@@ -235,10 +253,12 @@ if ~options.block
         display_problematic_vars_Jacobian(infrow,infcol,M,dr.ys,'dynamic','MODEL_DIAGNOSTICS: ')
     end
     if any(any(~isreal(jacobia_)))
-        problem_dummy=1;
         [imagrow,imagcol]=find(abs(imag(jacobia_))>1e-15);
-        fprintf('\nMODEL_DIAGNOSTICS: The Jacobian of the dynamic model contains imaginary parts. The problem arises from: \n\n')
-        display_problematic_vars_Jacobian(imagrow,imagcol,M,dr.ys,'dynamic','MODEL_DIAGNOSTICS: ')
+        if ~isempty(imagrow)
+            problem_dummy=1;
+            fprintf('\nMODEL_DIAGNOSTICS: The Jacobian of the dynamic model contains imaginary parts. The problem arises from: \n\n')
+            display_problematic_vars_Jacobian(imagrow,imagcol,M,dr.ys,'dynamic','MODEL_DIAGNOSTICS: ')
+        end
     end
     if exist('hessian1','var')
         if any(any(isinf(hessian1) | isnan(hessian1)))
