@@ -2,7 +2,7 @@ classdef dseries<handle % --*-- Unitary tests --*--
 
 % Class for time series.
 
-% Copyright (C) 2013-2019 Dynare Team
+% Copyright (C) 2013-2021 Dynare Team
 %
 % This code is free software: you can redistribute it and/or modify
 % it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@ classdef dseries<handle % --*-- Unitary tests --*--
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 properties
     data  = [];         % Array of data (a column per variable, a row per observation)
@@ -66,7 +66,7 @@ methods
             elseif ischar(varargin{1})
                 [init, o.data, o.name, o.tex, o.ops, o.tags] = load_data(varargin{1});
                 o.dates = init:init+(nobs(o)-1);
-            elseif ~isoctave() && ~matlab_ver_less_than('8.2') && istable(varargin{1})
+            elseif ~isoctave() && istable(varargin{1})
                 % It is assumed that the dates are in the first column.
                 o.name = varargin{1}.Properties.VariableNames(2:end);
                 o.tex = name2tex(o.name);
@@ -124,7 +124,7 @@ methods
                 init = dates(1,1);
             elseif (isdates(b) && isequal(length(b),1))
                 init = b;
-            elseif ischar(b) && isdate(b)% Weekly, Monthly, Quaterly or Annual data (string).
+            elseif ischar(b) && isdate(b)% Daily, Weekly, Monthly, Quaterly, Bi-annual or Annual data (string).
                 init = dates(b);
             elseif (isnumeric(b) && isscalar(b) && isint(b)) % Yearly data.
                 init = dates([num2str(b) 'Y']);
@@ -206,7 +206,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,12);
 %$     t(3) = dassert(ts.init.freq,12);
-%$     t(4) = dassert(ts.init.time,[1938, 11]);
+%$     t(4) = dassert(ts.init.time,1938*12+11);
 %$ end
 %$
 %$ T = all(t);
@@ -226,7 +226,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1994, 3]);
+%$     t(4) = dassert(ts.init.time,1994*4+3);
 %$     t(5) = dassert(ts.vobs,2);
 %$     t(6) = dassert(ts.nobs,100);
 %$     t(7) = dassert(length(ts.ops), 2);
@@ -250,7 +250,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1994, 3]);
+%$     t(4) = dassert(ts.init.time,1994*4+3);
 %$     t(5) = dassert(ts.vobs,2);
 %$     t(6) = dassert(ts.nobs,100);
 %$     t(7) = dassert(length(ts.ops), 2);
@@ -274,7 +274,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1990, 1]);
+%$     t(4) = dassert(ts.init.time,1990*4+1);
 %$     t(5) = dassert(ts.vobs,4);
 %$     t(6) = dassert(ts.nobs,4);
 %$     t(7) = dassert(ts.name,{'azert';'yuiop';'qsdfg';'jklm'});
@@ -299,7 +299,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,1);
 %$     t(3) = dassert(ts.init.freq,1);
-%$     t(4) = dassert(ts.init.time,[1, 1]);
+%$     t(4) = dassert(ts.init.time,1);
 %$     t(5) = dassert(ts.vobs,1);
 %$     t(6) = dassert(ts.nobs,5);
 %$     t(7) = dassert(ts.name,{'Variable_1'});
@@ -324,7 +324,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1950, 1]);
+%$     t(4) = dassert(ts.init.time,1950*4+1);
 %$     t(5) = dassert(ts.vobs,1);
 %$     t(6) = dassert(ts.nobs,5);
 %$     t(7) = dassert(ts.name,{'Variable_1'});
@@ -349,7 +349,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1950, 1]);
+%$     t(4) = dassert(ts.init.time,1950*4+1);
 %$     t(5) = dassert(ts.vobs,2);
 %$     t(6) = dassert(ts.nobs,5);
 %$     t(7) = dassert(ts.name,{'Output'; 'Consumption'});
@@ -377,7 +377,7 @@ end % classdef
 %$ if t(1)
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1990, 1]);
+%$     t(4) = dassert(ts.init.time,1990*4+1);
 %$     t(5) = dassert(ts.vobs,3);
 %$     t(6) = dassert(ts.nobs,5);
 %$     t(7) = dassert(ts.name,{'GDP';'Consumption';'CPI'});
@@ -405,7 +405,7 @@ end % classdef
 %$ if t(1)
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1990, 1]);
+%$     t(4) = dassert(ts.init.time,1990*4+1);
 %$     t(5) = dassert(ts.vobs,3);
 %$     t(6) = dassert(ts.nobs,5);
 %$     t(7) = dassert(ts.name,{'Variable_1';'Variable_2';'Variable_3'});
@@ -433,7 +433,7 @@ end % classdef
 %$ if t(1)
 %$     t(2) = dassert(ts.freq,1);
 %$     t(3) = dassert(ts.init.freq,1);
-%$     t(4) = dassert(ts.init.time,[1, 1]);
+%$     t(4) = dassert(ts.init.time,1);
 %$     t(5) = dassert(ts.vobs,3);
 %$     t(6) = dassert(ts.nobs,5);
 %$     t(7) = dassert(ts.name,{'Variable_1';'Variable_2';'Variable_3'});
@@ -461,7 +461,7 @@ end % classdef
 %$ if t(1)
 %$     t(2) = dassert(ts.freq,1);
 %$     t(3) = dassert(ts.init.freq,1);
-%$     t(4) = dassert(ts.init.time,[1, 1]);
+%$     t(4) = dassert(ts.init.time,1);
 %$     t(5) = dassert(ts.vobs,3);
 %$     t(6) = dassert(ts.nobs,5);
 %$     t(7) = dassert(ts.name,{'GDP';'Consumption';'CPI'});
@@ -486,7 +486,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1990, 1]);
+%$     t(4) = dassert(ts.init.time,1990*4+1);
 %$     t(5) = dassert(ts.vobs,1);
 %$     t(6) = dassert(ts.nobs,4);
 %$     t(7) = dassert(length(ts.ops), 1);
@@ -509,7 +509,7 @@ end % classdef
 %$ if length(t)>1
 %$     t(2) = dassert(ts.freq,4);
 %$     t(3) = dassert(ts.init.freq,4);
-%$     t(4) = dassert(ts.init.time,[1990, 1]);
+%$     t(4) = dassert(ts.init.time,1990*4+1);
 %$     t(5) = dassert(ts.vobs,2);
 %$     t(6) = dassert(ts.nobs,4);
 %$     t(7) = dassert(ts.data, [ones(4,1), 2*ones(4,1)]);
