@@ -1,8 +1,8 @@
-function o = resetopts(o, ops)
+function o = resetops(o, ops)
 
 % Redefine ops member.
 
-% Copyright (C) 2017 Dynare Team
+% Copyright (C) 2017-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -17,6 +17,6 @@ function o = resetopts(o, ops)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 o.ops = ops;
\ No newline at end of file
