@@ -17,7 +17,7 @@ function save(o, basename, format) % --*-- Unitary tests --*--
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 if nargin<3 || isempty(format)
     format = 'mat';
@@ -122,6 +122,10 @@ switch format
         fprintf(fid, '%s%s\n',date2string(o.dates(t)), str);
     end
     fclose(fid);
+  case 'fame'
+    fame.write([basename '.db'], A);
+  otherwise
+    error('Unknown format!')
 end
 
 %@test:1
