@@ -16,7 +16,7 @@ function [freq, init, data, varlist, tex, ops, tags] = load_m_file_data(file) %
 % be a scalar integer and INIT__ a string like '1938M11', '1945Q3', '1973W3' or '2009A'. If these variables are not specified
 % default values for freq and init are 1 and dates(1,1).
 
-% Copyright (C) 2012-2018 Dynare Team
+% Copyright (C) 2012-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -31,7 +31,7 @@ function [freq, init, data, varlist, tex, ops, tags] = load_m_file_data(file) %
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 if isoctave
     run(file);
@@ -45,11 +45,15 @@ if isoctave
     clear argn;
 end
 
-save tmp__dataSet__.mat;
+% store variables to structure, workaround for https://git.dynare.org/Dynare/dseries/-/merge_requests/40
+fields=[who].';
 
-[freq, init, data, varlist, tex, ops, tags] = load_mat_file_data('tmp__dataSet__.mat');
+assigns= cellfun(@(f) ['tmp__dataSet__struct.' f '=' f ';'],fields,'uniformoutput',0);
+for line_iter=1:length(assigns)
+    eval(assigns{line_iter});
+end
 
-delete tmp__dataSet__.mat;
+[freq, init, data, varlist, tex, ops, tags] = load_mat_file_data(tmp__dataSet__struct);
 
 return
 
@@ -86,7 +90,7 @@ return
  t(2) = dassert(freq,4);
  t(3) = isdates(init);
  t(4) = dassert(init.freq,4);
- t(5) = dassert(init.time,[1938 4]);
+ t(5) = dassert(init.time,1938*4+4);
  t(6) = dassert(varlist,{'azert';'yuiop'});
  t(7) = dassert(tex,{'azert';'yuiop'});
  t(8) = dassert(ops,{'method1(azert)';'method2(yuiop)'});
