@@ -1,6 +1,6 @@
 function C = issubperiod(A,B) % --*-- Unitary tests --*--
 
-% Copyright (C) 2013-2015 Dynare Team
+% Copyright (C) 2013-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -15,12 +15,16 @@ function C = issubperiod(A,B) % --*-- Unitary tests --*--
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 if isfreq(B)
-    C = all(isint(A)) && all(A>=1) && all(A<=B);
+    if B == 365
+        error('issubperiod:: The function does not support daily frequency!')
+    else
+        C = all(isint(A)) && all(A>=1) && all(A<=B);
+    end
 else
-    error('issubperiod:: Second input argument must be equal to 1, 4, 12 or 52 (frequency)!')
+    error('issubperiod:: Second input argument must be equal to 1, 2, 4, 12 or 52 (frequency)!')
 end
 
 %@test:1
@@ -66,4 +70,19 @@ end
 %$ end
 %$
 %$ T = all(t);
-%@eof:3
\ No newline at end of file
+%@eof:3
+
+%@test:4
+%$ try
+%$    b = issubperiod(3, 2);
+%$    t(1) = true;
+%$ catch
+%$    t(1) = false;
+%$ end
+%$
+%$ if t(1)
+%$     t(2) = dassert(b, false);
+%$ end
+%$
+%$ T = all(t);
+%@eof:4
\ No newline at end of file
