@@ -1,5 +1,5 @@
 function rep = CommResidTablePage(rep, db_q, dc_q, trange, vline_after)
-% Copyright (C) 2013-2017 Dynare Team
+% Copyright (C) 2013-2019 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -14,25 +14,25 @@ function rep = CommResidTablePage(rep, db_q, dc_q, trange, vline_after)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 seriesNames = {{'RES_LRPOIL_GAP_WORLD'}, ...
-               {'RES_LRPOIL_BAR_WORLD'}, ...
-               {'RES_LRPOIL_G_WORLD'}, ...
-               {'RES_LRPFOOD_GAP_WORLD'}, ...
-               {'RES_LRPFOOD_BAR_WORLD'}, ...
-               {'RES_LRPFOOD_G_WORLD'}};
+    {'RES_LRPOIL_BAR_WORLD'}, ...
+    {'RES_LRPOIL_G_WORLD'}, ...
+    {'RES_LRPFOOD_GAP_WORLD'}, ...
+    {'RES_LRPFOOD_BAR_WORLD'}, ...
+    {'RES_LRPFOOD_G_WORLD'}};
 
-rep = rep.addTable('title', 'Commodities', ...
-                   'range', trange, ...
-                   'vlineAfter', vline_after);
+rep.addTable('title', 'Commodities', ...
+    'range', trange, ...
+    'vlineAfter', vline_after);
 
-for i=1:length(seriesNames)
-    rep = rep.addSeries('data', db_q{seriesNames{i}{1}});
+for i = 1:length(seriesNames)
+    rep.addSeries('data', db_q{seriesNames{i}{1}});
     delta = db_q{seriesNames{i}{1}} - dc_q{seriesNames{i}{1}};
-    delta = delta.tex_rename('$\Delta$');
-    rep = rep.addSeries('data', delta, ...
-                        'tableShowMarkers', true, ...
-                        'tableAlignRight', true);
+    delta.tex_rename_('$\Delta$');
+    rep.addSeries('data', delta, ...
+        'tableShowMarkers', true, ...
+        'tableAlignRight', true);
 end
 end
