@@ -25,7 +25,7 @@ function ms_write_markov_file(fname, options)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 n_chains = length(options.ms.ms_chain);
 nvars = length(options.varobs);
@@ -52,6 +52,10 @@ for i_chain = 1:n_chains
     %//=====================================================//
     fprintf(fh,'//== Number of states for state_variable[%d] ==//\n', ...
             i_chain);
+    if ~isfield(options.ms.ms_chain(i_chain),'regime') ...
+                || isempty(options.ms.ms_chain(i_chain).regime)
+        error('Chain %u does not feature any regime states. Did you forget to specify them?',i_chain)
+    end
     n_states = length(options.ms.ms_chain(i_chain).regime);
     fprintf(fh,'%d\n\n',n_states);
 
