@@ -2,7 +2,7 @@ function ys1 = add_auxiliary_variables_to_steadystate(ys,aux_vars,fname, ...
                                                   exo_steady_state, exo_det_steady_state,params, byte_code)
 % Add auxiliary variables to the steady state vector
 
-% Copyright (C) 2009-2017 Dynare Team
+% Copyright (C) 2009-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -17,15 +17,15 @@ function ys1 = add_auxiliary_variables_to_steadystate(ys,aux_vars,fname, ...
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 n = length(aux_vars);
 ys1 = [ys;zeros(n,1)];
 
 for i=1:n+1
     if byte_code
-        [info, res] = bytecode('static','evaluate',ys1,...
-                               [exo_steady_state; ...
-                            exo_det_steady_state],params);
+        res = bytecode('static','evaluate',ys1,...
+                       [exo_steady_state; ...
+                        exo_det_steady_state],params);
     else
         res = feval([fname '.static'],ys1,...
                     [exo_steady_state; ...
