@@ -55,7 +55,7 @@ function [dr,info,M_,options_,oo_] = dr1_PI(dr,task,M_,options_,oo_)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global lq_instruments;
 info = 0;
@@ -154,7 +154,7 @@ else
 end
 
 if options_.debug
-    save([M_.fname '_debug.mat'],'jacobia_')
+    save([M_.dname filesep 'Output' filesep M_.fname '_debug.mat'],'jacobia_')
 end
 
 dr=set_state_space(dr,M_,options_);
