@@ -21,7 +21,7 @@ function [pmean, pmode, pmedian, pstdev, p025, p975, covariance] = online_auxili
 % - p975                     [double]    n×1 vector, 97.5 percent of the particles are below p975(i) for i=1,…,n.
 % - covariance               [double]    n×n matrix, covariance of the particles at the end of the sample.
 
-% Copyright (C) 2013-2019 Dynare Team
+% Copyright © 2013-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -36,7 +36,7 @@ function [pmean, pmode, pmedian, pstdev, p025, p975, covariance] = online_auxili
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 % Set seed for randn().
 set_dynare_seed('default');
@@ -103,7 +103,7 @@ for t=1:sample_size
     if t>1
         fprintf('\nSubsample with %s first observations.\n\n', int2str(t))
     else
-        fprintf('\nSubsample with only the first observation.\n\n', int2str(t))
+        fprintf('\nSubsample with only the first observation.\n\n')
     end
     % Moments of parameters particles distribution
     m_bar = xparam*(weights');
@@ -271,7 +271,7 @@ for t=1:sample_size
     for l=1:size(xparam,1)
         str = sprintf('%s\n %5.4f \t\t %7.5f \t\t %5.4f', str, lb95_xparam(l,t), mean_xparam(l,t), ub95_xparam(l,t));
     end
-    disp([str])
+    disp(str)
     disp('')
 end
 
@@ -298,29 +298,21 @@ if TeX
 end
 
 for plt = 1:nbplt
-    if TeX
-        NAMES = [];
-        TeXNAMES = [];
-    end
     hh = dyn_figure(DynareOptions.nodisplay,'Name','Parameters Trajectories');
     for k=1:length(pmean)
         subplot(nr,nc,k)
         [name,texname] = get_the_name(k,TeX,Model,EstimatedParameters,DynareOptions);
-        if TeX
-            if isempty(NAMES)
-                NAMES = name;
-                TeXNAMES = texname;
-            else
-                NAMES = char(NAMES,name);
-                TeXNAMES = char(TeXNAMES,texname);
-            end
-        end
         % Draw the surface for an interval containing 95% of the particles.
-        shade(1:sample_size, ub95_xparam(k,:)', 1:sample_size, lb95_xparam(k,:)', 'FillType',[1 2], 'LineStyle', 'none', 'Marker', 'none')
+        area(1:sample_size, ub95_xparam(k,:), 'FaceColor', [.9 .9 .9], 'BaseValue', min(lb95_xparam(k,:)));
         hold on
+        area(1:sample_size, lb95_xparam(k,:), 'FaceColor', [1 1 1], 'BaseValue', min(lb95_xparam(k,:)));
         % Draw the mean of particles.
         plot(1:sample_size, mean_xparam(k,:), '-k', 'linewidth', 2)
-        title(name,'interpreter','none')
+        if TeX
+            title(texname,'interpreter','latex')
+        else
+            title(name,'interpreter','none')
+        end
         hold off
         axis tight
         drawnow
@@ -329,9 +321,6 @@ for plt = 1:nbplt
     if TeX
         % TeX eps loader file
         fprintf(fidTeX,'\\begin{figure}[H]\n');
-        for jj = 1:length(x)
-            fprintf(fidTeX,'\\psfrag{%s}[1][][0.5][0]{%s}\n',deblank(NAMES(jj,:)),deblank(TeXNAMES(jj,:)));
-        end
         fprintf(fidTeX,'\\centering \n');
         fprintf(fidTeX,'\\includegraphics[scale=0.5]{%s_ParamTraj%s}\n',Model.fname,int2str(plt));
         fprintf(fidTeX,'\\caption{Parameters trajectories.}');
@@ -346,29 +335,20 @@ number_of_grid_points = 2^9;      % 2^9 = 512 !... Must be a power of two.
 bandwidth = 0;                    % Rule of thumb optimal bandwidth parameter.
 kernel_function = 'gaussian';     % Gaussian kernel for Fast Fourier Transform approximation.
 for plt = 1:nbplt
-    if TeX
-        NAMES = [];
-        TeXNAMES = [];
-    end
     hh = dyn_figure(DynareOptions.nodisplay,'Name','Parameters Densities');
     for k=1:length(pmean)
         subplot(nr,nc,k)
         [name,texname] = get_the_name(k,TeX,Model,EstimatedParameters,DynareOptions);
-        if TeX
-            if isempty(NAMES)
-                NAMES = name;
-                TeXNAMES = texname;
-            else
-                NAMES = char(NAMES,name);
-                TeXNAMES = char(TeXNAMES,texname);
-            end
-        end
         optimal_bandwidth = mh_optimal_bandwidth(xparam(k,:)',number_of_particles,bandwidth,kernel_function);
         [density(:,1),density(:,2)] = kernel_density_estimate(xparam(k,:)', number_of_grid_points, ...
                                                           number_of_particles, optimal_bandwidth, kernel_function);
         plot(density(:,1), density(:,2));
         hold on
-        title(name, 'interpreter', 'none')
+        if TeX
+            title(texname,'interpreter','latex')
+        else
+            title(name,'interpreter','none')
+        end
         hold off
         axis tight
         drawnow
@@ -377,9 +357,6 @@ for plt = 1:nbplt
     if TeX && any(strcmp('eps',cellstr(DynareOptions.graph_format)))
         % TeX eps loader file
         fprintf(fidTeX, '\\begin{figure}[H]\n');
-        for jj = 1:length(x)
-            fprintf(fidTeX, '\\psfrag{%s}[1][][0.5][0]{%s}\n', deblank(NAMES(jj,:)), deblank(TeXNAMES(jj,:)));
-        end
         fprintf(fidTeX,'\\centering \n');
         fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%_param_density%s}\n',min(k/nc,1),M_.fname,int2str(plt));
         fprintf(fidTeX,'\\caption{Parameter densities based on the Liu/West particle filter.}');
