@@ -33,7 +33,7 @@ function info = perfect_foresight_simulation(compute_linear_solution,steady_stat
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global M_ options_ it_ oo_
 
@@ -74,7 +74,7 @@ else
 end
 
 if ~isstruct(compute_linear_solution) && compute_linear_solution
-    [dr,info,M_,options_,oo_] = resol(0,M_,options_,oo_);
+    [dr,info,M_,oo_] = resol(0,M_,options_,oo_);
 elseif isstruct(compute_linear_solution)
     dr = compute_linear_solution;
     compute_linear_solution = 1;
