@@ -27,7 +27,7 @@ function perfect_foresight_solver()
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global M_ options_ oo_
 
@@ -70,14 +70,15 @@ if ~oo_.deterministic_simulation.status && ~options_.no_homotopy
         fprintf('Switching to a homotopy method...\n')
     end
 
-    if ~M_.maximum_lag
+    if ~M_.maximum_lag && M_.maximum_lead>0
         disp('Homotopy not implemented for purely forward models!')
         disp('Failed to solve the model!')
         disp('Return with empty oo_.endo_simul.')
         oo_.endo_simul = [];
         return
     end
-    if ~M_.maximum_lead
+
+    if ~M_.maximum_lead && M_.maximum_lag>0
         disp('Homotopy not implemented for purely backward models!')
         disp('Failed to solve the model!')
         disp('Return with empty oo_.endo_simul.')
@@ -85,6 +86,14 @@ if ~oo_.deterministic_simulation.status && ~options_.no_homotopy
         return
     end
 
+    if ~M_.maximum_lead && ~M_.maximum_lag
+        disp('Homotopy not implemented for purely static models!')
+        disp('Failed to solve the model!')
+        disp('Return with empty oo_.endo_simul.')
+        oo_.endo_simul = [];
+        return
+    end
+
     % Disable warnings if homotopy
     warning_old_state = warning;
     warning off all
