@@ -38,7 +38,7 @@ function [endogenousvariables, info] = sim1_linear(endogenousvariables, exogenou
 % to center the variables around the deterministic steady state to solve the
 % perfect foresight model.
 
-% Copyright (C) 2015-2017 Dynare Team
+% Copyright (C) 2015-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -53,7 +53,7 @@ function [endogenousvariables, info] = sim1_linear(endogenousvariables, exogenou
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 verbose = options.verbosity;
 
@@ -114,6 +114,22 @@ x = repmat(transpose(steadystate_x), 1+M.maximum_exo_lag+M.maximum_exo_lead, 1);
 
 % Evaluate the Jacobian of the dynamic model at the deterministic steady state.
 [d1, jacobian] = dynamicmodel(z, x, params, steadystate_y, M.maximum_exo_lag+1);
+if options.debug
+    column=find(all(jacobian==0,1));
+    if ~isempty(column)
+        fprintf('The dynamic Jacobian is singular. The problem derives from:\n')
+        for iter=1:length(column)
+            [var_row,var_index]=find(M.lead_lag_incidence==column(iter));            
+            if var_row==2
+                fprintf('The derivative with respect to %s being 0 for all equations.\n',M.endo_names{var_index})
+            elseif var_row==1
+                fprintf('The derivative with respect to the lag of %s being 0 for all equations.\n',M.endo_names{var_index})
+            elseif var_row==3
+                fprintf('The derivative with respect to the lead of %s being 0 for all equations.\n',M.endo_names{var_index})
+            end            
+        end
+    end   
+end
 
 % Check that the dynamic model was evaluated at the steady state.
 if max(abs(d1))>options.solve_tolf
@@ -159,16 +175,9 @@ for it = (maximum_lag+1):(maximum_lag+periods)
         nv = length(vv);
         iA(iv+m,:) = [i_rows(rr),i_cols_A(cc),vv];
     end
-    if M.maximum_exo_lag > 0
-        % needed as jacobian for lagged exogenous variables is wrong
-        % in current version of Dynare
-        zz = Y(i_cols);
-        res(i_rows) = dynamicmodel(zz, exogenousvariables, params, steadystate_y, it);
-    else
-        z(jendo) = Y(i_cols);
-        z(jexog) = transpose(exogenousvariables(it,:));
-        res(i_rows) = jacobian*z;
-    end
+    z(jendo) = Y(i_cols);
+    z(jexog) = transpose(exogenousvariables(it,:));
+    res(i_rows) = jacobian*z;
     m = m + nv;
     i_rows = i_rows + ny;
     i_cols = i_cols + ny;
@@ -251,4 +260,4 @@ end
 
 if verbose
     skipline();
-end
\ No newline at end of file
+end
