@@ -19,7 +19,7 @@ function plot_identification(params, idemoments, idehess, idemodel, idelre, adva
 % SPECIAL REQUIREMENTS
 %    None
 
-% Copyright (C) 2008-2017 Dynare Team
+% Copyright (C) 2008-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -34,7 +34,7 @@ function plot_identification(params, idemoments, idehess, idemodel, idelre, adva
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global M_ options_
 
@@ -59,7 +59,8 @@ if SampleSize == 1
     subplot(211)
     mmm = (idehess.ide_strength_dMOMENTS);
     [ss, is] = sort(mmm);
-    if ~all(isnan(idehess.ide_strength_dMOMENTS_prior))
+    if ~all(isnan(idehess.ide_strength_dMOMENTS_prior)) ...
+       && ~(nparam == 1 && ~isoctave && matlab_ver_less_than('9.7')) % MATLAB < R2019b does not accept bar(1, [2 3])
         bar(1:nparam,log([idehess.ide_strength_dMOMENTS(:,is)' idehess.ide_strength_dMOMENTS_prior(:,is)']))
     else
         bar(1:nparam,log([idehess.ide_strength_dMOMENTS(:,is)' ]))
@@ -105,7 +106,8 @@ if SampleSize == 1
     end
 
     subplot(212)
-    if ~all(isnan(idehess.deltaM_prior))
+    if ~all(isnan(idehess.deltaM_prior)) ...
+       && ~(nparam == 1 && ~isoctave && matlab_ver_less_than('9.7')) % MATLAB < R2019b does not accept bar(1, [2 3])
         bar(1:nparam, log([idehess.deltaM(is) idehess.deltaM_prior(is)]))
     else
         bar(1:nparam, log([idehess.deltaM(is)]))
