@@ -26,7 +26,7 @@ function [out, steady_state] = plot_shock_decomposition(M_,oo_,options_,varlist)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 options_.nodisplay = options_.plot_shock_decomp.nodisplay;
 options_.graph_format = options_.plot_shock_decomp.graph_format;
@@ -50,7 +50,7 @@ if size(varlist,1) == 0
     end
 end
 
-if isfield(options_.plot_shock_decomp,'init2shocks') % private trap for uimenu calls
+if isfield(options_.plot_shock_decomp,'init2shocks') && options_.plot_shock_decomp.realtime==0 % private trap for uimenu calls
     init2shocks=options_.plot_shock_decomp.init2shocks;
 else
     init2shocks=[];
@@ -59,7 +59,7 @@ if ~isempty(init2shocks)
     init2shocks=M_.init2shocks.(init2shocks);
 end
 
-
+orig_endo_names = M_.endo_names;
 epilogue_decomp=false;
 if exist_varlist && any(ismember(varlist,M_.epilogue_names))
     epilogue_decomp=true;
@@ -252,15 +252,15 @@ if ~isempty(init2shocks) && ~expand
     n=size(init2shocks,1);
     M_.exo_names_init=M_.exo_names;
     for i=1:n
-        j=strmatch(init2shocks{i}{1},M_.endo_names,'exact');
+        j=strmatch(init2shocks{i}{1},orig_endo_names,'exact');
         if ~isempty(init2shocks{i}{2})
             jj=strmatch(init2shocks{i}{2},M_.exo_names,'exact');
-            M_.exo_names_init{jj}=[M_.exo_names_init{jj} ' + ' M_.endo_names{j}];
-            z(:,jj,:)= z(:,jj,:) + oo_.initval_decomposition (:,j,:);
+            M_.exo_names_init{jj}=[M_.exo_names_init{jj} ' + ' orig_endo_names{j}];
+            z(:,jj,:)= z(:,jj,:) + oo_.initval_decomposition (:,j,1:size(z,3));
         else
-            z(:,end,:)= z(:,end,:) - oo_.initval_decomposition (:,j,:);
+            z(:,end,:)= z(:,end,:) - oo_.initval_decomposition (:,j,1:size(z,3));
         end
-        z(:,end-1,:)= z(:,end-1,:) - oo_.initval_decomposition (:,j,:);
+        z(:,end-1,:)= z(:,end-1,:) - oo_.initval_decomposition (:,j,1:size(z,3));
 
     end
 end
@@ -271,6 +271,9 @@ if ~epilogue_decomp
     else
         steady_state = oo_.steady_state;
     end
+    if isfield(oo_.shock_decomposition_info,'i_var') && (M_.endo_nbr<length(orig_endo_names))
+        steady_state=steady_state(oo_.shock_decomposition_info.i_var);
+    end
 else
     steady_state = oo_.shock_decomposition_info.epilogue_steady_state;
 end
@@ -286,7 +289,7 @@ if isequal(type,'aoa') && isstruct(q2a)
         % take all dates where realtime is saved
         qqq=options_.initial_date+options_.shock_decomp.save_realtime(:)-1;
         % take the first Q4 of saved realtime
-        t0=min(options_.shock_decomp.save_realtime(qqq.time(:,2)==4));
+        t0=min(options_.shock_decomp.save_realtime(subperiod(qqq)==4));
         if isempty(t0)
             error('the realtime decompositions are not stored in Q4! Please check your dates and settings.')
         end
@@ -421,12 +424,12 @@ switch type
         t0=1; % we assume the sample starts Q1 of 1st year
         initial_date = dates('1Y');
     else
-        initial_date0 = dates([int2str(initial_date.time(1)) 'Y']);
-        if initial_date.time(2)==1  % the first year is full
+        initial_date0 = dates([int2str(year(initial_date)) 'Y']);
+        if subperiod(initial_date)==1  % the first year is full
             t0=1;
             initial_date1=initial_date0;
         else
-            t0=(4-initial_date.time(2)+2); % 1st period of the 1st full year in sample
+            t0=(4-subperiod(initial_date)+2); % 1st period of the 1st full year in sample
             initial_date1=initial_date0+1;
         end
     end
@@ -492,16 +495,16 @@ switch type
         end
     else
         % this is for quarterly-annualized variables already defined in model, so we can just take Q4
-        t0=4-initial_date.time(2)+1;
+        t0=4-subperiod(initial_date)+1;
         initial_date = initial_date0;
         z=z(:,:,t0:4:end);
     end
 
     if ~isempty(options_.plot_shock_decomp.plot_init_date)
-        options_.plot_shock_decomp.plot_init_date = dates([int2str(options_.plot_shock_decomp.plot_init_date.time(1)) 'Y']);
+        options_.plot_shock_decomp.plot_init_date = dates([int2str(year(options_.plot_shock_decomp.plot_init_date)) 'Y']);
     end
     if ~isempty(options_.plot_shock_decomp.plot_end_date)
-        options_.plot_shock_decomp.plot_end_date = dates([int2str(options_.plot_shock_decomp.plot_end_date.time(1)) 'Y']);
+        options_.plot_shock_decomp.plot_end_date = dates([int2str(year(options_.plot_shock_decomp.plot_end_date)) 'Y']);
     end
 
 
