@@ -12,7 +12,7 @@ function z = resid(junk)
 % SPECIAL REQUIREMENTS
 %    none
 
-% Copyright (C) 2001-2018 Dynare Team
+% Copyright (C) 2001-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -27,7 +27,7 @@ function z = resid(junk)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global M_ options_ oo_
 
@@ -67,26 +67,9 @@ if options_.steadystate_flag
 end
 
 % Compute the residuals
-if options_.block && ~options_.bytecode
-    z = zeros(M_.endo_nbr,1);
-    for i = 1:length(M_.block_structure_stat.block)
-        [r, g, yy, var_indx] = feval([M_.fname '.static'],...
-                                     i,...
-                                     oo_.steady_state,...
-                                     [oo_.exo_steady_state; ...
-                            oo_.exo_det_steady_state], M_.params);
-        idx = M_.block_structure_stat.block(i).equation;
-        z(idx) = r;
-    end
-elseif options_.bytecode
-    [check, z] = bytecode('evaluate','static');
-    mexErrCheck('bytecode', check);
-else
-    z = feval([M_.fname '.static'],...
-              oo_.steady_state,...
-              [oo_.exo_steady_state; ...
-               oo_.exo_det_steady_state], M_.params);
-end
+z = evaluate_static_model(oo_.steady_state, [oo_.exo_steady_state; ...
+                                             oo_.exo_det_steady_state], ...
+                          M_.params, M_, options_);
 
 M_.Sigma_e = Sigma_e;
 
@@ -97,11 +80,11 @@ if nargout == 0
     ind = [];
     disp('Residuals of the static equations:')
     skipline()
-    for i=1:M_.orig_endo_nbr
-        if abs(z(i)) < options_.solve_tolf/100
+    for i=1:M_.orig_eq_nbr
+        if abs(z(i+M_.ramsey_eq_nbr)) < options_.solve_tolf/100
             tmp = 0;
         else
-            tmp = z(i);
+            tmp = z(i+M_.ramsey_eq_nbr);
         end
         if istag
             tg = tags(cell2mat(tags(:,1)) == i,2:3); % all tags for equation i
