@@ -1,4 +1,4 @@
-function [dr, info, M, options, oo] = resol(check_flag, M, options, oo)
+function [dr, info, M, oo] = resol(check_flag, M, options, oo)
 
 % Computes the perturbation based decision rules of the DSGE model (orders 1 to 3)
 %
@@ -12,7 +12,6 @@ function [dr, info, M, options, oo] = resol(check_flag, M, options, oo)
 % - dr            [structure]     Reduced form model.
 % - info          [integer]       scalar or vector, error code.
 % - M             [structure]     Matlab's structure describing the model (M_).
-% - options       [structure]     Matlab's structure describing the current options (options_).
 % - oo            [structure]     Matlab's structure containing the results (oo_).
 %
 % REMARKS
@@ -48,10 +47,27 @@ function [dr, info, M, options, oo] = resol(check_flag, M, options, oo)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 if isfield(oo,'dr')
-    dr = oo.dr;
+    if isfield(oo.dr,'kstate')
+        dr.kstate = oo.dr.kstate;
+    end
+    if isfield(oo.dr,'inv_order_var')
+        dr.inv_order_var = oo.dr.inv_order_var;
+    end
+    if isfield(oo.dr,'order_var')
+        dr.order_var = oo.dr.order_var;
+    end
+    if isfield(oo.dr,'restrict_var_list')
+        dr.restrict_var_list = oo.dr.restrict_var_list;
+    end
+    if isfield(oo.dr,'restrict_columns')
+        dr.restrict_columns = oo.dr.restrict_columns;
+    end
+    if isfield(oo.dr,'obs_var')
+        dr.obs_var = oo.dr.obs_var;
+    end
 end
 
 if M.exo_nbr == 0
@@ -96,7 +112,7 @@ if options.loglinear
 end
 
 if options.block
-    [dr,info,M,options,oo] = dr_block(dr,check_flag,M,options,oo);
+    [dr,info,M,oo] = dr_block(dr,check_flag,M,options,oo);
     dr.edim = nnz(abs(dr.eigval) > options.qz_criterium);
     dr.sdim = dr.nd-dr.edim;
 else
