@@ -35,7 +35,7 @@ function SampleAddress = selec_posterior_draws(SampleSize,drsize)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 global M_ options_ estim_params_ oo_
 
@@ -115,7 +115,7 @@ if info
             pdraws(i,1) = {x2(SampleAddress(i,4),:)};
             if info-1
                 set_parameters(pdraws{i,1});
-                [dr,info,M_,options_,oo_] =compute_decision_rules(M_,options_,oo_);
+                [dr,info,M_,oo_] =compute_decision_rules(M_,options_,oo_);
                 pdraws(i,2) = { dr };
             end
             old_mhfile = mhfile;
@@ -142,7 +142,7 @@ if info
             pdraws(linee,1) = {x2(SampleAddress(i,4),:)};
             if info-1
                 set_parameters(pdraws{linee,1});
-                [dr,info,M_,options_,oo_] = resol(0,M_,options_,oo_);
+                [dr,info,M_,options_,oo_] = compute_decision_rules(M_,options_,oo_);
                 pdraws(linee,2) = { dr };
             end
             old_mhfile = mhfile;
