@@ -27,7 +27,7 @@ function [cmm, mm] = simulated_moment_uncertainty(indx, periods, replic,options_
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 mm=zeros(length(indx),replic);
 
@@ -71,7 +71,7 @@ else
     logged_steady_state_indicator=0;
 end
 
-[dr,info,M_,options_,oo_] = resol(0,M_,options_,oo_);
+[dr,info,M_,oo_] = compute_decision_rules(M_,options_,oo_);
 oo_.dr=dr;
 if info(1)
     fprintf('\nsimulated_moment_uncertainty: model could not be solved')
