@@ -60,13 +60,13 @@ function [y_out,DynareResults] =simult(y0, dr,DynareModel,DynareOptions,DynareRe
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 order = DynareOptions.order;
 replic = DynareOptions.simul_replic;
 
 if replic > 1
-    fname = [DynareModel.fname,'_simul'];
+    fname = [DynareModel.dname filesep 'Output' DynareModel.fname,'_simul'];
     fh = fopen(fname,'w+');
 end
 
