@@ -1,5 +1,5 @@
-function [y, info] = solve_one_boundary(fname, y, x, params, steady_state, ...
-                                        y_index_eq, nze, periods, is_linear, Block_Num, y_kmin, maxit_, solve_tolf, lambda, cutoff, stack_solve_algo, forward_backward, is_dynamic, verbose, M, options, oo)
+function [y, T, oo_, info] = solve_one_boundary(fname, y, x, params, steady_state, T, ...
+                                                y_index_eq, nze, periods, is_linear, Block_Num, y_kmin, maxit_, solve_tolf, lambda, cutoff, stack_solve_algo, is_forward, is_dynamic, verbose, M, options, oo_)
 % Computes the deterministic simulation of a block of equation containing
 % lead or lag variables
 %
@@ -10,13 +10,13 @@ function [y, info] = solve_one_boundary(fname, y, x, params, steady_state, ...
 %   x                   [matrix]        All the exogenous variables of the model
 %   params              [vector]        All the parameters of the model
 %   steady_state        [vector]        steady state of the model
+%   T                   [matrix]        Temporary terms
 %   y_index_eq          [vector of int] The index of the endogenous variables of
 %                                       the block
 %   nze                 [integer]       number of non-zero elements in the
 %                                       jacobian matrix
 %   periods             [integer]       number of simulation periods
-%   is_linear           [integer]       if is_linear=1 the block is linear
-%                                       if is_linear=0 the block is not linear
+%   is_linear           [logical]       whether the block is linear
 %   Block_Num           [integer]       block number
 %   y_kmin              [integer]       maximum number of lag in the model
 %   maxit_              [integer]       maximum number of iteration in Newton
@@ -31,20 +31,18 @@ function [y, info] = solve_one_boundary(fname, y, x, params, steady_state, ...
 %                                            - 2 GMRES
 %                                            - 3 BicGStab
 %                                            - 4 Optimal path length
-%   forward_backward    [integer]       The block has to be solve forward
-%                                       (1) or backward (0)
-%   is_dynamic          [integer]       (1) The block belong to the dynamic
+%   is_forward          [logical]       Whether the block has to be solved forward
+%                                       If false, the block is solved backward
+%   is_dynamic          [logical]       If true, the block belongs to the dynamic file
 %                                           file and the oo_.deterministic_simulation field has to be uptated
-%                                       (0) The block belong to the static
-%                                           file and th oo_.deteerminstic
+%                                       If false, the block belongs to the static
+%                                           file and the oo_.detereministic_simulation
 %                                           field remains unchanged
-%   verbose            [integer]        (0) iterations are not printed
-%                                       (1) iterations are printed
-%   indirect_call      [integer]        (0) direct call to the fname
-%                                       (1) indirect call via the
-%                                       local_fname wrapper
+%   verbose             [logical]       Whether iterations are to be printed
+%
 % OUTPUTS
 %   y                  [matrix]         All endogenous variables of the model
+%   T                  [matrix]        Temporary terms
 %   info               [integer]        >=0 no error
 %                                       <0 error
 %
@@ -55,7 +53,7 @@ function [y, info] = solve_one_boundary(fname, y, x, params, steady_state, ...
 %   none.
 %
 
-% Copyright (C) 1996-2018 Dynare Team
+% Copyright (C) 1996-2020 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -70,17 +68,15 @@ function [y, info] = solve_one_boundary(fname, y, x, params, steady_state, ...
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 
 Blck_size=size(y_index_eq,2);
-g2 = [];
-g3 = [];
 correcting_factor=0.01;
 ilu_setup.droptol=1e-10;
 max_resa=1e100;
 reduced = 0;
-if forward_backward
+if is_forward
     incr = 1;
     start = y_kmin+1;
     finish = periods+y_kmin;
@@ -96,19 +92,20 @@ for it_=start:incr:finish
     g1=spalloc( Blck_size, Blck_size, nze);
     while ~(cvg==1 || iter>maxit_)
         if is_dynamic
-            [r, y, g1, g2, g3] = feval(fname, y, x, params, steady_state, it_, 0);
+            [r, yy, T(:, it_), g1] = feval(fname, Block_Num, dynvars_from_endo_simul(y, it_, M), x, params, steady_state, T(:, it_), it_, false);
+            y(:, it_) = yy(M.lead_lag_incidence(M.maximum_endo_lag+1,:));
         else
-            [r, y, g1] = feval(fname, y, x, params);
+            [r, y, T, g1] = feval(fname, Block_Num, y, x, params, T);
         end
         if ~isreal(r)
             max_res=(-(max(max(abs(r))))^2)^0.5;
         else
             max_res=max(max(abs(r)));
         end
-        if verbose==1
+        if verbose
             disp(['iteration : ' int2str(iter+1) ' => ' num2str(max_res) ' time = ' int2str(it_)])
             if is_dynamic
-                disp([M.endo_names{y_index_eq} num2str([y(it_,y_index_eq)' r g1])])
+                disp([M.endo_names{y_index_eq} num2str([y(y_index_eq, it_) r g1])])
             else
                 disp([M.endo_names{y_index_eq} num2str([y(y_index_eq) r g1])])
             end
@@ -139,7 +136,7 @@ for it_=start:incr:finish
                                     disp(['    correcting_factor=' num2str(correcting_factor,'%f') ' max(Jacobian)=' num2str(full(max_factor),'%f')])
                                 end
                                 dx = - r/(g1+correcting_factor*speye(Blck_size));
-                                y(it_,y_index_eq)=ya_save+lambda*dx;
+                                y(y_index_eq, it_)=ya_save+lambda*dx;
                                 continue
                             else
                                 if verbose
@@ -156,7 +153,7 @@ for it_=start:incr:finish
                             disp(['reducing the path length: lambda=' num2str(lambda,'%f')])
                         end
                         if is_dynamic
-                            y(it_,y_index_eq)=ya_save-lambda*dx;
+                            y(y_index_eq, it_)=ya_save-lambda*dx;
                         else
                             y(y_index_eq)=ya_save-lambda*dx;
                         end
@@ -170,10 +167,10 @@ for it_=start:incr:finish
                             end
                         end
                         if is_dynamic
-                            oo_.deterministic_simulation.status = 0;
+                            oo_.deterministic_simulation.status = false;
                             oo_.deterministic_simulation.error = max_res;
                             oo_.deterministic_simulation.iterations = iter;
-                            oo_.deterministic_simulation.block(Block_Num).status = 0;% Convergency failed.
+                            oo_.deterministic_simulation.block(Block_Num).status = false;% Convergency failed.
                             oo_.deterministic_simulation.block(Block_Num).error = max_res;
                             oo_.deterministic_simulation.block(Block_Num).iterations = iter;
                         end
@@ -187,52 +184,13 @@ for it_=start:incr:finish
                 end
             end
             if is_dynamic
-                ya = y(it_,y_index_eq)';
+                ya = y(y_index_eq, it_);
             else
                 ya = y(y_index_eq);
             end
             ya_save=ya;
             g1a=g1;
-            if ~is_dynamic && options.solve_algo==0
-                if verbose
-                    disp('steady: fsolve')
-                end
-                if ~isoctave
-                    if ~user_has_matlab_license('optimization_toolbox')
-                        error('SOLVE_ONE_BOUNDARY: you can''t use solve_algo=0 since you don''t have MATLAB''s Optimization Toolbox')
-                    end
-                end
-                options=optimset('fsolve');
-                options.MaxFunEvals = 50000;
-                options.MaxIter = 2000;
-                options.TolFun=1e-8;
-                options.Display = 'iter';
-                options.Jacobian = 'on';
-                if ~isoctave
-                    [yn,fval,exitval,output] = fsolve(@local_fname, y(y_index_eq), ...
-                                                      options, x, params, steady_state, y, y_index_eq, fname, 0);
-                else
-                    % Under Octave, use a wrapper, since fsolve() does not have a 4th arg
-                    func = @(z) local_fname(z, x, params, steady_state, y, y_index_eq, fname, 0);
-                    % The Octave version of fsolve does not converge when it starts from the solution
-                    fvec = feval(func,y(y_index_eq));
-                    if max(abs(fvec)) >= options.solve_tolf
-                        [yn,fval,exitval,output] = fsolve(func,y(y_index_eq),options);
-                    else
-                        yn = y(y_index_eq);
-                        exitval = 3;
-                    end
-                end
-                y(y_index_eq) = yn;
-                if exitval > 0
-                    info = 0;
-                else
-                    info = -Block_Num*10;
-                end
-            elseif (~is_dynamic && options.solve_algo==2) || (is_dynamic && stack_solve_algo==4)
-                if verbose==1 && ~is_dynamic
-                    disp('steady: LU + lnsrch1')
-                end
+            if is_dynamic && stack_solve_algo==4
                 lambda=1;
                 stpmx = 100 ;
                 if is_dynamic
@@ -244,43 +202,27 @@ for it_=start:incr:finish
                 g = (r'*g1)';
                 f = 0.5*r'*r;
                 p = -g1\r ;
-                if is_dynamic
-                    [ya,f,r,check]=lnsrch1(y(it_,:)',f,g,p,stpmax, ...
-                                           'lnsrch1_wrapper_one_boundary',nn, ...
-                                           y_index_eq, options.solve_tolx, y_index_eq, fname, y, x, params, steady_state, it_);
-                    dx = ya' - y(it_, :);
-                else
-                    [ya,f,r,check]=lnsrch1(y,f,g,p,stpmax,fname,nn,y_index_eq, options.solve_tolx, x, ...
-                                           params, steady_state,0);
-                    dx = ya - y(y_index_eq);
-                end
-                if is_dynamic
-                    y(it_,:) = ya';
-                else
-                    y = ya';
-                end
-            elseif ~is_dynamic && options.solve_algo==3
-                if verbose==1
-                    disp('steady: csolve')
-                end
-                [yn,info] = csolve(@local_fname, y(y_index_eq),@ ...
-                                   local_fname,1e-6,500, x, params, steady_state, y, y_index_eq, fname, 1);
-                dx = ya - yn;
-                y(y_index_eq) = yn;
-            elseif (stack_solve_algo==1 && is_dynamic) || (stack_solve_algo==0 && is_dynamic) || (~is_dynamic && (options.solve_algo==1 || options.solve_algo==6))
-                if verbose==1 && ~is_dynamic
+                [ya,f,r,check]=lnsrch1(ya,f,g,p,stpmax, ...
+                                       'lnsrch1_wrapper_one_boundary',nn, ...
+                                       y_index_eq, options.solve_tolx, M.lead_lag_incidence(M.maximum_endo_lag+1, :), fname, Block_Num, dynvars_from_endo_simul(y, it_, M), x, params, steady_state, T(:, it_), it_);
+                %% Recompute temporary terms, since they are not given as output of lnsrch1
+                [~, ~, T(:, it_)] = feval(fname, Block_Num, dynvars_from_endo_simul(y, it_, M), x, params, steady_state, T(:, it_), it_, false);
+                dx = ya' - y(index_eq, it_);
+                y(index_eq, it_) = ya;
+            elseif (is_dynamic && (stack_solve_algo==1 || stack_solve_algo==0)) || (~is_dynamic && options.solve_algo==6)
+                if verbose && ~is_dynamic
                     disp('steady: Sparse LU ')
                 end
                 dx =  g1\r;
                 ya = ya - lambda*dx;
                 if is_dynamic
-                    y(it_,y_index_eq) = ya';
+                    y(y_index_eq, it_) = ya;
                 else
                     y(y_index_eq) = ya;
                 end
             elseif (stack_solve_algo==2 && is_dynamic) || (options.solve_algo==7 && ~is_dynamic)
                 flag1=1;
-                if verbose == 1 && ~is_dynamic
+                if verbose && ~is_dynamic
                     disp('steady: GMRES ')
                 end
                 while flag1>0
@@ -301,7 +243,7 @@ for it_=start:incr:finish
                     else
                         ya = ya + lambda*dx;
                         if is_dynamic
-                            y(it_,y_index_eq) = ya';
+                            y(y_index_eq, it_) = ya;
                         else
                             y(y_index_eq) = ya';
                         end
@@ -309,22 +251,22 @@ for it_=start:incr:finish
                 end
             elseif (stack_solve_algo==3 && is_dynamic) || (options.solve_algo==8 && ~is_dynamic)
                 flag1=1;
-                if verbose == 1 && ~is_dynamic
+                if verbose && ~is_dynamic
                     disp('steady: BiCGStab')
                 end
                 while flag1>0
                     [L1, U1]=ilu(g1,ilu_setup);
                     phat = ya - U1 \ (L1 \ r);
                     if is_dynamic
-                        y(it_,y_index_eq) = phat;
+                        y(y_index_eq, it_) = phat;
                     else
                         y(y_index_eq) = phat;
                     end
                     if is_dynamic
-                        [r, y, g1, g2, g3] = feval(fname, y, x, params, ...
-                                                   steady_state, it_, 0);
+                        [r, ~, T(:, it_), g1] = feval(fname, Block_Num, dynvars_from_endo_simul(y, it_, M), x, params, ...
+                                                   steady_state, T(:, it_), it_, false);
                     else
-                        [r, y, g1] = feval(fname, y, x, params);
+                        [r, ~, T, g1] = feval(fname, Block_Num, y, x, params, T);
                     end
                     if max(abs(r))>=options.solve_tolf
                         [dx,flag1] = bicgstab(g1,-r,1e-7,Blck_size,L1,U1);
@@ -347,7 +289,7 @@ for it_=start:incr:finish
                     else
                         ya = ya + lambda*dx;
                         if is_dynamic
-                            y(it_,y_index_eq) = ya';
+                            y(y_index_eq, it_) = ya;
                         else
                             y(y_index_eq) = ya';
                         end
@@ -377,11 +319,11 @@ for it_=start:incr:finish
                 fprintf('Error in simul: Convergence not achieved in block %d, at time %d, after %d iterations.\n Increase "options_.simul.maxit" or set "cutoff=0" in model options.\n',Block_Num, it_,iter);
             end
         end
-        if(is_dynamic)
-            oo_.deterministic_simulation.status = 0;
+        if is_dynamic
+            oo_.deterministic_simulation.status = false;
             oo_.deterministic_simulation.error = max_res;
             oo_.deterministic_simulation.iterations = iter;
-            oo_.deterministic_simulation.block(Block_Num).status = 0;% Convergency failed.
+            oo_.deterministic_simulation.block(Block_Num).status = false;% Convergency failed.
             oo_.deterministic_simulation.block(Block_Num).error = max_res;
             oo_.deterministic_simulation.block(Block_Num).iterations = iter;
         end
@@ -392,19 +334,12 @@ end
 
 if is_dynamic
     info = 1;
-    oo_.deterministic_simulation.status = 1;
+    oo_.deterministic_simulation.status = true;
     oo_.deterministic_simulation.error = max_res;
     oo_.deterministic_simulation.iterations = iter;
-    oo_.deterministic_simulation.block(Block_Num).status = 1;
+    oo_.deterministic_simulation.block(Block_Num).status = true;
     oo_.deterministic_simulation.block(Block_Num).error = max_res;
     oo_.deterministic_simulation.block(Block_Num).iterations = iter;
 else
     info = 0;
 end
-
-function [err, G]=local_fname(yl, x, params, steady_state, y, y_index_eq, fname, is_csolve)
-y(y_index_eq) = yl;
-[err, y, G] = feval(fname, y, x, params, steady_state, 0);
-if(is_csolve)
-    G = full(G);
-end
\ No newline at end of file
