@@ -17,7 +17,7 @@ function [hasPackage] = user_has_octave_forge_package(package, min_version)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 [desc,flag] = pkg("describe", package);
 
@@ -26,12 +26,11 @@ if isequal(flag{1,1}, "Not installed")
 else
     if isequal(flag{1,1}, "Not loaded")
         pkg("load", package);
-    endif
+    end
     if nargin > 1 && compare_versions(desc{1,1}.version, min_version, "<")
         hasPackage = 0;
     else
         hasPackage = 1;
-    endif
-endif
-endfunction
-
+    end
+end
+end
\ No newline at end of file
