@@ -9,7 +9,7 @@ function xlag = lagged(x, n)
 % OUTPUT
 % xlag = backward shifted series
 
-% Copyright (C) 2017 Dynare Team
+% Copyright (C) 2017-2021 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -24,11 +24,15 @@ function xlag = lagged(x, n)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 if nargin==1
     n=1;
 end
 
 x=x(:);
-xlag=[NaN(n,1); x(1:end-n)];
\ No newline at end of file
+if n>0
+    xlag=[NaN(n,1); x(1:end-n)];
+else
+    xlag=[x(abs(n)+1:end); NaN(abs(n),1)];
+end
\ No newline at end of file
