@@ -1,5 +1,5 @@
 /*
- * Copyright © 2007-2017 Dynare Team
+ * Copyright © 2007-2021 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -14,20 +14,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#include <cstring>
 #include <sstream>
 #include <algorithm>
-#include "Interpreter.hh"
-#define BIG 1.0e+8;
-#define SMALL 1.0e-5;
-///#define DEBUG
+#include <cstring>
 
-Interpreter::~Interpreter()
-{
-}
+#include "Interpreter.hh"
+
+constexpr double BIG = 1.0e+8, SMALL = 1.0e-5;
 
 Interpreter::Interpreter(double *params_arg, double *y_arg, double *ya_arg, double *x_arg, double *steady_y_arg, double *steady_x_arg,
                          double *direction_arg, size_t y_size_arg,
@@ -35,16 +31,8 @@ Interpreter::Interpreter(double *params_arg, double *y_arg, double *ya_arg, doub
                          int maxit_arg_, double solve_tolf_arg, size_t size_of_direction_arg, double slowc_arg, int y_decal_arg, double markowitz_c_arg,
                          string &filename_arg, int minimal_solving_periods_arg, int stack_solve_algo_arg, int solve_algo_arg,
                          bool global_temporary_terms_arg, bool print_arg, bool print_error_arg, mxArray *GlobalTemporaryTerms_arg,
-                         bool steady_state_arg, bool print_it_arg, int col_x_arg, int col_y_arg
-#ifdef CUDA
-                         , const int CUDA_device_arg, cublasHandle_t cublas_handle_arg, cusparseHandle_t cusparse_handle_arg, cusparseMatDescr_t descr_arg
-#endif
-                         )
-: dynSparseMatrix(y_size_arg, y_kmin_arg, y_kmax_arg, print_it_arg, steady_state_arg, periods_arg, minimal_solving_periods_arg, slowc_arg
-#ifdef CUDA
-                  , CUDA_device_arg, cublas_handle_arg, cusparse_handle_arg, descr_arg
-#endif
-                  )
+                         bool steady_state_arg, bool print_it_arg, int col_x_arg, int col_y_arg)
+: dynSparseMatrix(y_size_arg, y_kmin_arg, y_kmax_arg, print_it_arg, steady_state_arg, periods_arg, minimal_solving_periods_arg, slowc_arg)
 {
   params = params_arg;
   y = y_arg;
@@ -53,12 +41,9 @@ Interpreter::Interpreter(double *params_arg, double *y_arg, double *ya_arg, doub
   steady_y = steady_y_arg;
   steady_x = steady_x_arg;
   direction = direction_arg;
-  //y_size = y_size_arg;
   nb_row_x = nb_row_x_arg;
   nb_row_xd = nb_row_xd_arg;
   periods = periods_arg;
-  //y_kmax = y_kmax_arg;
-  //y_kmin = y_kmin_arg;
   maxit_ = maxit_arg_;
   solve_tolf = solve_tolf_arg;
   size_of_direction = size_of_direction_arg;
@@ -67,7 +52,7 @@ Interpreter::Interpreter(double *params_arg, double *y_arg, double *ya_arg, doub
   y_decal = y_decal_arg;
   markowitz_c = markowitz_c_arg;
   filename = filename_arg;
-  T = NULL;
+  T = nullptr;
   minimal_solving_periods = minimal_solving_periods_arg;
   stack_solve_algo = stack_solve_algo_arg;
   solve_algo = solve_algo_arg;
@@ -77,9 +62,7 @@ Interpreter::Interpreter(double *params_arg, double *y_arg, double *ya_arg, doub
   col_y = col_y_arg;
   GlobalTemporaryTerms = GlobalTemporaryTerms_arg;
   print_error = print_error_arg;
-  //steady_state = steady_state_arg;
   print_it = print_it_arg;
-
 }
 
 void
@@ -87,12 +70,12 @@ Interpreter::evaluate_a_block(bool initialization)
 {
   it_code_type begining;
 
-  switch (type)
+  switch (static_cast<BlockSimulationType>(type))
     {
-    case EVALUATE_FORWARD:
+    case BlockSimulationType::evaluateForward:
       if (steady_state)
         {
-          compute_block_time(0, true, /*block_num, size, steady_state, */ false);
+          compute_block_time(0, true, false);
           if (block >= 0)
             for (int j = 0; j < size; j++)
               residual[j] = y[Block_Contain[j].Variable] - ya[Block_Contain[j].Variable];
@@ -107,7 +90,7 @@ Interpreter::evaluate_a_block(bool initialization)
             {
               it_code = begining;
               Per_y_ = it_*y_size;
-              compute_block_time(0, true, /*block_num, size, steady_state, */ false);
+              compute_block_time(0, true, false);
               if (block >= 0)
                 for (int j = 0; j < size; j++)
                   residual[it_*size+j] = y[it_*y_size+Block_Contain[j].Variable] - ya[it_*y_size+Block_Contain[j].Variable];
@@ -117,14 +100,14 @@ Interpreter::evaluate_a_block(bool initialization)
             }
         }
       break;
-    case SOLVE_FORWARD_SIMPLE:
+    case BlockSimulationType::solveForwardSimple:
       g1 = static_cast<double *>(mxMalloc(size*size*sizeof(double)));
       test_mxMalloc(g1, __LINE__, __FILE__, __func__, size*size*sizeof(double));
       r = static_cast<double *>(mxMalloc(size*sizeof(double)));
       test_mxMalloc(r, __LINE__, __FILE__, __func__, size*sizeof(double));
       if (steady_state)
         {
-          compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+          compute_block_time(0, true, false);
           if (block < 0)
             for (int j = 0; j < size; j++)
               residual[Block_Contain[j].Equation] = r[j];
@@ -139,7 +122,7 @@ Interpreter::evaluate_a_block(bool initialization)
             {
               it_code = begining;
               Per_y_ = it_*y_size;
-              compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+              compute_block_time(0, true, false);
               if (block < 0)
                 for (int j = 0; j < size; j++)
                   residual[Per_y_+Block_Contain[j].Equation] = r[j];
@@ -151,20 +134,20 @@ Interpreter::evaluate_a_block(bool initialization)
       mxFree(g1);
       mxFree(r);
       break;
-    case SOLVE_FORWARD_COMPLETE:
+    case BlockSimulationType::solveForwardComplete:
       if (initialization)
         {
           fixe_u(&u, u_count_int, u_count_int);
           Read_SparseMatrix(bin_base_name, size, 1, 0, 0, false, stack_solve_algo, solve_algo);
         }
 #ifdef DEBUG
-      mexPrintf("in SOLVE_FORWARD_COMPLETE r = mxMalloc(%d*sizeof(double))\n", size);
+      mexPrintf("in SOLVE FORWARD COMPLETE r = mxMalloc(%d*sizeof(double))\n", size);
 #endif
       r = static_cast<double *>(mxMalloc(size*sizeof(double)));
       test_mxMalloc(r, __LINE__, __FILE__, __func__, size*sizeof(double));
       if (steady_state)
         {
-          compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+          compute_block_time(0, true, false);
           if (block < 0)
             for (int j = 0; j < size; j++)
               residual[Block_Contain[j].Equation] = r[j];
@@ -179,7 +162,7 @@ Interpreter::evaluate_a_block(bool initialization)
             {
               it_code = begining;
               Per_y_ = it_*y_size;
-              compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+              compute_block_time(0, true, false);
               if (block < 0)
                 for (int j = 0; j < size; j++)
                   residual[it_*y_size+Block_Contain[j].Equation] = r[j];
@@ -190,10 +173,10 @@ Interpreter::evaluate_a_block(bool initialization)
         }
       mxFree(r);
       break;
-    case EVALUATE_BACKWARD:
+    case BlockSimulationType::evaluateBackward:
       if (steady_state)
         {
-          compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+          compute_block_time(0, true, false);
           if (block >= 0)
             for (int j = 0; j < size; j++)
               residual[j] = y[Block_Contain[j].Variable] - ya[Block_Contain[j].Variable];
@@ -208,7 +191,7 @@ Interpreter::evaluate_a_block(bool initialization)
             {
               it_code = begining;
               Per_y_ = it_*y_size;
-              compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+              compute_block_time(0, true, false);
               if (block >= 0)
                 for (int j = 0; j < size; j++)
                   residual[it_*size+j] = y[it_*y_size+Block_Contain[j].Variable] - ya[it_*y_size+Block_Contain[j].Variable];
@@ -218,14 +201,14 @@ Interpreter::evaluate_a_block(bool initialization)
             }
         }
       break;
-    case SOLVE_BACKWARD_SIMPLE:
+    case BlockSimulationType::solveBackwardSimple:
       g1 = static_cast<double *>(mxMalloc(size*size*sizeof(double)));
       test_mxMalloc(g1, __LINE__, __FILE__, __func__, size*size*sizeof(double));
       r = static_cast<double *>(mxMalloc(size*sizeof(double)));
       test_mxMalloc(r, __LINE__, __FILE__, __func__, size*sizeof(double));
       if (steady_state)
         {
-          compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+          compute_block_time(0, true, false);
           if (block < 0)
             for (int j = 0; j < size; j++)
               residual[Block_Contain[j].Equation] = r[j];
@@ -240,7 +223,7 @@ Interpreter::evaluate_a_block(bool initialization)
             {
               it_code = begining;
               Per_y_ = it_*y_size;
-              compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+              compute_block_time(0, true, false);
               if (block < 0)
                 for (int j = 0; j < size; j++)
                   residual[Per_y_+Block_Contain[j].Equation] = r[j];
@@ -252,7 +235,7 @@ Interpreter::evaluate_a_block(bool initialization)
       mxFree(g1);
       mxFree(r);
       break;
-    case SOLVE_BACKWARD_COMPLETE:
+    case BlockSimulationType::solveBackwardComplete:
       if (initialization)
         {
           fixe_u(&u, u_count_int, u_count_int);
@@ -262,7 +245,7 @@ Interpreter::evaluate_a_block(bool initialization)
       test_mxMalloc(r, __LINE__, __FILE__, __func__, size*sizeof(double));
       if (steady_state)
         {
-          compute_block_time(0, true, /*block_num, size, steady_state,*/ false);
+          compute_block_time(0, true, false);
           if (block < 0)
             for (int j = 0; j < size; j++)
               residual[Block_Contain[j].Equation] = r[j];
@@ -277,7 +260,7 @@ Interpreter::evaluate_a_block(bool initialization)
             {
               it_code = begining;
               Per_y_ = it_*y_size;
-              compute_block_time(0, true, /*block_num, size, steady_state, */ false);
+              compute_block_time(0, true, false);
               if (block < 0)
                 for (int j = 0; j < size; j++)
                   residual[Per_y_+Block_Contain[j].Equation] = r[j];
@@ -288,8 +271,8 @@ Interpreter::evaluate_a_block(bool initialization)
         }
       mxFree(r);
       break;
-    case SOLVE_TWO_BOUNDARIES_SIMPLE:
-    case SOLVE_TWO_BOUNDARIES_COMPLETE:
+    case BlockSimulationType::solveTwoBoundariesSimple:
+    case BlockSimulationType::solveTwoBoundariesComplete:
       if (initialization)
         {
           fixe_u(&u, u_count_int, u_count_int);
@@ -304,7 +287,7 @@ Interpreter::evaluate_a_block(bool initialization)
           Per_u_ = (it_-y_kmin)*u_count_int;
           Per_y_ = it_*y_size;
           it_code = begining;
-          compute_block_time(Per_u_, true, /*block_num, size, steady_state,*/ false);
+          compute_block_time(Per_u_, true, false);
           if (block < 0)
             for (int j = 0; j < size; j++)
               residual[it_*y_size+Block_Contain[j].Equation] = r[j];
@@ -314,11 +297,13 @@ Interpreter::evaluate_a_block(bool initialization)
         }
       mxFree(r);
       break;
+    case BlockSimulationType::unknown:
+      throw FatalExceptionHandling("UNKNOWN block simulation type: impossible case");
     }
 }
 
 int
-Interpreter::simulate_a_block(vector_table_conditional_local_type vector_table_conditional_local)
+Interpreter::simulate_a_block(const vector_table_conditional_local_type &vector_table_conditional_local)
 {
   it_code_type begining;
   max_res = 0;
@@ -329,44 +314,44 @@ Interpreter::simulate_a_block(vector_table_conditional_local_type vector_table_c
   mexPrintf("simulate_a_block type = %d, periods=%d, y_kmin=%d, y_kmax=%d\n", type, periods, y_kmin, y_kmax);
   mexEvalString("drawnow;");
 #endif
-  switch (type)
+  switch (static_cast<BlockSimulationType>(type))
     {
-    case EVALUATE_FORWARD:
+    case BlockSimulationType::evaluateForward:
 #ifdef DEBUG
-      mexPrintf("EVALUATE_FORWARD\n");
+      mexPrintf("EVALUATE FORWARD\n");
       mexEvalString("drawnow;");
 #endif
       evaluate_over_periods(true);
       break;
-    case EVALUATE_BACKWARD:
+    case BlockSimulationType::evaluateBackward:
 #ifdef DEBUG
-      mexPrintf("EVALUATE_BACKWARD\n");
+      mexPrintf("EVALUATE BACKWARD\n");
       mexEvalString("drawnow;");
 #endif
       evaluate_over_periods(false);
       break;
-    case SOLVE_FORWARD_SIMPLE:
+    case BlockSimulationType::solveForwardSimple:
 #ifdef DEBUG
-      mexPrintf("SOLVE_FORWARD_SIMPLE size=%d\n", size);
+      mexPrintf("SOLVE FORWARD SIMPLE size=%d\n", size);
       mexEvalString("drawnow;");
 #endif
       solve_simple_over_periods(true);
       break;
-    case SOLVE_BACKWARD_SIMPLE:
+    case BlockSimulationType::solveBackwardSimple:
 #ifdef DEBUG
-      mexPrintf("SOLVE_BACKWARD_SIMPLE\n");
+      mexPrintf("SOLVE BACKWARD SIMPLE\n");
       mexEvalString("drawnow;");
 #endif
       solve_simple_over_periods(false);
       break;
-    case SOLVE_FORWARD_COMPLETE:
+    case BlockSimulationType::solveForwardComplete:
 #ifdef DEBUG
-      mexPrintf("SOLVE_FORWARD_COMPLETE\n");
+      mexPrintf("SOLVE FORWARD COMPLETE\n");
       mexEvalString("drawnow;");
 #endif
       if (vector_table_conditional_local.size())
         {
-          it_code_type curr_it_code = it_code;
+          auto curr_it_code = it_code;
           evaluate_a_block(true);
           it_code = curr_it_code;
         }
@@ -386,14 +371,14 @@ Interpreter::simulate_a_block(vector_table_conditional_local_type vector_table_c
       memset(direction, 0, size_of_direction);
       End_Solver();
       break;
-    case SOLVE_BACKWARD_COMPLETE:
+    case BlockSimulationType::solveBackwardComplete:
 #ifdef DEBUG
-      mexPrintf("SOLVE_BACKWARD_COMPLETE\n");
+      mexPrintf("SOLVE BACKWARD COMPLETE\n");
       mexEvalString("drawnow;");
 #endif
       if (vector_table_conditional_local.size())
         {
-          it_code_type curr_it_code = it_code;
+          auto curr_it_code = it_code;
           evaluate_a_block(true);
           it_code = curr_it_code;
         }
@@ -413,20 +398,20 @@ Interpreter::simulate_a_block(vector_table_conditional_local_type vector_table_c
       mxFree(u);
       End_Solver();
       break;
-    case SOLVE_TWO_BOUNDARIES_SIMPLE:
-    case SOLVE_TWO_BOUNDARIES_COMPLETE:
+    case BlockSimulationType::solveTwoBoundariesSimple:
+    case BlockSimulationType::solveTwoBoundariesComplete:
 #ifdef DEBUG
-      mexPrintf("SOLVE_TWO_BOUNDARIES\n");
+      mexPrintf("SOLVE TWO BOUNDARIES\n");
       mexEvalString("drawnow;");
 #endif
       if (steady_state)
         {
-          mexPrintf("SOLVE_TWO_BOUNDARIES in a steady state model: impossible case\n");
+          mexPrintf("SOLVE TWO BOUNDARIES in a steady state model: impossible case\n");
           return ERROR_ON_EXIT;
         }
       if (vector_table_conditional_local.size())
         {
-          it_code_type curr_it_code = it_code;
+          auto curr_it_code = it_code;
           evaluate_a_block(true);
           it_code = curr_it_code;
         }
@@ -456,25 +441,17 @@ Interpreter::simulate_a_block(vector_table_conditional_local_type vector_table_c
               res1 = 0;
               max_res = 0;
               max_res_idx = 0;
-              memcpy(y_save, y, y_size*sizeof(double)*(periods+y_kmax+y_kmin));
+              copy_n(y, y_size*(periods+y_kmax+y_kmin), y_save);
               if (vector_table_conditional_local.size())
-                {
-                  for (vector_table_conditional_local_type::iterator it1 = vector_table_conditional_local.begin(); it1 != vector_table_conditional_local.end(); it1++)
-                    {
-                      if (it1->is_cond)
-                        {
-                          //mexPrintf("y[%d] = %f\n", it1->var_endo + y_kmin * size, y[it1->var_endo + y_kmin * size]);
-                          y[it1->var_endo + y_kmin * size] = it1->constrained_value;
-                        }
-
-                    }
-                }
+                for (auto & it1 : vector_table_conditional_local)
+                  if (it1.is_cond)
+                    y[it1.var_endo + y_kmin * size] = it1.constrained_value;
               compute_complete_2b(false, &res1, &res2, &max_res, &max_res_idx);
               end_code = it_code;
               if (!(isnan(res1) || isinf(res1)))
                 cvg = (max_res < solve_tolf);
               if (isnan(res1) || isinf(res1) || (stack_solve_algo == 4 && iter > 0))
-                memcpy(y, y_save, y_size*sizeof(double)*(periods+y_kmax+y_kmin));
+                copy_n(y_save, y_size*(periods+y_kmax+y_kmin), y);
               u_count = u_count_saved;
               int prev_iter = iter;
               Simulate_Newton_Two_Boundaries(block_num, symbol_table_endo_nbr, y_kmin, y_kmax, size, periods, cvg, minimal_solving_periods, stack_solve_algo, endo_name_length, P_endo_names, vector_table_conditional_local);
@@ -488,11 +465,9 @@ Interpreter::simulate_a_block(vector_table_conditional_local_type vector_table_c
                 }
             }
           if (!cvg)
-            {
-              ostringstream tmp;
-              tmp << " in Solve two boundaries, convergence not achieved in block " << block_num+1 << ", after " << iter << " iterations\n";
-              throw FatalExceptionHandling(tmp.str());
-            }
+            throw FatalExceptionHandling(" in Solve two boundaries, convergence not achieved in block "
+                                         + to_string(block_num+1) + ", after "
+                                         + to_string(iter) + " iterations\n");
         }
       else
         {
@@ -524,9 +499,7 @@ Interpreter::simulate_a_block(vector_table_conditional_local_type vector_table_c
       End_Solver();
       break;
     default:
-      ostringstream tmp;
-      tmp << " in simulate_a_block, Unknown type = " << type << "\n";
-      throw FatalExceptionHandling(tmp.str());
+      throw FatalExceptionHandling(" in simulate_a_block, Unknown type = " + to_string(type) + "\n");
       return ERROR_ON_EXIT;
     }
   return NO_ERROR_ON_EXIT;
@@ -549,7 +522,7 @@ Interpreter::print_a_block()
   bool space = false;
   while (go_on)
     {
-      if (it_code->first == FENDBLOCK)
+      if (it_code->first == Tags::FENDBLOCK)
         {
           go_on = false;
           it_code++;
@@ -585,52 +558,46 @@ Interpreter::ReadCodeFile(string file_name, CodeLoad &code)
   code_liste = code.get_op_code(file_name);
   EQN_block_number = code.get_block_number();
   if (!code_liste.size())
-    {
-      ostringstream tmp;
-      tmp << " in compute_blocks, " << file_name << ".cod cannot be opened\n";
-      throw FatalExceptionHandling(tmp.str());
-    }
+    throw FatalExceptionHandling(" in compute_blocks, " + file_name + ".cod cannot be opened\n");
   if (block >= static_cast<int>(code.get_block_number()))
-    {
-      ostringstream tmp;
-      tmp << " in compute_blocks, input argument block = " << block+1 << " is greater than the number of blocks in the model (" << code.get_block_number() << " see M_.block_structure_stat.block)\n";
-      throw FatalExceptionHandling(tmp.str());
-    }
-
+    throw FatalExceptionHandling(" in compute_blocks, input argument block = " + to_string(block+1)
+                                 + " is greater than the number of blocks in the model ("
+                                 + to_string(code.get_block_number()) + " see M_.block_structure_stat.block)\n");
 }
 
 void
-Interpreter::check_for_controlled_exo_validity(FBEGINBLOCK_ *fb, vector<s_plan> sconstrained_extended_path)
+Interpreter::check_for_controlled_exo_validity(FBEGINBLOCK_ *fb, const vector<s_plan> &sconstrained_extended_path)
 {
-  vector<unsigned int> exogenous = fb->get_exogenous();
+  vector<int> exogenous = fb->get_exogenous();
   vector<int> endogenous = fb->get_endogenous();
-  for (vector<s_plan>::iterator it = sconstrained_extended_path.begin(); it != sconstrained_extended_path.end(); it++)
+  for (auto & it : sconstrained_extended_path)
     {
-      if ((find(endogenous.begin(), endogenous.end(), it->exo_num) != endogenous.end()) && (find(exogenous.begin(), exogenous.end(), it->var_num) == exogenous.end()))
-        {
-          ostringstream tmp;
-          tmp << "\n the conditional forecast involving as constrained variable " << get_variable(SymbolType::endogenous, it->exo_num) << " and as endogenized exogenous " << get_variable(SymbolType::exogenous, it->var_num) << " that do not appear in block=" << Block_Count+1 << ")\n You should not use block in model options\n";
-          throw FatalExceptionHandling(tmp.str());
-        }
-      else if ((find(endogenous.begin(), endogenous.end(), it->exo_num) != endogenous.end()) && (find(exogenous.begin(), exogenous.end(), it->var_num) != exogenous.end()) && ((fb->get_type() == EVALUATE_FORWARD) || (fb->get_type() != EVALUATE_BACKWARD)))
-        {
-          ostringstream tmp;
-          tmp << "\n the conditional forecast cannot be implemented for the block=" << Block_Count+1 << ") that has to be evaluated instead to be solved\n You should not use block in model options\n";
-          throw FatalExceptionHandling(tmp.str());
-        }
-      else if (find(previous_block_exogenous.begin(), previous_block_exogenous.end(), it->var_num) != previous_block_exogenous.end())
-        {
-          ostringstream tmp;
-          tmp << "\n the conditional forecast involves in the block " << Block_Count+1 << " the endogenized exogenous " << get_variable(SymbolType::exogenous, it->var_num) << " that appear also in a previous block\n You should not use block in model options\n";
-          throw FatalExceptionHandling(tmp.str());
-        }
+      if (find(endogenous.begin(), endogenous.end(), it.exo_num) != endogenous.end()
+          && find(exogenous.begin(), exogenous.end(), it.var_num) == exogenous.end())
+        throw FatalExceptionHandling("\n the conditional forecast involving as constrained variable "
+                                     + get_variable(SymbolType::endogenous, it.exo_num)
+                                     + " and as endogenized exogenous " + get_variable(SymbolType::exogenous, it.var_num)
+                                     + " that do not appear in block=" + to_string(Block_Count+1)
+                                     + ")\n You should not use block in model options\n");
+      else if (find(endogenous.begin(), endogenous.end(), it.exo_num) != endogenous.end()
+               && find(exogenous.begin(), exogenous.end(), it.var_num) != exogenous.end()
+               && (fb->get_type() == static_cast<uint8_t>(BlockSimulationType::evaluateForward)
+                   || fb->get_type() == static_cast<uint8_t>(BlockSimulationType::evaluateBackward)))
+        throw FatalExceptionHandling("\n the conditional forecast cannot be implemented for the block="
+                                     + to_string(Block_Count+1) + ") that has to be evaluated instead to be solved\n You should not use block in model options\n");
+      else if (find(previous_block_exogenous.begin(), previous_block_exogenous.end(), it.var_num)
+               != previous_block_exogenous.end())
+        throw FatalExceptionHandling("\n the conditional forecast involves in the block "
+                                     + to_string(Block_Count+1) + " the endogenized exogenous "
+                                     + get_variable(SymbolType::exogenous, it.var_num)
+                                     + " that appear also in a previous block\n You should not use block in model options\n");
     }
-  for (vector<unsigned int>::iterator it = exogenous.begin(); it != exogenous.end(); it++)
-    previous_block_exogenous.push_back(*it);
+  for (auto it : exogenous)
+    previous_block_exogenous.push_back(it);
 }
 
 bool
-Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int block, bool last_call, bool constrained, vector<s_plan> sconstrained_extended_path, vector_table_conditional_local_type vector_table_conditional_local)
+Interpreter::MainLoop(const string &bin_basename, const CodeLoad &code, bool evaluate, int block, bool last_call, bool constrained, const vector<s_plan> &sconstrained_extended_path, const vector_table_conditional_local_type &vector_table_conditional_local)
 {
   int var;
   Block_Count = -1;
@@ -650,7 +617,7 @@ Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int blo
     {
       switch (it_code->first)
         {
-        case FBEGINBLOCK:
+        case Tags::FBEGINBLOCK:
           Block_Count++;
 #ifdef DEBUG
           mexPrintf("---------------------------------------------------------\n");
@@ -661,7 +628,7 @@ Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int blo
 #endif
           //it's a new block
           {
-            FBEGINBLOCK_ *fb = static_cast<FBEGINBLOCK_ *>(it_code->second);
+            auto *fb = static_cast<FBEGINBLOCK_ *>(it_code->second);
             Block_Contain = fb->get_Block_Contain();
             it_code++;
             if (constrained)
@@ -698,8 +665,8 @@ Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int blo
             else
               {
 #ifdef DEBUG
-                mexPrintf("endo in Block_Count=%d, block=%d, type=%d, steady_state=%d, print_it=%d, Block_Count=%d, fb->get_is_linear()=%d, fb->get_endo_nbr()=%d, fb->get_Max_Lag()=%d, fb->get_Max_Lead()=%d, fb->get_u_count_int()=%d\n",
-                          Block_Count, fb->get_size(), fb->get_type(), steady_state, print_it, Block_Count, fb->get_is_linear(), fb->get_endo_nbr(), fb->get_Max_Lag(), fb->get_Max_Lead(), fb->get_u_count_int());
+                mexPrintf("endo in Block_Count=%d, size=%d, type=%d, steady_state=%d, print_it=%d, fb->get_is_linear()=%d, fb->get_endo_nbr()=%d, fb->get_Max_Lag()=%d, fb->get_Max_Lead()=%d, fb->get_u_count_int()=%d\n",
+                          Block_Count+1, fb->get_size(), fb->get_type(), steady_state, print_it, fb->get_is_linear(), fb->get_endo_nbr(), fb->get_Max_Lag(), fb->get_Max_Lead(), fb->get_u_count_int());
 #endif
                 bool result;
                 if (sconstrained_extended_path.size())
@@ -737,20 +704,17 @@ Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int blo
               delete fb;
           }
           if (block >= 0)
-            {
-
-              go_on = false;
-            }
+            go_on = false;
 
           break;
-        case FEND:
+        case Tags::FEND:
 #ifdef DEBUG
           mexPrintf("FEND\n");
 #endif
           go_on = false;
           it_code++;
           break;
-        case FDIMT:
+        case Tags::FDIMT:
 #ifdef DEBUG
           mexPrintf("FDIMT size=%d\n", static_cast<FDIMT_ *>(it_code->second)->get_size());
 #endif
@@ -760,13 +724,11 @@ Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int blo
           T = static_cast<double *>(mxMalloc(var*(periods+y_kmin+y_kmax)*sizeof(double)));
           test_mxMalloc(T, __LINE__, __FILE__, __func__, var*(periods+y_kmin+y_kmax)*sizeof(double));
           if (block >= 0)
-            {
-              it_code = code_liste.begin() + code.get_begin_block(block);
-            }
+            it_code = code_liste.begin() + code.get_begin_block(block);
           else
             it_code++;
           break;
-        case FDIMST:
+        case Tags::FDIMST:
 #ifdef DEBUG
           mexPrintf("FDIMST size=%d\n", static_cast<FDIMST_ *>(it_code->second)->get_size());
 #endif
@@ -775,9 +737,10 @@ Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int blo
             mxFree(T);
           if (global_temporary_terms)
             {
-              if (GlobalTemporaryTerms == NULL)
+              if (!GlobalTemporaryTerms)
                 {
-                  mexPrintf("GlobalTemporaryTerms is NULL\n"); mexEvalString("drawnow;");
+                  mexPrintf("GlobalTemporaryTerms is nullptr\n");
+                  mexEvalString("drawnow;");
                 }
               if (var != static_cast<int>(mxGetNumberOfElements(GlobalTemporaryTerms)))
                 GlobalTemporaryTerms = mxCreateDoubleMatrix(var, 1, mxREAL);
@@ -795,9 +758,9 @@ Interpreter::MainLoop(string bin_basename, CodeLoad code, bool evaluate, int blo
             it_code++;
           break;
         default:
-          ostringstream tmp;
-          tmp << " in compute_blocks, unknown command " << it_code->first << " (block=" << Block_Count << ")\n";
-          throw FatalExceptionHandling(tmp.str());
+          throw FatalExceptionHandling(" in compute_blocks, unknown command "
+                                       + to_string(static_cast<int>(it_code->first)) + " (block="
+                                       + to_string(Block_Count) + ")\n");
         }
     }
   max_res = max_res_local;
@@ -847,20 +810,17 @@ Interpreter::elastic(string str, unsigned int len, bool left)
 }
 
 bool
-Interpreter::extended_path(string file_name, string bin_basename, bool evaluate, int block, int &nb_blocks, int nb_periods, vector<s_plan> sextended_path, vector<s_plan> sconstrained_extended_path, vector<string> dates, table_conditional_global_type table_conditional_global)
+Interpreter::extended_path(const string &file_name, const string &bin_basename, bool evaluate, int block, int &nb_blocks, int nb_periods, const vector<s_plan> &sextended_path, const vector<s_plan> &sconstrained_extended_path, const vector<string> &dates, const table_conditional_global_type &table_conditional_global)
 {
   CodeLoad code;
 
   ReadCodeFile(file_name, code);
   it_code = code_liste.begin();
   it_code_type Init_Code = code_liste.begin();
-  /*size_t size_of_direction = y_size*(periods + y_kmax + y_kmin)*sizeof(double);
-    double *y_save = (double *) mxMalloc(size_of_direction);
-    double *x_save = (double *) mxMalloc((periods + y_kmax + y_kmin) * col_x *sizeof(double));*/
   size_t size_of_direction = y_size*col_y*sizeof(double);
-  double *y_save = static_cast<double *>(mxMalloc(size_of_direction));
+  auto *y_save = static_cast<double *>(mxMalloc(size_of_direction));
   test_mxMalloc(y_save, __LINE__, __FILE__, __func__, size_of_direction);
-  double *x_save = static_cast<double *>(mxMalloc(nb_row_x * col_x *sizeof(double)));
+  auto *x_save = static_cast<double *>(mxMalloc(nb_row_x * col_x *sizeof(double)));
   test_mxMalloc(x_save, __LINE__, __FILE__, __func__, nb_row_x * col_x *sizeof(double));
 
   vector_table_conditional_local_type vector_table_conditional_local;
@@ -906,15 +866,13 @@ Interpreter::extended_path(string file_name, string bin_basename, bool evaluate,
           mexPrintf("|%s|", elastic(dates[t], date_length+2, false).c_str());
           mexEvalString("drawnow;");
         }
-      for (vector<s_plan>::const_iterator it = sextended_path.begin(); it != sextended_path.end(); it++)
-        {
-          x[y_kmin + (it->exo_num - 1) * /*(nb_periods + y_kmax + y_kmin)*/ nb_row_x] = it->value[t];
-        }
+      for (const auto & it : sextended_path)
+        x[y_kmin + (it.exo_num - 1) * nb_row_x] = it.value[t];
 
       it_code = Init_Code;
       vector_table_conditional_local.clear();
-      if (table_conditional_global.size())
-        vector_table_conditional_local = table_conditional_global[t];
+      if (auto it = table_conditional_global.find(t); it != table_conditional_global.end())
+        vector_table_conditional_local = it->second;
       if (t < nb_periods)
         MainLoop(bin_basename, code, evaluate, block, false, true, sconstrained_extended_path, vector_table_conditional_local);
       else
@@ -945,11 +903,6 @@ Interpreter::extended_path(string file_name, string bin_basename, bool evaluate,
         }
     }
   print_it = old_print_it;
-  /*for (int j = 0; j < y_size; j++)
-    {
-    for(int k = nb_periods; k < periods; k++)
-    y_save[j + (k + y_kmin) * y_size] = y[ j +  ( k - (nb_periods-1) + y_kmin) * y_size];
-    }*/
   for (int i = 0; i < y_size * col_y; i++)
     y[i] = y_save[i];
   for (int j = 0; j < col_x * nb_row_x; j++)
@@ -967,14 +920,14 @@ Interpreter::extended_path(string file_name, string bin_basename, bool evaluate,
 }
 
 bool
-Interpreter::compute_blocks(string file_name, string bin_basename, bool evaluate, int block, int &nb_blocks)
+Interpreter::compute_blocks(const string &file_name, const string &bin_basename, bool evaluate, int block, int &nb_blocks)
 {
   CodeLoad code;
   ReadCodeFile(file_name, code);
 
   //The big loop on intructions
   it_code = code_liste.begin();
-  it_code_type Init_Code = it_code;
+  auto Init_Code = it_code;
   vector<s_plan> s_plan_junk;
   vector_table_conditional_local_type vector_table_conditional_local_junk;
 
