@@ -1,5 +1,5 @@
 /*
- * Copyright © 2010-2017 Dynare Team
+ * Copyright © 2010-2020 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _MEXMOD
@@ -42,11 +42,5 @@ extern bool utIsInterruptPending();
 void msExit(int status);
 extern int constant_seed;
 
-/* Write Matlab Output
-   mxArray *globalMatlabStruct;*/
-void mex_write_to_matlab_matfile(double *, int, int, const char *, const char *);
-void mex_write_to_matlab_global_struct(double *, int, int, const char *);
-mxArray *getMxArray(double *, int, int);
-
 #endif
 #endif
