@@ -1,5 +1,5 @@
 /*
- * Copyright © 2003-2019 Dynare Team
+ * Copyright © 2003-2021 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -14,18 +14,21 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <cstdlib>
 #include <cassert>
 #include <iostream>
 #include <regex>
-
+#include <algorithm>
+#include <iterator>
 #include <filesystem>
 
 #include "DataTree.hh"
 
+bool DataTree::no_commutativity = false;
+
 void
 DataTree::initConstants()
 {
@@ -99,9 +102,12 @@ DataTree::operator=(const DataTree &d)
   initConstants();
 
   /* Model local variables must be next, because they can be evaluated in Add*
-     methods when the model equations are added */
-  for (const auto &it : d.local_variables_table)
-    local_variables_table[it.first] = it.second->clone(*this);
+     methods when the model equations are added. They need to be cloned in
+     order of appearance in the model block (hence with
+     local_variables_vector), because if there is a model_local_variable statement
+     the symbol IDs ordering may not be the right one (see dynare#1782) */
+  for (const auto &id : d.local_variables_vector)
+    local_variables_table[id] = d.local_variables_table.at(id)->clone(*this);
 
   for (const auto &it : d.node_list)
     it->clone(*this);
@@ -113,7 +119,7 @@ DataTree::operator=(const DataTree &d)
   return *this;
 }
 
-expr_t
+NumConstNode *
 DataTree::AddNonNegativeConstant(const string &value)
 {
   int id = num_constants.AddNonNegativeConstant(value);
@@ -201,7 +207,7 @@ DataTree::AddPlus(expr_t iArg1, expr_t iArg2)
 
   // To treat commutativity of "+"
   // Nodes iArg1 and iArg2 are sorted by index
-  if (iArg1->idx > iArg2->idx)
+  if (iArg1->idx > iArg2->idx && !no_commutativity)
     swap(iArg1, iArg2);
   return AddBinaryOp(iArg1, BinaryOpcode::plus, iArg2);
 }
@@ -280,7 +286,7 @@ DataTree::AddTimes(expr_t iArg1, expr_t iArg2)
 
   // To treat commutativity of "*"
   // Nodes iArg1 and iArg2 are sorted by index
-  if (iArg1->idx > iArg2->idx)
+  if (iArg1->idx > iArg2->idx && !no_commutativity)
     swap(iArg1, iArg2);
   return AddBinaryOp(iArg1, BinaryOpcode::times, iArg2);
 }
@@ -689,10 +695,12 @@ DataTree::AddPacExpectation(const string &model_name)
   return p;
 }
 
-expr_t
+BinaryOpNode *
 DataTree::AddEqual(expr_t iArg1, expr_t iArg2)
 {
-  return AddBinaryOp(iArg1, BinaryOpcode::equal, iArg2);
+  /* We know that we can safely cast to BinaryOpNode because
+     BinaryOpCode::equal can never be reduced to a constant. */
+  return dynamic_cast<BinaryOpNode *>(AddBinaryOp(iArg1, BinaryOpcode::equal, iArg2));
 }
 
 void
@@ -854,46 +862,6 @@ DataTree::isBinaryOpUsedOnType(SymbolType type, BinaryOpcode opcode) const
   return false;
 }
 
-bool
-DataTree::isTrinaryOpUsed(TrinaryOpcode opcode) const
-{
-  for (const auto &it : trinary_op_node_map)
-    if (get<3>(it.first) == opcode)
-      return true;
-
-  return false;
-}
-
-bool
-DataTree::isExternalFunctionUsed(int symb_id) const
-{
-  for (const auto &it : external_function_node_map)
-    if (it.first.second == symb_id)
-      return true;
-
-  return false;
-}
-
-bool
-DataTree::isFirstDerivExternalFunctionUsed(int symb_id) const
-{
-  for (const auto &it : first_deriv_external_function_node_map)
-    if (get<2>(it.first) == symb_id)
-      return true;
-
-  return false;
-}
-
-bool
-DataTree::isSecondDerivExternalFunctionUsed(int symb_id) const
-{
-  for (const auto &it : second_deriv_external_function_node_map)
-    if (get<3>(it.first) == symb_id)
-      return true;
-
-  return false;
-}
-
 int
 DataTree::minLagForSymbol(int symb_id) const
 {
@@ -904,13 +872,6 @@ DataTree::minLagForSymbol(int symb_id) const
   return r;
 }
 
-void
-DataTree::writePowerDerivCHeader(ostream &output) const
-{
-  if (isBinaryOpUsed(BinaryOpcode::powerDeriv))
-    output << "double getPowerDeriv(double, double, int);" << endl;
-}
-
 void
 DataTree::writePowerDeriv(ostream &output) const
 {
@@ -920,10 +881,7 @@ DataTree::writePowerDeriv(ostream &output) const
            << " */" << endl
            << "double getPowerDeriv(double x, double p, int k)" << endl
            << "{" << endl
-           << "#ifdef _MSC_VER" << endl
-           << "# define nearbyint(x) (fabs((x)-floor(x)) < fabs((x)-ceil(x)) ? floor(x) : ceil(x))" << endl
-           << "#endif" << endl
-           << "  if ( fabs(x) < " << near_zero << " && p > 0 && k > p && fabs(p-nearbyint(p)) < " << near_zero << " )" << endl
+           << "  if (fabs(x) < " << near_zero << " && p > 0 && k > p && fabs(p-nearbyint(p)) < " << near_zero << ')' << endl
            << "    return 0.0;" << endl
            << "  else" << endl
            << "    {" << endl
@@ -936,6 +894,33 @@ DataTree::writePowerDeriv(ostream &output) const
            << "}" << endl;
 }
 
+void
+DataTree::writePowerDerivJulia(ostream &output) const
+{
+  if (isBinaryOpUsed(BinaryOpcode::powerDeriv))
+    output << "nearbyint(x::Float64) = (abs((x)-floor(x)) < abs((x)-ceil(x)) ? floor(x) : ceil(x))" << endl
+	   << endl
+	   << "function get_power_deriv(x::Float64, p::Float64, k::Int64)" << endl
+	   << "    if (abs(x) < 1e-12 && p > 0 && k > p && abs(p-nearbyint(p)) < 1e-12 )" << endl
+	   << "        return 0.0" << endl
+	   << "    else" << endl
+	   << "        dxp = x^(p-k)" << endl
+	   << "        for i = 1:k" << endl
+	   << "	     dxp *= p" << endl
+	   << "	     p -= 1" << endl
+	   << "	 end" << endl
+	   << "	 return dxp" << endl
+	   << "    end" << endl
+	   << "end" << endl;
+}
+
+void
+DataTree::writePowerDerivHeader(ostream &output) const
+{
+  if (isBinaryOpUsed(BinaryOpcode::powerDeriv))
+    output << "double getPowerDeriv(double x, double p, int k);" << endl;
+}
+
 string
 DataTree::packageDir(const string &package)
 {
@@ -944,3 +929,26 @@ DataTree::packageDir(const string &package)
   filesystem::create_directories(dirname);
   return dirname;
 }
+
+void
+DataTree::writeToFileIfModified(stringstream &new_contents, const string &filename)
+{
+  ifstream old_file{filename, ios::in | ios::binary};
+  if (old_file.is_open()
+      && equal(istreambuf_iterator<char>{old_file}, istreambuf_iterator<char>{},
+               istreambuf_iterator<char>{new_contents}, istreambuf_iterator<char>{}))
+    return;
+  old_file.close();
+
+  new_contents.seekg(0);
+
+  ofstream new_file{filename, ios::out | ios::binary};
+  if (!new_file.is_open())
+    {
+      cerr << "Error: Can't open file " << filename << " for writing" << endl;
+      exit(EXIT_FAILURE);
+    }
+  copy(istreambuf_iterator<char>{new_contents}, istreambuf_iterator<char>{},
+       ostreambuf_iterator<char>{new_file});
+  new_file.close();
+}
