@@ -1,5 +1,5 @@
 /*
- * Copyright © 2006-2020 Dynare Team
+ * Copyright © 2006-2021 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <cstdlib>
@@ -45,8 +45,6 @@ ModFile::ModFile(WarningConsolidation &warnings_arg)
                                        trend_component_model_table, var_model_table},
     orig_ramsey_dynamic_model{symbol_table, num_constants, external_functions_table,
                               trend_component_model_table, var_model_table},
-    non_linear_equations_dynamic_model{symbol_table, num_constants, external_functions_table,
-                                       trend_component_model_table, var_model_table},
     epilogue{symbol_table, num_constants, external_functions_table,
              trend_component_model_table, var_model_table},
     static_model{symbol_table, num_constants, external_functions_table},
@@ -138,6 +136,7 @@ ModFile::checkPass(bool nostrict, bool stochastic)
     || mod_file_struct.discretionary_policy_present
     || mod_file_struct.calib_smoother_present
     || mod_file_struct.identification_present
+    || mod_file_struct.mom_estimation_present
     || mod_file_struct.sensitivity_present
     || stochastic;
 
@@ -181,32 +180,15 @@ ModFile::checkPass(bool nostrict, bool stochastic)
       exit(EXIT_FAILURE);
     }
 
-  if (mod_file_struct.k_order_solver && byte_code)
+  if (mod_file_struct.k_order_solver && bytecode)
     {
       cerr << "ERROR: 'k_order_solver' (which is implicit if order >= 3), is not yet compatible with 'bytecode'." << endl;
       exit(EXIT_FAILURE);
     }
 
-  if (use_dll && (block || byte_code || linear_decomposition))
+  if (use_dll && bytecode)
     {
-      cerr << "ERROR: In 'model' block, 'use_dll' option is not compatible with 'block', 'bytecode' or 'linear_decomposition'" << endl;
-      exit(EXIT_FAILURE);
-    }
-  if (block && linear_decomposition)
-    {
-      cerr << "ERROR: In 'model' block, 'block' option is not compatible with 'linear_decomposition'" << endl;
-      exit(EXIT_FAILURE);
-    }
-
-  if (!byte_code && linear_decomposition)
-    {
-      cerr << "ERROR: For the moment in 'model' block, 'linear_decomposition' option is compatible only with 'bytecode' option" << endl;
-      exit(EXIT_FAILURE);
-    }
-
-  if ((block || byte_code) && dynamic_model.isModelLocalVariableUsed())
-    {
-      cerr << "ERROR: In 'model' block, 'block' or 'bytecode' options are not yet compatible with pound expressions" << endl;
+      cerr << "ERROR: In 'model' block, 'use_dll' option is not compatible with 'bytecode'" << endl;
       exit(EXIT_FAILURE);
     }
 
@@ -376,6 +358,13 @@ ModFile::checkPass(bool nostrict, bool stochastic)
           exit(EXIT_FAILURE);
         }
     }
+
+  // See dynare#1726
+  if ((stochastic_statement_present || mod_file_struct.check_present) && dynamic_model.mfs > 0)
+    {
+      cerr << "ERROR: mfs > 0 is incompatible with check, stoch_simul, estimation, osr, ramsey_policy, discretionary_policy, calib_smoother, identification, methods_of_moments and sensitivity commands" << endl;
+      exit(EXIT_FAILURE);
+    }
 }
 
 void
@@ -401,6 +390,9 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
   original_model = dynamic_model;
   dynamic_model.expandEqTags();
 
+  // Replace all model-local variables by their expression
+  dynamic_model.substituteModelLocalVariables();
+
   // Check that all declared endogenous are used in equations
   set<int> unusedEndogs = dynamic_model.findUnusedEndogenous();
   bool unusedEndogsIsErr = !nostrict && !mod_file_struct.bvar_present && unusedEndogs.size();
@@ -450,10 +442,10 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
   // Create auxiliary variable and equations for Diff operators
   auto [diff_nodes, diff_subst_table] = dynamic_model.substituteDiff(pac_growth);
 
-  // Fill Trend Component Model Table
+  // Fill trend component and VAR model tables
   dynamic_model.fillTrendComponentModelTable();
   original_model.fillTrendComponentModelTableFromOrigModel();
-  dynamic_model.fillTrendComponentmodelTableAREC(diff_subst_table);
+  dynamic_model.fillTrendComponentModelTableAREC(diff_subst_table);
   dynamic_model.fillVarModelTable();
   original_model.fillVarModelTableFromOrigModel();
 
@@ -492,18 +484,20 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
             cerr << "Error: aux_model_name not recognized as VAR model or Trend Component model" << endl;
             exit(EXIT_FAILURE);
           }
+        if (pms->growth)
+          dynamic_model.createPacGrowthNeutralityParameter(pms->name);
+
         auto eqtag_and_lag = dynamic_model.walkPacParameters(pms->name);
-        original_model.getPacMaxLag(pms->name, eqtag_and_lag);
         if (pms->aux_model_name.empty())
           dynamic_model.addPacModelConsistentExpectationEquation(pms->name, symbol_table.getID(pms->discount),
-                                                                 eqtag_and_lag, diff_subst_table);
+                                                                 eqtag_and_lag, diff_subst_table,
+                                                                 pms->growth);
         else
           dynamic_model.fillPacModelInfo(pms->name, lhs, max_lag, aux_model_type,
                                          eqtag_and_lag, nonstationary, pms->growth);
         dynamic_model.substitutePacExpectation(pms->name);
       }
-
-  dynamic_model.addEquationsForVar();
+  dynamic_model.checkNoRemainingPacExpectation();
 
   // Create auxiliary vars for Expectation operator
   dynamic_model.substituteExpectation(mod_file_struct.partial_information);
@@ -588,7 +582,16 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
 
       /* Create auxiliary parameters and the expression to be substituted into
          the var_expectations statement */
-      auto subst_expr = dynamic_model.Zero;
+      expr_t subst_expr = dynamic_model.Zero;
+      if (var_model_table.isExistingVarModelName(vems->aux_model_name))
+        {
+          /* If the auxiliary model is a VAR, add a parameter corresponding to
+             the constant. */
+          string constant_param_name = "var_expectation_model_" + model_name + "_constant";
+          int constant_param_id = symbol_table.addSymbol(constant_param_name, SymbolType::parameter);
+          vems->aux_params_ids.push_back(constant_param_id);
+          subst_expr = dynamic_model.AddPlus(subst_expr, dynamic_model.AddVariable(constant_param_id));
+        }
       for (int lag = 0; lag < max_lag; lag++)
         for (auto variable : lhs)
           {
@@ -598,7 +601,7 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
 
             subst_expr = dynamic_model.AddPlus(subst_expr,
                                                dynamic_model.AddTimes(dynamic_model.AddVariable(new_param_id),
-                                                                      dynamic_model.AddVariable(variable, -lag)));
+                                                                      dynamic_model.AddVariable(variable, -lag + vems->time_shift)));
           }
 
       if (var_expectation_subst_table.find(model_name) != var_expectation_subst_table.end())
@@ -610,30 +613,28 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
     }
   // And finally perform the substitutions
   dynamic_model.substituteVarExpectation(var_expectation_subst_table);
-  dynamic_model.createVariableMapping(original_model.equation_number());
+  dynamic_model.createVariableMapping(mod_file_struct.orig_eq_nbr +
+                                      (mod_file_struct.ramsey_model_present ?
+                                       mod_file_struct.ramsey_eq_nbr : 0));
 
-  if (mod_file_struct.stoch_simul_present
-      || mod_file_struct.estimation_present
-      || mod_file_struct.osr_present
-      || mod_file_struct.ramsey_policy_present
-      || mod_file_struct.discretionary_policy_present
-      || mod_file_struct.calib_smoother_present
-      || mod_file_struct.identification_present
-      || mod_file_struct.sensitivity_present
-      || stochastic)
-    {
-      // In stochastic models, create auxiliary vars for leads and lags greater than 2, on both endos and exos
-      dynamic_model.substituteEndoLeadGreaterThanTwo(false);
-      dynamic_model.substituteExoLead(false);
-      dynamic_model.substituteEndoLagGreaterThanTwo(false);
-      dynamic_model.substituteExoLag(false);
-    }
-  else
-    {
-      // In deterministic models, create auxiliary vars for leads and lags endogenous greater than 2, only on endos (useless on exos)
-      dynamic_model.substituteEndoLeadGreaterThanTwo(true);
-      dynamic_model.substituteEndoLagGreaterThanTwo(true);
-    }
+  /* Create auxiliary vars for leads and lags greater than 2, on both endos and
+     exos. The transformation is not exactly the same on stochastic and
+     deterministic models, because there is no need to take into account the
+     Jensen inequality on the latter. */
+  bool deterministic_model = !(mod_file_struct.stoch_simul_present
+                               || mod_file_struct.estimation_present
+                               || mod_file_struct.osr_present
+                               || mod_file_struct.ramsey_policy_present
+                               || mod_file_struct.discretionary_policy_present
+                               || mod_file_struct.calib_smoother_present
+                               || mod_file_struct.identification_present
+                               || mod_file_struct.mom_estimation_present
+                               || mod_file_struct.sensitivity_present
+                               || stochastic);
+  dynamic_model.substituteEndoLeadGreaterThanTwo(deterministic_model);
+  dynamic_model.substituteExoLead(deterministic_model);
+  dynamic_model.substituteEndoLagGreaterThanTwo(deterministic_model);
+  dynamic_model.substituteExoLag(deterministic_model);
 
   dynamic_model.updateVarAndTrendModel();
 
@@ -672,7 +673,6 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
   */
   if (!(mod_file_struct.ramsey_model_present || mod_file_struct.discretionary_policy_present)
       && !(mod_file_struct.bvar_present && dynamic_model.equation_number() == 0)
-      && !(mod_file_struct.occbin_option)
       && (dynamic_model.equation_number() != symbol_table.endo_nbr()))
     {
       cerr << "ERROR: There are " << dynamic_model.equation_number() << " equations but " << symbol_table.endo_nbr() << " endogenous variables!" << endl;
@@ -697,6 +697,22 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
       exit(EXIT_FAILURE);
     }
 
+  if (mod_file_struct.occbin_constraints_present
+      && (mod_file_struct.osr_present || mod_file_struct.mom_estimation_present
+          || mod_file_struct.ramsey_model_present || mod_file_struct.ramsey_policy_present
+          || mod_file_struct.discretionary_policy_present || mod_file_struct.extended_path_present
+          || mod_file_struct.identification_present || mod_file_struct.sensitivity_present))
+    {
+      cerr << "ERROR: the 'occbin_constraints' block is not compatible with commands other than 'estimation', 'stoch_simul', and 'calib_smoother'." << endl;
+      exit(EXIT_FAILURE);
+    }
+
+  if (mod_file_struct.shocks_surprise_present && !mod_file_struct.occbin_constraints_present)
+    {
+      cerr << "ERROR: the 'shocks(surprise)' block can only be used in conjunction with the 'occbin_constraints' block." << endl;
+      exit(EXIT_FAILURE);
+    }
+
   if (!mod_file_struct.ramsey_model_present)
     cout << "Found " << dynamic_model.equation_number() << " equation(s)." << endl;
   else
@@ -725,7 +741,7 @@ ModFile::transformPass(bool nostrict, bool stochastic, bool compute_xrefs, bool
 }
 
 void
-ModFile::computingPass(bool no_tmp_terms, FileOutputType output, int params_derivs_order)
+ModFile::computingPass(bool no_tmp_terms, OutputType output, int params_derivs_order)
 {
   // Mod file may have no equation (for example in a standalone BVAR estimation)
   if (dynamic_model.equation_number() > 0)
@@ -735,66 +751,76 @@ ModFile::computingPass(bool no_tmp_terms, FileOutputType output, int params_deri
 
       // Compute static model and its derivatives
       static_model = static_cast<StaticModel>(dynamic_model);
-      if (linear_decomposition)
-        {
-          non_linear_equations_dynamic_model = dynamic_model;
-          non_linear_equations_dynamic_model.set_cutoff_to_zero();
-          non_linear_equations_dynamic_model.computingPass(true, 1, 0, global_eval_context, no_tmp_terms, block, use_dll, byte_code, linear_decomposition);
-        }
       if (!no_static)
         {
           if (mod_file_struct.stoch_simul_present
               || mod_file_struct.estimation_present || mod_file_struct.osr_present
               || mod_file_struct.ramsey_model_present || mod_file_struct.identification_present
-              || mod_file_struct.calib_smoother_present)
+              || mod_file_struct.calib_smoother_present || mod_file_struct.mom_estimation_present)
             static_model.set_cutoff_to_zero();
 
           int derivsOrder = 1;
           int paramsDerivsOrder = 0;
           if (mod_file_struct.identification_present || mod_file_struct.estimation_analytic_derivation)
-            {
-              derivsOrder = 2;
-              paramsDerivsOrder = params_derivs_order;
-            }
-          static_model.computingPass(derivsOrder, paramsDerivsOrder, global_eval_context, no_tmp_terms, block, byte_code);
+            derivsOrder = 2;
+
+          if (mod_file_struct.identification_present 
+              || mod_file_struct.estimation_analytic_derivation
+              || (mod_file_struct.GMM_present && (mod_file_struct.analytic_standard_errors_present || mod_file_struct.analytic_jacobian_present)))
+            paramsDerivsOrder = params_derivs_order;
+
+          static_model.computingPass(derivsOrder, paramsDerivsOrder, global_eval_context, no_tmp_terms, block, bytecode);
         }
       // Set things to compute for dynamic model
       if (mod_file_struct.perfect_foresight_solver_present || mod_file_struct.check_present
           || mod_file_struct.stoch_simul_present
           || mod_file_struct.estimation_present || mod_file_struct.osr_present
           || mod_file_struct.ramsey_model_present || mod_file_struct.identification_present
-          || mod_file_struct.calib_smoother_present)
+          || mod_file_struct.calib_smoother_present || mod_file_struct.mom_estimation_present)
         {
           if (mod_file_struct.perfect_foresight_solver_present)
-            dynamic_model.computingPass(true, 1, 0, global_eval_context, no_tmp_terms, block, use_dll, byte_code, linear_decomposition);
+            {
+              int derivsOrder = 1;
+              if (output == OutputType::second)
+                derivsOrder = 2;
+              else if  (output == OutputType::third)
+                derivsOrder = 3;
+              dynamic_model.computingPass(true, derivsOrder, 0, global_eval_context, no_tmp_terms, block, use_dll, bytecode);
+            }
           else
             {
               if (mod_file_struct.stoch_simul_present
                   || mod_file_struct.estimation_present || mod_file_struct.osr_present
                   || mod_file_struct.ramsey_model_present || mod_file_struct.identification_present
-                  || mod_file_struct.calib_smoother_present)
+                  || mod_file_struct.calib_smoother_present || mod_file_struct.mom_estimation_present)
                 dynamic_model.set_cutoff_to_zero();
               if (mod_file_struct.order_option < 1)
                 {
                   cerr << "ERROR: Incorrect order option..." << endl;
                   exit(EXIT_FAILURE);
                 }
-              int derivsOrder = max(mod_file_struct.order_option,
-                                    mod_file_struct.identification_order + 1); // See preprocessor#40
-              if (mod_file_struct.sensitivity_present || linear || output == FileOutputType::second)
+              int derivsOrder = max(mod_file_struct.order_option,mod_file_struct.identification_order + 1); // See preprocessor#40
+              if (mod_file_struct.GMM_present 
+                  && (mod_file_struct.analytic_standard_errors_present || mod_file_struct.analytic_jacobian_present)) //analytic_standard_errors or analytic_jacobian require one order more
+                derivsOrder = max(mod_file_struct.order_option,
+                                    max(mod_file_struct.identification_order,mod_file_struct.mom_order) + 1); // See preprocessor#40
+
+              if (mod_file_struct.sensitivity_present || linear || output == OutputType::second)
                 derivsOrder = max(derivsOrder, 2);
-              if (mod_file_struct.estimation_analytic_derivation || output == FileOutputType::third)
+              if (mod_file_struct.estimation_analytic_derivation || output == OutputType::third)
                 derivsOrder = max(derivsOrder, 3);
               int paramsDerivsOrder = 0;
-              if (mod_file_struct.identification_present || mod_file_struct.estimation_analytic_derivation)
+              if (mod_file_struct.identification_present 
+                  || mod_file_struct.estimation_analytic_derivation
+                  || (mod_file_struct.GMM_present && (mod_file_struct.analytic_standard_errors_present || mod_file_struct.analytic_jacobian_present)))
                 paramsDerivsOrder = params_derivs_order;
-              dynamic_model.computingPass(true, derivsOrder, paramsDerivsOrder, global_eval_context, no_tmp_terms, block, use_dll, byte_code, linear_decomposition);
+              dynamic_model.computingPass(true, derivsOrder, paramsDerivsOrder, global_eval_context, no_tmp_terms, block, use_dll, bytecode);
               if (linear && mod_file_struct.ramsey_model_present)
-                orig_ramsey_dynamic_model.computingPass(true, 2, paramsDerivsOrder, global_eval_context, no_tmp_terms, block, use_dll, byte_code, linear_decomposition);
+                orig_ramsey_dynamic_model.computingPass(true, 2, paramsDerivsOrder, global_eval_context, no_tmp_terms, block, use_dll, bytecode);
             }
         }
       else // No computing task requested, compute derivatives up to 2nd order by default
-        dynamic_model.computingPass(true, 2, 0, global_eval_context, no_tmp_terms, block, use_dll, byte_code, linear_decomposition);
+        dynamic_model.computingPass(true, 2, 0, global_eval_context, no_tmp_terms, block, use_dll, bytecode);
 
       /* Check that the model is linear.
          FIXME: this check always passes if derivsOrder = 1, i.e. for a perfect
@@ -824,23 +850,26 @@ ModFile::computingPass(bool no_tmp_terms, FileOutputType output, int params_deri
         }
     }
 
+  // Those matrices can only be filled here, because we use derivatives
+  dynamic_model.fillVarModelTableMatrices();
+
   for (auto &statement : statements)
-    statement->computingPass();
+    statement->computingPass(mod_file_struct);
 
   // Compute epilogue derivatives (but silence standard output)
   streambuf *oldcout = cout.rdbuf();
   cout.rdbuf(nullptr);
-  epilogue.computingPass(true, 2, 0, global_eval_context, true, false, false, false, false);
+  epilogue.computingPass(true, 2, 0, global_eval_context, true, false, false, false);
   cout.rdbuf(oldcout);
 }
 
 void
-ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_global, bool no_log, bool no_warn,
-                          bool console, bool nograph, bool nointeractive, const ConfigFile &config_file,
-                          bool check_model_changes, bool minimal_workspace, bool compute_xrefs,
-                          const string &mexext,
-                          const filesystem::path &matlabroot,
-                          const filesystem::path &dynareroot, bool onlymodel, bool gui) const
+ModFile::writeMOutput(const string &basename, bool clear_all, bool clear_global, bool no_warn,
+                      bool console, bool nograph, bool nointeractive, const ConfigFile &config_file,
+                      bool check_model_changes, bool minimal_workspace, bool compute_xrefs,
+                      const string &mexext,
+                      const filesystem::path &matlabroot,
+                      const filesystem::path &dynareroot, bool onlymodel, bool gui, bool notime) const
 {
   bool hasModelChanged = !dynamic_model.isChecksumMatching(basename, block) || !check_model_changes;
   if (hasModelChanged)
@@ -909,8 +938,10 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
   else if (clear_global)
     mOutputFile << "clear M_ options_ oo_ estim_params_ bayestopt_ dataset_ dataset_info estimation_info ys0_ ex0_;" << endl;
 
-  mOutputFile << "tic0 = tic;" << endl
-              << "% Define global variables." << endl
+  if (!notime)
+    mOutputFile << "tic0 = tic;" << endl;
+
+  mOutputFile << "% Define global variables." << endl
               << "global M_ options_ oo_ estim_params_ bayestopt_ dataset_ dataset_info estimation_info ys0_ ex0_" << endl
               << "options_ = [];" << endl
               << "M_.fname = '" << basename << "';" << endl
@@ -922,10 +953,7 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
               << "%" << endl;
   if (!onlymodel)
     config_file.writeHooks(mOutputFile);
-  mOutputFile << "global_initialization;" << endl
-              << "diary off;" << endl;
-  if (!no_log)
-    mOutputFile << "diary('" << basename << ".log');" << endl;
+  mOutputFile << "global_initialization;" << endl;
 
   if (minimal_workspace)
     mOutputFile << "options_.minimal_workspace = true;" << endl;
@@ -946,6 +974,9 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
 
   symbol_table.writeOutput(mOutputFile);
 
+  // Fill the fields related to solve_algo={12,14} if possible.
+  mOutputFile << "M_ = setup_solvers(M_);" << endl;
+
   var_model_table.writeOutput(basename, mOutputFile);
   trend_component_model_table.writeOutput(basename, mOutputFile);
 
@@ -967,16 +998,18 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
   // May be later modified by a shocks block
   mOutputFile << "M_.sigma_e_is_diagonal = true;" << endl;
 
-  // Initialize M_.det_shocks
-  mOutputFile << "M_.det_shocks = [];" << endl;
+  // Initialize M_.det_shocks, M_.surprise_shocks and M_.heteroskedastic_shocks
+  mOutputFile << "M_.det_shocks = [];" << endl
+              << "M_.surprise_shocks = [];" << endl
+              << "M_.heteroskedastic_shocks.Qvalue_orig = [];" << endl
+              << "M_.heteroskedastic_shocks.Qscale_orig = [];" << endl;
 
   auto to_matlab_logical = [](bool m) { return m ? "true" : "false"; };
 
   mOutputFile << "options_.linear = " << to_matlab_logical(linear) << ";" << endl
               << "options_.block = " << to_matlab_logical(block) << ";" << endl
-              << "options_.bytecode = " << to_matlab_logical(byte_code) << ";" << endl
-              << "options_.use_dll = " << to_matlab_logical(use_dll) << ";" << endl
-              << "options_.linear_decomposition = " << to_matlab_logical(linear_decomposition) << ";" << endl;
+              << "options_.bytecode = " << to_matlab_logical(bytecode) << ";" << endl
+              << "options_.use_dll = " << to_matlab_logical(use_dll) << ";" << endl;
 
   if (parallel_local_files.size() > 0)
     {
@@ -1004,7 +1037,7 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
   if (!onlymodel)
     config_file.writeCluster(mOutputFile);
 
-  if (byte_code)
+  if (bytecode)
     mOutputFile << "if exist('bytecode') ~= 3" << endl
                 << "  error('DYNARE: Can''t find bytecode DLL. Please compile it or remove the ''bytecode'' option.')" << endl
                 << "end" << endl;
@@ -1018,15 +1051,13 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
 
   if (dynamic_model.equation_number() > 0)
     {
-      if (linear_decomposition)
-        non_linear_equations_dynamic_model.writeOutput(mOutputFile, basename, block, true, byte_code, use_dll, mod_file_struct.estimation_present, compute_xrefs, false);
-      dynamic_model.writeOutput(mOutputFile, basename, block, false, byte_code, use_dll, mod_file_struct.estimation_present, compute_xrefs, false);
+      dynamic_model.writeDriverOutput(mOutputFile, basename, block, use_dll, mod_file_struct.occbin_constraints_present, mod_file_struct.estimation_present, compute_xrefs);
       if (!no_static)
-        static_model.writeOutput(mOutputFile, block);
+        static_model.writeDriverOutput(mOutputFile, block);
     }
 
   if (onlymodel || gui)
-    for (auto &statement : statements)
+    for (const auto &statement : statements)
       {
         /* Special treatment for initval block: insert initial values for the
            auxiliary variables and initialize exo det */
@@ -1062,7 +1093,7 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
       }
   else
     {
-      for (auto &statement : statements)
+      for (const auto &statement : statements)
         {
           statement->writeOutput(mOutputFile, basename, minimal_workspace);
 
@@ -1083,25 +1114,30 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
             static_model.writeAuxVarInitval(mOutputFile, ExprNodeOutputType::matlabOutsideModel);
         }
 
-      mOutputFile << "save('" << basename << "_results.mat', 'oo_', 'M_', 'options_');" << endl
+      if (!notime)
+          mOutputFile << endl << endl
+                      << "oo_.time = toc(tic0);" << endl
+                      << "disp(['Total computing time : ' dynsec2hms(oo_.time) ]);" << endl;
+
+      mOutputFile << "if ~exist([M_.dname filesep 'Output'],'dir')" << endl
+                  << "    mkdir(M_.dname,'Output');" << endl
+                  << "end" << endl
+                  << "save([M_.dname filesep 'Output' filesep '" << basename << "_results.mat'], 'oo_', 'M_', 'options_');" << endl
                   << "if exist('estim_params_', 'var') == 1" << endl
-                  << "  save('" << basename << "_results.mat', 'estim_params_', '-append');" << endl << "end" << endl
+                  << "  save([M_.dname filesep 'Output' filesep '" << basename << "_results.mat'], 'estim_params_', '-append');" << endl << "end" << endl
                   << "if exist('bayestopt_', 'var') == 1" << endl
-                  << "  save('" << basename << "_results.mat', 'bayestopt_', '-append');" << endl << "end" << endl
+                  << "  save([M_.dname filesep 'Output' filesep '" << basename << "_results.mat'], 'bayestopt_', '-append');" << endl << "end" << endl
                   << "if exist('dataset_', 'var') == 1" << endl
-                  << "  save('" << basename << "_results.mat', 'dataset_', '-append');" << endl << "end" << endl
+                  << "  save([M_.dname filesep 'Output' filesep '" << basename << "_results.mat'], 'dataset_', '-append');" << endl << "end" << endl
                   << "if exist('estimation_info', 'var') == 1" << endl
-                  << "  save('" << basename << "_results.mat', 'estimation_info', '-append');" << endl << "end" << endl
+                  << "  save([M_.dname filesep 'Output' filesep '" << basename << "_results.mat'], 'estimation_info', '-append');" << endl << "end" << endl
                   << "if exist('dataset_info', 'var') == 1" << endl
-                  << "  save('" << basename << "_results.mat', 'dataset_info', '-append');" << endl << "end" << endl
+                  << "  save([M_.dname filesep 'Output' filesep '" << basename << "_results.mat'], 'dataset_info', '-append');" << endl << "end" << endl
                   << "if exist('oo_recursive_', 'var') == 1" << endl
-                  << "  save('" << basename << "_results.mat', 'oo_recursive_', '-append');" << endl << "end" << endl;
+                  << "  save([M_.dname filesep 'Output' filesep '" << basename << "_results.mat'], 'oo_recursive_', '-append');" << endl << "end" << endl;
 
       config_file.writeEndParallel(mOutputFile);
 
-      mOutputFile << endl << endl
-                  << "disp(['Total computing time : ' dynsec2hms(toc(tic0)) ]);" << endl;
-
       if (!no_warn)
         {
           if (warnings.countWarnings() > 0)
@@ -1113,9 +1149,6 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
         }
     }
 
-  if (!no_log)
-    mOutputFile << "diary off" << endl;
-
   mOutputFile.close();
 
   if (hasModelChanged)
@@ -1125,17 +1158,11 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
         {
           if (!no_static)
             {
-              static_model.writeStaticFile(basename, block, byte_code, use_dll, mexext, matlabroot, dynareroot, false);
+              static_model.writeStaticFile(basename, block, bytecode, use_dll, mexext, matlabroot, dynareroot, false);
               static_model.writeParamsDerivativesFile(basename, false);
             }
 
-          if (linear_decomposition)
-            {
-              non_linear_equations_dynamic_model.writeDynamicFile(basename, block, linear_decomposition, byte_code, use_dll, mexext, matlabroot, dynareroot, false);
-              non_linear_equations_dynamic_model.writeParamsDerivativesFile(basename, false);
-            }
-
-          dynamic_model.writeDynamicFile(basename, block, false, byte_code, use_dll, mexext, matlabroot, dynareroot, false);
+          dynamic_model.writeDynamicFile(basename, block, bytecode, use_dll, mexext, matlabroot, dynareroot, false);
 
           dynamic_model.writeParamsDerivativesFile(basename, false);
 
@@ -1153,142 +1180,19 @@ ModFile::writeOutputFiles(const string &basename, bool clear_all, bool clear_glo
 }
 
 void
-ModFile::writeExternalFiles(const string &basename, LanguageOutputType language) const
+ModFile::writeJuliaOutput(const string &basename) const
 {
-  switch (language)
-    {
-    case LanguageOutputType::julia:
-      writeExternalFilesJulia(basename);
-      break;
-    case LanguageOutputType::matlab:
-      cerr << "The 'output' option cannot be used when language=matlab" << endl;
-      exit(EXIT_FAILURE);
-    }
-}
-
-void
-ModFile::writeExternalFilesJulia(const string &basename) const
-{
-  ofstream jlOutputFile;
-  if (basename.size())
-    {
-      string fname(basename);
-      fname += ".jl";
-      jlOutputFile.open(fname, ios::out | ios::binary);
-      if (!jlOutputFile.is_open())
-        {
-          cerr << "ERROR: Can't open file " << fname
-               << " for writing" << endl;
-          exit(EXIT_FAILURE);
-        }
-    }
-  else
-    {
-      cerr << "ERROR: Missing file name" << endl;
-      exit(EXIT_FAILURE);
-    }
-
-  jlOutputFile << "module " << basename << endl
-               << "#" << endl
-               << "# NB: this file was automatically generated by Dynare" << endl
-               << "#     from " << basename << ".mod" << endl
-               << "#" << endl << endl
-               << "using DynareModel" << endl
-               << "using DynareOptions" << endl
-               << "using DynareOutput" << endl << endl
-               << "using Utils" << endl
-               << "using SteadyState" << endl << endl
-               << "using " << basename << "Static" << endl
-               << "using " << basename << "Dynamic" << endl
-               << R"(if isfile(")" << basename << R"(SteadyState.jl"))" << endl
-               << "    using " << basename << "SteadyState" << endl
-               << "end" << endl
-               << R"(if isfile(")" << basename << R"(SteadyState2.jl"))" << endl
-               << "    using " << basename << "SteadyState2" << endl
-               << "end" << endl << endl
-               << "export model_, options_, oo_" << endl;
-
-  // Write Output
-  jlOutputFile << endl
-               << "oo_ = dynare_output()" << endl
-               << R"(oo_.dynare_version = ")" << PACKAGE_VERSION << R"(")" << endl;
-
-  // Write Options
-  jlOutputFile << endl
-               << "options_ = dynare_options()" << endl
-               << R"(options_.dynare_version = ")" << PACKAGE_VERSION << R"(")" << endl;
-  if (linear)
-    jlOutputFile << "options_.linear = true" << endl;
-
-  // Write Model
-  jlOutputFile << endl
-               << "model_ = dynare_model()" << endl
-               << R"(model_.fname = ")" << basename << R"(")" << endl
-               << R"(model_.dynare_version = ")" << PACKAGE_VERSION << R"(")" << endl
-               << "model_.sigma_e = zeros(Float64, " << symbol_table.exo_nbr() << ", "
-               << symbol_table.exo_nbr() << ")" << endl
-               << "model_.correlation_matrix = ones(Float64, " << symbol_table.exo_nbr() << ", "
-               << symbol_table.exo_nbr() << ")" << endl
-               << "model_.orig_eq_nbr = " << mod_file_struct.orig_eq_nbr << endl
-               << "model_.eq_nbr = " << dynamic_model.equation_number() << endl
-               << "model_.ramsey_eq_nbr = " << mod_file_struct.ramsey_eq_nbr << endl;
-
-  if (mod_file_struct.calibrated_measurement_errors)
-    jlOutputFile << "model_.h = zeros(Float64,"
-                 << symbol_table.observedVariablesNbr() << ", "
-                 << symbol_table.observedVariablesNbr() << ");" << endl
-                 << "model_.correlation_matrix_me = ones(Float64, "
-                 << symbol_table.observedVariablesNbr() << ", "
-                 << symbol_table.observedVariablesNbr() << ");" << endl;
-  else
-    jlOutputFile << "model_.h = zeros(Float64, 1, 1)" << endl
-                 << "model_.correlation_matrix_me = ones(Float64, 1, 1)" << endl;
-
-  cout << "Processing outputs ..." << endl;
-  symbol_table.writeJuliaOutput(jlOutputFile);
-
   if (dynamic_model.equation_number() > 0)
     {
-      dynamic_model.writeOutput(jlOutputFile, basename, false, false, false, false,
-                                mod_file_struct.estimation_present, false, true);
       if (!no_static)
         {
           static_model.writeStaticFile(basename, false, false, false, "", {}, {}, true);
           static_model.writeParamsDerivativesFile(basename, true);
         }
-      dynamic_model.writeDynamicFile(basename, block, linear_decomposition, byte_code, use_dll,
-                                     "", {}, {}, true);
+      dynamic_model.writeDynamicFile(basename, block, bytecode, use_dll, "", {}, {}, true);
       dynamic_model.writeParamsDerivativesFile(basename, true);
     }
   steady_state_model.writeSteadyStateFile(basename, mod_file_struct.ramsey_model_present, true);
-
-  // Print statements (includes parameter values)
-  for (auto &statement : statements)
-    statement->writeJuliaOutput(jlOutputFile, basename);
-
-  jlOutputFile << "model_.static = " << basename << "Static.static!" << endl
-               << "model_.dynamic = " << basename << "Dynamic.dynamic!" << endl
-               << "model_.temporaries.static = " << basename << "Static.tmp_nbr" << endl
-               << "model_.temporaries.dynamic = " << basename << "Dynamic.tmp_nbr" << endl
-               << R"(if isfile(")" << basename << R"(SteadyState.jl"))" << endl
-               << "    model_.user_written_analytical_steady_state = true" << endl
-               << "    model_.steady_state = " << basename << "SteadyState.steady_state!" << endl
-               << "end" << endl
-               << R"(if isfile(")" << basename << R"(SteadyState2.jl"))" << endl
-               << "    model_.analytical_steady_state = true" << endl
-               << "    model_.steady_state = " << basename << "SteadyState2.steady_state!" << endl
-               << "end" << endl
-               << R"(if isfile(")" << basename << R"(StaticParamsDerivs.jl"))" << endl
-               << "    using " << basename << "StaticParamsDerivs" << endl
-               << "    model_.static_params_derivs = " << basename << "StaticParamsDerivs.params_derivs" << endl
-               << "end" << endl
-               << R"(if isfile(")" << basename << R"(DynamicParamsDerivs.jl"))" << endl
-               << "    using " << basename << "DynamicParamsDerivs" << endl
-               << "    model_.dynamic_params_derivs = " << basename << "DynamicParamsDerivs.params_derivs" << endl
-               << "end" << endl
-               << "end" << endl;
-  jlOutputFile.close();
-  cout << "done" << endl;
 }
 
 void
