@@ -1,5 +1,5 @@
 /*
- * Copyright © 2010-2019 Dynare Team
+ * Copyright © 2010-2021 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -14,11 +14,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <cassert>
 #include <algorithm>
+#include <sstream>
 
 #include "ModelEquationBlock.hh"
 
@@ -190,17 +191,9 @@ SteadyStateModel::writeSteadyStateFile(const string &basename, bool ramsey_model
   if (def_table.size() == 0)
     return;
 
-  string filename = julia ? basename + "SteadyState2.jl" : packageDir(basename) + "/steadystate.m";
-  ofstream output;
-  output.open(filename, ios::out | ios::binary);
-  if (!output.is_open())
-    {
-      cerr << "ERROR: Can't open file " << filename << " for writing" << endl;
-      exit(EXIT_FAILURE);
-    }
-
   ExprNodeOutputType output_type = (julia ? ExprNodeOutputType::juliaSteadyStateFile : ExprNodeOutputType::steadyStateFile);
 
+  stringstream output;
   if (!julia)
     output << "function [ys_, params, info] = steadystate("
            << "ys_, exo_, params)" << endl
@@ -243,7 +236,8 @@ SteadyStateModel::writeSteadyStateFile(const string &basename, bool ramsey_model
   output << "end" << endl;
   if (julia)
     output << "end" << endl;
-  output.close();
+
+  writeToFileIfModified(output, julia ? basename + "SteadyState2.jl" : packageDir(basename) + "/steadystate.m");
 }
 
 void
@@ -274,7 +268,7 @@ SteadyStateModel::writeJsonSteadyStateFile(ostream &output, bool transformComput
         }
       if (symb_ids.size() > 1)
         output << "]";
-      output << ", \"rhs\":\"";
+      output << R"(, "rhs":")";
       def_table[i].second->writeJsonOutput(output, {}, {}, false);
       output << "\"}" << endl;
     }
