@@ -1,5 +1,5 @@
 /*
- * Copyright © 2003-2019 Dynare Team
+ * Copyright © 2003-2021 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+ * along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <algorithm>
@@ -130,10 +130,9 @@ SymbolTable::freeze() noexcept(false)
           param_ids.push_back(i);
           break;
         default:
-          tsi = -1;
-          break;
+          continue;
         }
-      type_specific_ids.push_back(tsi);
+      type_specific_ids[i] = tsi;
     }
 }
 
@@ -361,7 +360,6 @@ SymbolTable::writeOutput(ostream &output) const noexcept(false)
             break;
           case AuxVarType::endoLag:
           case AuxVarType::exoLag:
-          case AuxVarType::varModel:
             output << "M_.aux_vars(" << i+1 << ").orig_index = " << getTypeSpecificID(aux_vars[i].get_orig_symb_id())+1 << ";" << endl
                    << "M_.aux_vars(" << i+1 << ").orig_lead_lag = " << aux_vars[i].get_orig_lead_lag() << ";" << endl;
             break;
@@ -628,28 +626,6 @@ SymbolTable::addUnaryOpAuxiliaryVar(int index, expr_t expr_arg, string unary_op,
   return symb_id;
 }
 
-int
-SymbolTable::addVarModelEndoLagAuxiliaryVar(int orig_symb_id, int orig_lead_lag, expr_t expr_arg) noexcept(false)
-{
-  int symb_id;
-  ostringstream varname;
-  varname << "AUX_VARMODEL_" << orig_symb_id << "_" << -orig_lead_lag;
-
-  try
-    {
-      symb_id = addSymbol(varname.str(), SymbolType::endogenous);
-    }
-  catch (AlreadyDeclaredException &e)
-    {
-      cerr << "ERROR: you should rename your variable called " << varname.str() << ", this name is internally used by Dynare" << endl;
-      exit(EXIT_FAILURE);
-    }
-
-  aux_vars.emplace_back(symb_id, AuxVarType::varModel, orig_symb_id, orig_lead_lag, 0, 0, expr_arg, "");
-
-  return symb_id;
-}
-
 int
 SymbolTable::addMultiplierAuxiliaryVar(int index) noexcept(false)
 {
@@ -719,12 +695,11 @@ SymbolTable::getOrigSymbIdForAuxVar(int aux_var_symb_id) const noexcept(false)
 int
 SymbolTable::getOrigLeadLagForDiffAuxVar(int diff_aux_var_symb_id) const noexcept(false)
 {
-  int lag = 0;
   for (const auto &aux_var : aux_vars)
     if ((aux_var.get_type() == AuxVarType::diffLag || aux_var.get_type() == AuxVarType::diffLead)
         && aux_var.get_symb_id() == diff_aux_var_symb_id)
-      lag += 1 + getOrigLeadLagForDiffAuxVar(aux_var.get_orig_symb_id());
-  return lag;
+      return (aux_var.get_type() == AuxVarType::diffLag ? 1 : -1) + getOrigLeadLagForDiffAuxVar(aux_var.get_orig_symb_id());
+  return 0;
 }
 
 int
@@ -917,138 +892,6 @@ SymbolTable::getOrigEndogenous() const
   return origendogs;
 }
 
-void
-SymbolTable::writeJuliaOutput(ostream &output) const noexcept(false)
-{
-  if (!frozen)
-    throw NotYetFrozenException();
-
-  output << "# Endogenous Variables" << endl
-         << "model_.endo = [" << endl;
-  if (endo_nbr() > 0)
-    for (int id = 0; id < endo_nbr(); id++)
-      output << R"(              DynareModel.Endo(")"
-             << getName(endo_ids[id]) << R"(", raw")"
-             << getTeXName(endo_ids[id]) << R"(", ")"
-             << getLongName(endo_ids[id]) << R"("))" << endl;
-  output << "             ]" << endl;
-  output << "model_.endo_nbr = " << endo_nbr() << ";" << endl;
-
-  output << "# Exogenous Variables" << endl
-         << "model_.exo = [" << endl;
-  if (exo_nbr() > 0)
-    for (int id = 0; id < exo_nbr(); id++)
-      output << R"(             DynareModel.Exo(")"
-             << getName(exo_ids[id]) << R"(", raw")"
-             << getTeXName(exo_ids[id]) << R"(", ")"
-             << getLongName(exo_ids[id]) << R"("))" << endl;
-  output << "            ]" << endl;
-  output << "model_.exo_nbr = " << exo_nbr() << ";" << endl;
-
-  if (exo_det_nbr() > 0)
-    {
-      output << "# Exogenous Deterministic Variables" << endl
-             << "model_.exo_det = [" << endl;
-      if (exo_det_nbr() > 0)
-        for (int id = 0; id < exo_det_nbr(); id++)
-          output << R"(                 DynareModel.ExoDet(")"
-                 << getName(exo_det_ids[id]) << R"(", raw")"
-                 << getTeXName(exo_det_ids[id]) << R"(", ")"
-                 << getLongName(exo_det_ids[id]) << R"("))" << endl;
-      output << "                ]" << endl;
-      output << "model_.exo_det_nbr = " << exo_det_nbr() << ";" << endl;
-    }
-
-  output << "# Parameters" << endl
-         << "model_.param = [" << endl;
-  if (param_nbr() > 0)
-    for (int id = 0; id < param_nbr(); id++)
-      output << R"(               DynareModel.Param(")"
-             << getName(param_ids[id]) << R"(", raw")"
-             << getTeXName(param_ids[id]) << R"(", ")"
-             << getLongName(param_ids[id]) << R"("))" << endl;
-  output << "              ]" << endl;
-  output << "model_.param_nbr = " << param_nbr() << ";" << endl;
-
-  output << "model_.orig_endo_nbr = " << orig_endo_nbr() << endl;
-
-  if (aux_vars.size() > 0)
-    {
-      output << "# Auxiliary Variables" << endl
-             << "model_.aux_vars = [" << endl;
-      for (const auto &aux_var : aux_vars)
-        {
-          output << "                   DynareModel.AuxVars("
-                 << getTypeSpecificID(aux_var.get_symb_id()) + 1 << ", "
-                 << aux_var.get_type_id() << ", ";
-          switch (aux_var.get_type())
-            {
-            case AuxVarType::endoLead:
-            case AuxVarType::exoLead:
-            case AuxVarType::endoLag:
-            case AuxVarType::exoLag:
-            case AuxVarType::varModel:
-              output << getTypeSpecificID(aux_var.get_orig_symb_id()) + 1 << ", "
-                     << aux_var.get_orig_lead_lag() << ", typemin(Int), string(), string()";
-              break;
-            case AuxVarType::unaryOp:
-              if (aux_var.get_orig_symb_id() >= 0)
-                output << getTypeSpecificID(aux_var.get_orig_symb_id()) + 1 << ", " << aux_var.get_orig_lead_lag();
-              else
-                output << "typemin(Int), typemin(Int)";
-              output << ", typemin(Int), string(), "
-                     << R"(")" << aux_var.get_unary_op() << R"(")" << endl;
-              break;
-            case AuxVarType::diff:
-            case AuxVarType::diffLag:
-            case AuxVarType::diffLead:
-              if (aux_var.get_orig_symb_id() >= 0)
-                output << getTypeSpecificID(aux_var.get_orig_symb_id()) + 1 << ", "
-                       << aux_var.get_orig_lead_lag() << ", typemin(Int), string(), string()";
-              break;
-            case AuxVarType::multiplier:
-              output << "typemin(Int), typemin(Int), " << aux_var.get_equation_number_for_multiplier() + 1
-                     << ", string(), string()";
-              break;
-            case AuxVarType::diffForward:
-              output << getTypeSpecificID(aux_var.get_orig_symb_id())+1 << ", typemin(Int), typemin(Int), string(), string()";
-              break;
-            case AuxVarType::expectation:
-              output << R"(typemin(Int), typemin(Int), typemin(Int), "\mathbb{E}_{t)"
-                     << (aux_var.get_information_set() < 0 ? "" : "+")
-                     << aux_var.get_information_set() << "}(";
-              aux_var.get_expr_node()->writeOutput(output, ExprNodeOutputType::latexDynamicModel);
-              output << R"lit()")lit";
-              break;
-            default:
-              output << " typemin(Int), typemin(Int), typemin(Int), string(), string()";
-            }
-          output << ")" << endl;
-        }
-      output << "]" << endl;
-    }
-
-  if (predeterminedNbr() > 0)
-    {
-      output << "# Predetermined Variables" << endl
-             << "model_.pred_vars = [ " << endl;
-      for (int predetermined_variable : predetermined_variables)
-        output << "                   DynareModel.PredVars("
-               << getTypeSpecificID(predetermined_variable)+1 << ")" << endl;
-      output << "                  ]" << endl;
-    }
-
-  if (observedVariablesNbr() > 0)
-    {
-      output << "# Observed Variables" << endl
-             << "options_.obs_vars = [" << endl;
-      for (int varob : varobs)
-        output << "                    DynareModel.ObsVars("
-               << getTypeSpecificID(varob)+1 << ")" << endl;
-      output << "                   ]" << endl;
-    }
-}
-
 void
 SymbolTable::writeJsonOutput(ostream &output) const
 {
@@ -1063,6 +906,48 @@ SymbolTable::writeJsonOutput(ostream &output) const
 
   output << R"(, "parameters": )";
   writeJsonVarVector(output, param_ids);
+
+  if (observedVariablesNbr() > 0)
+    {
+      output << R"(, "varobs": [)";
+      for (size_t i = 0; i < varobs.size(); i++)
+	{
+	  if (i != 0)
+	    output << ", ";
+	  output << R"(")" << getName(varobs[i]) << R"(")";
+	}
+      output << "]" << endl;
+      
+      output << R"(, "varobs_ids": [)";
+      for (size_t i = 0; i < varobs.size(); i++)
+	{
+	  if (i != 0)
+	    output << ", ";
+	  output << getTypeSpecificID(varobs[i])+1;
+	}
+      output << "]" << endl;
+    }
+
+  if (observedExogenousVariablesNbr() > 0)
+    {
+      output << R"(, "varexobs": [)";
+      for (size_t i = 0; i < varexobs.size(); i++)
+	{
+	  if (i != 0)
+	    output << ", ";
+	  output << R"(")" << getName(varexobs[i]) << R"(")";
+	}
+      output << "]" << endl;
+      
+      output << R"(, "varexobs_ids": [)";
+      for (size_t i = 0; i < varexobs.size(); i++)
+	{
+	  if (i != 0)
+	    output << ", ";
+	  output << getTypeSpecificID(varexobs[i])+1;
+	}
+      output << "]" << endl;
+    }
 }
 
 void
