@@ -1,4 +1,4 @@
-function run_ls2003(block, bytecode, solve_algo, stack_solve_algo)
+function run_ls2003(block, storage, solve_algo, stack_solve_algo)
 
 % Copyright (C) 2010-2013 Dynare Team
 %
@@ -15,16 +15,18 @@ function run_ls2003(block, bytecode, solve_algo, stack_solve_algo)
 % GNU General Public License for more details.
 %
 % You should have received a copy of the GNU General Public License
-% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
   disp(['TEST: ls2003 (block=' num2str(block) ', bytecode=' ...
-      num2str(bytecode) ', solve_algo=' num2str(solve_algo) ...
-      ', stack_solve_algo=' num2str(stack_solve_algo) ')...']);
+        num2str(storage==2) ', use_dll=' num2str(storage==1) ...
+        ', solve_algo=' num2str(solve_algo) ...
+        ', stack_solve_algo=' num2str(stack_solve_algo) ')...']);
   fid = fopen('ls2003_tmp.mod', 'w');
   assert(fid > 0);
   fprintf(fid, ['@#define block = %d\n@#define bytecode = %d\n' ...
+      '@#define use_dll = %d\n' ...
       '@#define solve_algo = %d\n@#define stack_solve_algo = %d\n' ...
-      '@#include \"ls2003.mod\"\n'], block, bytecode, ...
+      '@#include \"ls2003.mod\"\n'], block, storage==2, storage==1, ...
       solve_algo, stack_solve_algo);
   fclose(fid);
   dynare('ls2003_tmp.mod','console')
