@@ -2,10 +2,10 @@
 
 # Produces Windows packages of Dynare (executable installer, 7z and zip archives).
 #
-# The binaries are cross compiled for Windows (32/64bits), Octave and MATLAB
+# The binaries are cross compiled for Windows (64-bit), Octave and MATLAB
 # (all supported versions).
 
-# Copyright © 2017-2020 Dynare Team
+# Copyright © 2017-2021 Dynare Team
 #
 # This file is part of Dynare.
 #
@@ -20,7 +20,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
+# along with Dynare.  If not, see <https://www.gnu.org/licenses/>.
 
 # Exit on first error and print commands as they are executed
 set -ex
@@ -51,9 +51,7 @@ fi
 BASENAME=dynare-$VERSION
 
 # Set directories for dependencies
-LIB32="$ROOT_DIRECTORY"/deps/lib32
 LIB64="$ROOT_DIRECTORY"/deps/lib64
-LIB32_MSYS2="$ROOT_DIRECTORY"/deps/lib32-msys2
 LIB64_MSYS2="$ROOT_DIRECTORY"/deps/lib64-msys2
 
 # Go to source root directory
@@ -78,58 +76,16 @@ if [[ -z $CI ]]; then
     make -j"$NTHREADS" pdf html
 fi
 make -j"$NTHREADS"
+x86_64-w64-mingw32-strip preprocessor/src/dynare-preprocessor.exe
 x86_64-w64-mingw32-strip matlab/preprocessor64/dynare_m.exe
 x86_64-w64-mingw32-strip dynare++/src/dynare++.exe
-mkdir -p dynare++/64-bit/
-cp dynare++/src/dynare++.exe dynare++/64-bit/
-
-## Compile 32-bit preprocessor and Dynare++
-# We do not want to clean the doc
-for d in preprocessor/src dynare++/integ dynare++/kord dynare++/parser/cc dynare++/src dynare++/sylv dynare++/tl dynare++/utils/cc; do
-    make -C "$d" clean
-done
-./configure --host=i686-w64-mingw32 \
-	    --with-boost="$LIB32_MSYS2" \
-	    --with-blas="$LIB32_MSYS2"/lib/libopenblas.a \
-	    --with-lapack="$LIB32_MSYS2"/lib/libopenblas.a \
-	    --with-matio="$LIB32_MSYS2" \
-	    --disable-octave \
-	    --disable-matlab \
-	    PACKAGE_VERSION="$VERSION" \
-	    PACKAGE_STRING="dynare $VERSION"
-make -j"$NTHREADS"
-i686-w64-mingw32-strip matlab/preprocessor32/dynare_m.exe
-i686-w64-mingw32-strip dynare++/src/dynare++.exe
-mkdir -p dynare++/32-bit/
-cp dynare++/src/dynare++.exe dynare++/32-bit/
 
 ## Define functions for building MEX files
 
 ## Note that we do out-of-tree compilation, since we want to do these in
 ## parallel
 
-# Create Windows 32-bit DLL binaries for MATLAB ≥ R2009b
-build_windows_matlab_mex_32 ()
-{
-    mkdir -p "$TMP_DIRECTORY"/matlab-win32/
-    cd "$TMP_DIRECTORY"/matlab-win32/
-    "$ROOT_DIRECTORY"/../mex/build/matlab/configure \
-                     --host=i686-w64-mingw32 \
-		     --with-gsl="$LIB32_MSYS2" \
-		     --with-matio="$LIB32_MSYS2" \
-		     --with-slicot="$LIB32"/Slicot/without-underscore \
-		     --with-matlab="$ROOT_DIRECTORY"/deps/matlab32/R2009b \
-		     MATLAB_VERSION=R2009b \
-		     MEXEXT=mexw32 \
-		     PACKAGE_VERSION="$VERSION" \
-		     PACKAGE_STRING="dynare $VERSION"
-    make -j"$NTHREADS" all
-    i686-w64-mingw32-strip -- **/*.mexw32
-    mkdir -p "$ROOT_DIRECTORY"/../mex/matlab/win32-7.9-8.6
-    mv -- **/*.mexw32 "$ROOT_DIRECTORY"/../mex/matlab/win32-7.9-8.6
-}
-
-# Create Windows 64-bit DLL binaries for MATLAB ≥ R2009b and ≤ R2017b
+# Create Windows 64-bit DLL binaries for MATLAB ≥ R2014a and ≤ R2017b
 build_windows_matlab_mex_64_a ()
 {
     mkdir -p "$TMP_DIRECTORY"/matlab-win64-a/
@@ -139,15 +95,14 @@ build_windows_matlab_mex_64_a ()
 		     --with-gsl="$LIB64_MSYS2" \
 		     --with-matio="$LIB64_MSYS2" \
 		     --with-slicot="$LIB64"/Slicot/without-underscore \
-		     --with-matlab="$ROOT_DIRECTORY"/deps/matlab64/R2009b \
-		     MATLAB_VERSION=R2009b \
+		     --with-matlab="$ROOT_DIRECTORY"/deps/matlab64/R2014a \
 		     MEXEXT=mexw64 \
 		     PACKAGE_VERSION="$VERSION" \
 		     PACKAGE_STRING="dynare $VERSION"
     make -j"$NTHREADS" all
     x86_64-w64-mingw32-strip -- **/*.mexw64
-    mkdir -p "$ROOT_DIRECTORY"/../mex/matlab/win64-7.9-9.3
-    mv -- **/*.mexw64 "$ROOT_DIRECTORY"/../mex/matlab/win64-7.9-9.3
+    mkdir -p "$ROOT_DIRECTORY"/../mex/matlab/win64-8.3-9.3
+    mv -- **/*.mexw64 "$ROOT_DIRECTORY"/../mex/matlab/win64-8.3-9.3
 }
 
 # Create Windows 64-bit DLL binaries for MATLAB ≥ R2018a
@@ -161,33 +116,13 @@ build_windows_matlab_mex_64_b ()
 		     --with-matio="$LIB64_MSYS2" \
 		     --with-slicot="$LIB64"/Slicot/without-underscore \
 		     --with-matlab="$ROOT_DIRECTORY"/deps/matlab64/R2018a \
-		     MATLAB_VERSION=R2018a \
 		     MEXEXT=mexw64 \
 		     PACKAGE_VERSION="$VERSION" \
 		     PACKAGE_STRING="dynare $VERSION"
     make -j"$NTHREADS" all
     x86_64-w64-mingw32-strip -- **/*.mexw64
-    mkdir -p "$ROOT_DIRECTORY"/../mex/matlab/win64-9.4-9.9
-    mv -- **/*.mexw64 "$ROOT_DIRECTORY"/../mex/matlab/win64-9.4-9.9
-}
-
-# Create Windows DLL binaries for Octave/MinGW (32bit)
-build_windows_octave_mex_32 ()
-{
-    mkdir -p "$TMP_DIRECTORY"/octave-32/
-    cd "$TMP_DIRECTORY"/octave-32/
-    "$ROOT_DIRECTORY"/../mex/build/octave/configure \
-                     --host=i686-w64-mingw32 \
-                     --with-gsl="$LIB32_MSYS2" \
-                     --with-matio="$LIB32_MSYS2" \
-                     --with-slicot="$LIB32"/Slicot/with-underscore \
-                     MKOCTFILE="$ROOT_DIRECTORY"/deps/mkoctfile32 \
-                     PACKAGE_VERSION="$VERSION" \
-                     PACKAGE_STRING="dynare $VERSION"
-    make -j"$NTHREADS" all
-    i686-w64-mingw32-strip -- **/*.mex
-    mkdir -p "$ROOT_DIRECTORY"/../mex/octave/win32
-    mv -- **/*.mex "$ROOT_DIRECTORY"/../mex/octave/win32
+    mkdir -p "$ROOT_DIRECTORY"/../mex/matlab/win64-9.4-9.11
+    mv -- **/*.mexw64 "$ROOT_DIRECTORY"/../mex/matlab/win64-9.4-9.11
 }
 
 # Create Windows DLL binaries for Octave/MinGW (64bit)
@@ -201,6 +136,7 @@ build_windows_octave_mex_64 ()
                      --with-matio="$LIB64_MSYS2" \
                      --with-slicot="$LIB64"/Slicot/with-underscore \
                      MKOCTFILE="$ROOT_DIRECTORY"/deps/mkoctfile64 \
+                     OCTAVE=/bin/true \
                      PACKAGE_VERSION="$VERSION" \
                      PACKAGE_STRING="dynare $VERSION"
     make -j"$NTHREADS" all
@@ -211,15 +147,15 @@ build_windows_octave_mex_64 ()
 
 ## Actually build the MEX files
 
-TASKS=(build_windows_matlab_mex_32 build_windows_matlab_mex_64_a build_windows_matlab_mex_64_b build_windows_octave_mex_32 build_windows_octave_mex_64)
-# Reset the number of threads. The mex files for MATLAB/Octave (32-bit and 64-bit) will be built
+TASKS=(build_windows_matlab_mex_64_a build_windows_matlab_mex_64_b build_windows_octave_mex_64)
+# Reset the number of threads. The mex files for MATLAB/Octave will be built
 # in parallel, so we need to account for the number of tasks and lower the value of NTHREADS.
 NTHREADS=$((NTHREADS/${#TASKS[@]}))
 [[ $NTHREADS -ge 1 ]] || NTHREADS=1 # Ensure that there is at least 1 thread
 # Build all the mex files (parallel).
 # Some variables and functions need to be available in subshells.
 cd "$ROOT_DIRECTORY"
-export TMP_DIRECTORY ROOT_DIRECTORY LIB32 LIB32_MSYS2 LIB64 LIB64_MSYS2 VERSION NTHREADS
+export TMP_DIRECTORY ROOT_DIRECTORY LIB64 LIB64_MSYS2 VERSION NTHREADS
 export -f "${TASKS[@]}"
 parallel "set -ex;shopt -s globstar;" ::: "${TASKS[@]}"
 
@@ -252,7 +188,6 @@ cp -p NEWS.md "$ZIPDIR"
 cp -p VERSION "$ZIPDIR"
 cp -p license.txt "$ZIPDIR"
 cp -p windows/README.txt "$ZIPDIR"
-cp -pr windows/deps/mingw32 "$ZIPDIR"
 cp -pr windows/deps/mingw64 "$ZIPDIR"
 mkdir -p "$ZIPDIR"/contrib/ms-sbvar/TZcode
 cp -pr contrib/ms-sbvar/TZcode/MatlabFiles "$ZIPDIR"/contrib/ms-sbvar/TZcode
@@ -261,16 +196,16 @@ cp -pr contrib/jsonlab/* "$ZIPDIR"/contrib/jsonlab
 mkdir "$ZIPDIR"/mex
 cp -pr mex/octave/ "$ZIPDIR"/mex
 cp -pr mex/matlab/ "$ZIPDIR"/mex
+mkdir "$ZIPDIR"/preprocessor
+cp -p preprocessor/src/dynare-preprocessor.exe "$ZIPDIR"/preprocessor
 cp -pr matlab "$ZIPDIR"
-mkdir -p "$ZIPDIR"/matlab/modules/dseries/externals/x13/windows/32
-cp -p windows/deps/lib32/x13as/x13as.exe "$ZIPDIR"/matlab/modules/dseries/externals/x13/windows/32
 mkdir -p "$ZIPDIR"/matlab/modules/dseries/externals/x13/windows/64
 cp -p windows/deps/lib64/x13as/x13as.exe "$ZIPDIR"/matlab/modules/dseries/externals/x13/windows/64
 cp -pr examples "$ZIPDIR"
 mkdir -p "$ZIPDIR"/scripts
 cp -p scripts/dynare.el "$ZIPDIR"/scripts
 mkdir "$ZIPDIR"/dynare++
-cp -pr dynare++/32-bit/ dynare++/64-bit/ dynare++/dynare_simul/dynare_simul.m "$ZIPDIR"/dynare++
+cp -pr dynare++/src/dynare++.exe dynare++/dynare_simul/dynare_simul.m "$ZIPDIR"/dynare++
 mkdir -p "$ZIPDIR"/doc/dynare++
 mkdir -p "$ZIPDIR"/doc/dynare-manual.html
 cp -pr doc/manual/build/html/* "$ZIPDIR"/doc/dynare-manual.html
